@@ -1,3 +1,4 @@
+static const char CVSID[] = "$Id: highlight.c,v 1.18.2.1 2001/09/14 15:59:43 edg Exp $";
 /*******************************************************************************
 *									       *
 * highlight.c -- Nirvana Editor syntax highlighting (text coloring and font    *
@@ -12,7 +13,7 @@
 * 									       *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
-* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
 * for more details.							       *
 * 									       *
 * You should have received a copy of the GNU General Public License along with *
@@ -27,16 +28,23 @@
 *******************************************************************************/
 #include <stdio.h>
 #include <limits.h>
+#include <math.h>
+#include <stdlib.h>
+#include <string.h>
 #ifdef VMS
 #include "../util/VMSparam.h"
 #else
+#ifndef __MVS__
 #include <sys/param.h>
+#endif
 #endif /*VMS*/
+
 #include <Xm/Xm.h>
 #include <Xm/XmP.h>
 #if XmVersion >= 1002
 #include <Xm/PrimitiveP.h>
 #endif
+
 #include "../util/misc.h"
 #include "../util/DialogF.h"
 #include "textBuf.h"
@@ -68,9 +76,9 @@
    with pass2 patterns */
 #define EQUIVALENT_STYLE(style1, style2, firstPass2Style) (style1 == style2 || \
     	(style1 == UNFINISHED_STYLE && \
-    	 (style2 == PLAIN_STYLE || style2 >= firstPass2Style)) || \
+    	 (style2 == PLAIN_STYLE || (unsigned char)style2 >= firstPass2Style)) || \
     	(style2 == UNFINISHED_STYLE && \
-    	 (style1 == PLAIN_STYLE || style1 >= firstPass2Style)))
+    	 (style1 == PLAIN_STYLE || (unsigned char)style1 >= firstPass2Style)))
 
 /* Scanning context can be reduced (with big efficiency gains) if we
    know that patterns can't cross line boundaries, which is implied
@@ -104,7 +112,7 @@ typedef struct {
 typedef struct {
     highlightDataRec *pass1Patterns;
     highlightDataRec *pass2Patterns;
-    char *majorStyles;
+    char *parentStyles;
     reparseContext contextRequirements;
     styleTableEntry *styleTable;
     int nStyles;
@@ -137,15 +145,15 @@ static void fillStyleString(char **stringPtr, char **stylePtr, char *toPtr,
 static void modifyStyleBuf(textBuffer *styleBuf, char *styleString,
     	int startPos, int endPos, int firstPass2Style);
 static int lastModified(textBuffer *styleBuf);
-static Pixel allocColor(Widget w, char *colorName);
+static Pixel allocColor(Widget w, const char *colorName);
 static int max(int i1, int i2);
 static int min(int i1, int i2);
 static char getPrevChar(textBuffer *buf, int pos);
-regexp *compileREAndWarn(Widget parent, char *re);
-static int startOfMajorStyle(char *majorStyles, textBuffer *styleBuf, int pos,
-    	int limit);
-static int moveBackwardToEnsureContext(textBuffer *buf, textBuffer *styleBuf,
-    	reparseContext *context, char *majorStyles, int *pos);
+static regexp *compileREAndWarn(Widget parent, const char *re);
+static int parentStyleOf(const char *parentStyles, int style);
+static int isParentStyle(const char *parentStyles, int style1, int style2);
+static int findSafeParseRestartPos(textBuffer *buf,
+    	windowHighlightData *highlightData, int *pos);
 static int backwardOneContext(textBuffer *buf, reparseContext *context,
     	int fromPos);
 static int forwardOneContext(textBuffer *buf, reparseContext *context,
@@ -153,7 +161,7 @@ static int forwardOneContext(textBuffer *buf, reparseContext *context,
 static void recolorSubexpr(regexp *re, int subexpr, int style, char *string,
     	char *styleString);
 static int indexOfNamedPattern(highlightPattern *patList, int nPats,
-    	char *patName);
+    	const char *patName);
 static int findTopLevelParentIndex(highlightPattern *patList, int nPats,
     	int index);
 static highlightDataRec *patternOfStyle(highlightDataRec *patterns, int style);
@@ -465,7 +473,7 @@ static void freeHighlightData(windowHighlightData *hd)
     	freePatterns(hd->pass1Patterns);
     if (hd->pass2Patterns != NULL)
     	freePatterns(hd->pass2Patterns);
-    XtFree(hd->majorStyles);
+    XtFree(hd->parentStyles);
     BufFree(hd->styleBuffer);
     XtFree((char *)hd->styleTable);
     XtFree((char *)hd);
@@ -526,7 +534,7 @@ static windowHighlightData *createHighlightData(WindowInfo *window,
 {
     int i, nPass1Patterns, nPass2Patterns;
     int noPass1, noPass2;
-    char *majorStyles, *majorStylesPtr;
+    char *parentStyles, *parentStylesPtr, *parentName;
     highlightPattern *pass1PatternSrc, *pass2PatternSrc, *p1Ptr, *p2Ptr;
     styleTableEntry *styleTable, *styleTablePtr;
     textBuffer *styleBuf;
@@ -567,8 +575,16 @@ any existing style", "Dismiss", patternSrc[i].style, patternSrc[i].name);
        shows this setting only on top patterns which is much less confusing) */
     for (i=0; i<nPatterns; i++) {
     	if (patternSrc[i].subPatternOf != NULL) {
-    	    if (patternSrc[findTopLevelParentIndex(patternSrc, nPatterns, i)].
-    	    	    flags & DEFER_PARSING)
+	    int parentindex;
+	    
+	    parentindex=findTopLevelParentIndex(patternSrc, nPatterns, i);
+	    if (parentindex==-1) {
+	       DialogF(DF_WARN, window->shell, 1,
+   		   "Pattern \"%s\" does not have valid parent",
+		   "Dismiss", patternSrc[i].name);
+	       return NULL;
+	    }
+    	    if (patternSrc[parentindex].flags & DEFER_PARSING)
     	    	patternSrc[i].flags |= DEFER_PARSING;
     	    else
     	    	patternSrc[i].flags &= ~DEFER_PARSING;
@@ -647,16 +663,22 @@ any existing style", "Dismiss", patternSrc[i].style, patternSrc[i].name);
     for (i=1; i<nPass2Patterns; i++)
     	pass2Pats[i].style = 'B' + (noPass1 ? 0 : nPass1Patterns-1) + i;
     
-    /* Create table for finding top level style from sub-pattern styles */
-    majorStylesPtr = majorStyles = XtMalloc(nPass1Patterns + nPass2Patterns + 1);
-    *majorStylesPtr++ = PLAIN_STYLE;
-    *majorStylesPtr++ = PLAIN_STYLE;
-    for (i=1; i<nPass1Patterns; i++)
-	*majorStylesPtr++ = pass1Pats[findTopLevelParentIndex(pass1PatternSrc,
-    	    	    nPass1Patterns, i)].style;
-    for (i=1; i<nPass2Patterns; i++)
-	*majorStylesPtr++ = pass2Pats[findTopLevelParentIndex(pass2PatternSrc,
-    	    	    nPass2Patterns, i)].style;
+    /* Create table for finding parent styles */
+    parentStylesPtr = parentStyles = XtMalloc(nPass1Patterns+nPass2Patterns+2);
+    *parentStylesPtr++ = '\0';
+    *parentStylesPtr++ = '\0';
+    for (i=1; i<nPass1Patterns; i++) {
+	parentName = pass1PatternSrc[i].subPatternOf;
+	*parentStylesPtr++ = parentName == NULL ? PLAIN_STYLE :
+		pass1Pats[indexOfNamedPattern(pass1PatternSrc,
+		nPass1Patterns, parentName)].style;
+    }
+    for (i=1; i<nPass2Patterns; i++) {
+	parentName = pass2PatternSrc[i].subPatternOf;
+	*parentStylesPtr++ = parentName == NULL ? PLAIN_STYLE :
+		pass2Pats[indexOfNamedPattern(pass2PatternSrc,
+		nPass2Patterns, parentName)].style;
+    }
     
     /* Set up table for mapping colors and fonts to syntax */
     styleTablePtr = styleTable = (styleTableEntry *)XtMalloc(
@@ -697,7 +719,7 @@ any existing style", "Dismiss", patternSrc[i].style, patternSrc[i].name);
     highlightData =(windowHighlightData *)XtMalloc(sizeof(windowHighlightData));
     highlightData->pass1Patterns = pass1Pats;
     highlightData->pass2Patterns = pass2Pats;
-    highlightData->majorStyles = majorStyles;
+    highlightData->parentStyles = parentStyles;
     highlightData->styleTable = styleTable;
     highlightData->nStyles = styleTablePtr - styleTable;
     highlightData->styleBuffer = styleBuf;
@@ -757,6 +779,12 @@ static highlightDataRec *compilePatterns(Widget dialogParent,
        just colors and fonts for sub-expressions of the parent pattern */
     for (i=0; i<nPatterns; i++) {
         compiledPats[i].colorOnly = patternSrc[i].flags & COLOR_ONLY;
+	if (compiledPats[i].colorOnly && compiledPats[i].nSubPatterns != 0) {
+	    DialogF(DF_WARN, dialogParent, 1,
+   		   "Color-only pattern \"%s\" may not have subpatterns",
+   		   "Dismiss", patternSrc[i].name);
+ 	    return NULL;
+	}
         nSubExprs = 0;
         if (patternSrc[i].startRE != NULL) {
             ptr = patternSrc[i].startRE;
@@ -866,7 +894,7 @@ static highlightDataRec *compilePatterns(Widget dialogParent,
 	}
 	*(ptr-1) = '\0';
 	compiledPats[patternNum].subPatternRE = CompileRE(bigPattern,
-	    	&compileMsg);
+	    	&compileMsg, REDFLT_STANDARD);
 	if (compiledPats[patternNum].subPatternRE == NULL) {
     	    fprintf(stderr, "Error compiling syntax highlight patterns:\n%s",
     	    	    compileMsg);
@@ -891,13 +919,13 @@ static void freePatterns(highlightDataRec *patterns)
     
     for (i=0; patterns[i].style!=0; i++) {
     	if (patterns[i].startRE != NULL)
-    	    XtFree((char *)patterns[i].startRE);
+    	    free((char *)patterns[i].startRE);
     	if (patterns[i].endRE != NULL)
-    	    XtFree((char *)patterns[i].endRE);
+    	    free((char *)patterns[i].endRE);
     	if (patterns[i].errorRE != NULL)
     	    XtFree((char *)patterns[i].errorRE);
     	if (patterns[i].subPatternRE != NULL)
-    	    XtFree((char *)patterns[i].subPatternRE);
+    	    free((char *)patterns[i].subPatternRE);
     }
     for (i=0; patterns[i].style!=0; i++)
     	if (patterns[i].subPatterns != NULL)
@@ -925,7 +953,7 @@ static void handleUnparsedRegionCB(textDisp *textD, int pos, void *cbArg)
     reparseContext *context = &highlightData->contextRequirements;
     highlightDataRec *pass2Patterns = highlightData->pass2Patterns;
     char *string, *styleString, *stringPtr, *stylePtr, c, prevChar;
-    int firstPass2Style = pass2Patterns[1].style;
+    int firstPass2Style = (unsigned char)pass2Patterns[1].style;
     
     /* If there are no pass 2 patterns to process, do nothing (but this
        should never be triggered) */
@@ -939,7 +967,8 @@ static void handleUnparsedRegionCB(textDisp *textD, int pos, void *cbArg)
     beginSafety = backwardOneContext(buf, context, beginParse);
     for (p=beginParse; p>=beginSafety; p--) {
     	c = BufGetCharacter(styleBuf, p);
-    	if (c != UNFINISHED_STYLE && c != PLAIN_STYLE && c < firstPass2Style) {
+    	if (c != UNFINISHED_STYLE && c != PLAIN_STYLE &&
+		(unsigned char)c < firstPass2Style) {
     	    beginSafety = p + 1;
     	    break;
     	}
@@ -953,11 +982,12 @@ static void handleUnparsedRegionCB(textDisp *textD, int pos, void *cbArg)
     endSafety = forwardOneContext(buf, context, endParse);
     for (p=pos; p<endSafety; p++) {
     	c = BufGetCharacter(styleBuf, p);
-    	if (c != UNFINISHED_STYLE && c != PLAIN_STYLE && c < firstPass2Style) {
+    	if (c != UNFINISHED_STYLE && c != PLAIN_STYLE &&
+		(unsigned char)c < firstPass2Style) {
     	    endParse = min(endParse, p);
     	    endSafety = p;
     	    break;
-    	} else if (c != UNFINISHED_STYLE && p < endParse) {
+    	} else if ((unsigned char)c != UNFINISHED_STYLE && p < endParse) {
     	    endParse = p;
     	    if (c < firstPass2Style)
     	    	endSafety = p;
@@ -996,88 +1026,77 @@ static void handleUnparsedRegionCB(textDisp *textD, int pos, void *cbArg)
 static void incrementalReparse(windowHighlightData *highlightData,
     	textBuffer *buf, int pos, int nInserted, char *delimiters)
 {
-    int beginParse, endParse, endAt, lastMod, posInStyle, nPasses;
+    int beginParse, endParse, endAt, lastMod, parseInStyle, nPasses;
     textBuffer *styleBuf = highlightData->styleBuffer;
     highlightDataRec *pass1Patterns = highlightData->pass1Patterns;
     highlightDataRec *pass2Patterns = highlightData->pass2Patterns;
+    highlightDataRec *startPattern;
     reparseContext *context = &highlightData->contextRequirements;
-    char *majorStyles = highlightData->majorStyles;
+    char *parentStyles = highlightData->parentStyles;
 
     /* Find the position "beginParse" at which to begin reparsing.  This is
        far enough back in the buffer such that the guranteed number of
        lines and characters of context are examined. */
     beginParse = pos;
-    posInStyle = !moveBackwardToEnsureContext(buf, styleBuf, context,
-    	    majorStyles, &beginParse);
+    parseInStyle = findSafeParseRestartPos(buf, highlightData, &beginParse);
 
-    /*
-    ** If moveBackwardToEnsureContext left us inside of a styled
-    ** region, it must be re-parsed first before proceeding to the
-    ** general parsing of unstyled text below.  Parsing extends forward
-    ** until nothing is changed within one context distance of the last
-    ** change.
-    **
-    ** This is clumsily organized as: parse your way out of the current
-    ** (major) style, then parse unstyled text separately, but with similar
-    ** code. It is not general enough to parse its way out of multiple levels
-    ** of nested styles.  Originally, I thought this should be possible, but
-    ** if a section were parsed with a deeply nested pattern, I don't know
-    ** how to tell at what level to resume after the pattern is finished,
-    ** without re-starting from the beginning of the style.  We can't simply
-    ** trust the original styles to be correct in case something really did
-    ** change in the new parsing.  This might somehow be deduced from the
-    ** existing style BEFORE the section began, but this is getting very
-    ** complicated and hard to test.
-    */
+    /* Find the position "endParse" at which point it is safe to stop
+       parsing, unless styles are getting changed beyond the last
+       modification */
     lastMod = pos + nInserted;
     endParse = forwardOneContext(buf, context, lastMod);
-    if (posInStyle) {
-    	for (nPasses=0; ; nPasses++) {
-    	    /* printf("parsing within %c from %d to %d\n",
-    	    	    BufGetCharacter(styleBuf, beginParse), beginParse,
-    	    	    endParse); */
-    	    endAt = parseBufferRange(patternOfStyle(pass1Patterns,
-    	    	    majorStyles[BufGetCharacter(styleBuf, beginParse)-'A']),
-    	    	    NULL, buf, styleBuf, context, beginParse, endParse,
-    	    	    delimiters);
-	    if (endAt < endParse) {
-	    	beginParse = endAt;
-	    	endParse = forwardOneContext(buf, context,
-	    	    	max(endAt, max(lastModified(styleBuf), lastMod)));
-	    	break;
-	    } else if (lastModified(styleBuf) <= lastMod)
-	    	return;
-	    lastMod = lastModified(styleBuf);
-    	    endParse = min(buf->length, forwardOneContext(buf, context, lastMod)
-    	    	    + (REPARSE_CHUNK_SIZE << nPasses));
-	    /* printf("falling back: "); */
-	 }   	
-    }
     
     /*
     ** Parse the buffer from beginParse, until styles compare
     ** with originals for one full context distance.  Distance increases
-    ** by powers of two until nothing changes from previous step.
+    ** by powers of two until nothing changes from previous step.  If
+    ** parsing ends before endParse, start again one level up in the
+    ** pattern hierarchy
     */
     for (nPasses=0; ; nPasses++) {
 	
 	/* Parse forward from beginParse to one context beyond the end
 	   of the last modification */
-    	/* printf("parsing from %d to %d\n", beginParse, endParse); */
-	parseBufferRange(pass1Patterns, pass2Patterns, buf, styleBuf,
-	    	context, beginParse, endParse, delimiters);
-    	if (endParse == buf->length || lastModified(styleBuf) <= lastMod)
-    	    return;
-    	
-    	/* Continue expanding region by powers of 2 * REPARSE_CHUNK_SIZE
-    	   until all changes have been propagated */
-    	lastMod = lastModified(styleBuf);
-    	beginParse = startOfMajorStyle(majorStyles, styleBuf, lastMod-1,
-    	    	beginParse);
-    	endParse = min(buf->length, forwardOneContext(buf, context, lastMod)
-    	    	+ (REPARSE_CHUNK_SIZE << nPasses));
-    	/* printf("falling back: "); */
-    }
+        startPattern = patternOfStyle(pass1Patterns, parseInStyle);
+        /* If there is no pattern matching the style, it must be a pass-2
+           style. It that case, it is (probably) safe to start parsing with
+           the root pass-1 pattern again. Anyway, passing a NULL-pointer to
+           the parse routine would result in a crash; restarting with pass-1 
+           patterns is certainly preferable, even if there is a slight chance 
+           of a faulty coloring. */
+	if (!startPattern) {
+	    startPattern = pass1Patterns;
+	}
+    	endAt = parseBufferRange(startPattern,
+    	    	pass2Patterns, buf, styleBuf, context, beginParse, endParse,
+		delimiters);
+	
+	/* If parse completed at this level, move one style up in the
+	   hierarchy and start again from where the previous parse left off. */
+	if (endAt < endParse) {
+	    beginParse = endAt;
+	    endParse = forwardOneContext(buf, context,
+	    	    max(endAt, max(lastModified(styleBuf), lastMod)));
+	    if (IS_PLAIN(parseInStyle)) {
+		fprintf(stderr,
+			"NEdit internal error: incr. reparse fell short\n");
+		return;
+	    }
+	    parseInStyle = parentStyleOf(parentStyles, parseInStyle);
+	    
+	/* One context distance beyond last style changed means we're done */
+	} else if (lastModified(styleBuf) <= lastMod) {
+	    return;
+	    
+	/* Styles are changing beyond the modification, continue extending
+	   the end of the parse range by powers of 2 * REPARSE_CHUNK_SIZE and
+	   reparse until nothing changes */
+	} else {
+	    lastMod = lastModified(styleBuf);
+    	    endParse = min(buf->length, forwardOneContext(buf, context, lastMod)
+    	    	    + (REPARSE_CHUNK_SIZE << nPasses));
+	}
+    }	
 }
 
 /*
@@ -1103,7 +1122,8 @@ static int parseBufferRange(highlightDataRec *pass1Patterns,
     char *string, *styleString, *stringPtr, *stylePtr, *temp, prevChar;
     int endSafety, endPass2Safety, startPass2Safety, tempLen;
     int modStart, modEnd, beginSafety, beginStyle, p, style;
-    int firstPass2Style = pass2Patterns==NULL?INT_MAX:pass2Patterns[1].style;
+    int firstPass2Style = pass2Patterns == NULL ? INT_MAX :
+	    (unsigned char)pass2Patterns[1].style;
     
     /* Begin parsing one context distance back (or to the last style change) */
     beginStyle = pass1Patterns->style;
@@ -1126,7 +1146,6 @@ static int parseBufferRange(highlightDataRec *pass1Patterns,
     	    }
     	}
     }
-    	
     
     /* Parse one parse context beyond requested end to gurantee that parsing
        at endParse is complete, unless patterns can't cross line boundaries,
@@ -1260,13 +1279,14 @@ static int parseString(highlightDataRec *pattern, char **string,
     int i;
     char *stringPtr, *stylePtr, *startingStringPtr;
     signed char *subExpr;
-    highlightDataRec *subPat, *subSubPat;
+    highlightDataRec *subPat = NULL, *subSubPat;
     
     if (length <= 0)
     	return False;
     
     stringPtr = *string;
     stylePtr = *styleString;
+    
     while(ExecRE(pattern->subPatternRE, NULL, stringPtr, anchored ? *string+1 :
 	    *string+length+1, False, *prevChar, '\0', delimiters)) {
     	
@@ -1388,17 +1408,17 @@ static void passTwoParseString(highlightDataRec *pattern, char *string,
     	char *delimiters)
 {
     int inParseRegion = False;
-    char *stylePtr, *stringPtr, temp, *parseStart, *parseEnd, *s, *c;
-    int firstPass2Style = pattern[1].style;
+    char *stylePtr, *stringPtr, temp, *parseStart = NULL, *parseEnd, *s, *c;
+    int firstPass2Style = (unsigned char)pattern[1].style;
     
     for (c = string, s = styleString; ; c++, s++) {
     	if (!inParseRegion && *c != '\0' && (*s == UNFINISHED_STYLE ||
-    	    	*s == PLAIN_STYLE || *s >= firstPass2Style)) {
+    	    	*s == PLAIN_STYLE || (unsigned char)*s >= firstPass2Style)) {
     	    parseStart = c;
     	    inParseRegion = True;
     	}
     	if (inParseRegion && (*c == '\0' || !(*s == UNFINISHED_STYLE ||
-    	    	*s == PLAIN_STYLE || *s >= firstPass2Style))) {
+    	    	*s == PLAIN_STYLE || (unsigned char)*s >= firstPass2Style))) {
     	    parseEnd = c;
 	    if (parseStart != string)
 	    	*prevChar = *(parseStart-1);
@@ -1466,7 +1486,7 @@ static void modifyStyleBuf(textBuffer *styleBuf, char *styleString,
     for (c=styleString, pos=startPos; pos<modStart && pos<endPos; c++, pos++) {
     	bufChar = BufGetCharacter(styleBuf, pos);
     	if (*c != bufChar && !(bufChar == UNFINISHED_STYLE &&
-    	    	(*c == PLAIN_STYLE || *c >= firstPass2Style))) {
+    	    	(*c == PLAIN_STYLE || (unsigned char)*c >= firstPass2Style))) {
     	    if (pos < minPos) minPos = pos;
     	    if (pos > maxPos) maxPos = pos;
     	}
@@ -1475,7 +1495,7 @@ static void modifyStyleBuf(textBuffer *styleBuf, char *styleString,
     	    pos<endPos; c++, pos++) {
     	bufChar = BufGetCharacter(styleBuf, pos);
     	if (*c != bufChar && !(bufChar == UNFINISHED_STYLE &&
-    	    	(*c == PLAIN_STYLE || *c >= firstPass2Style))) {
+    	    	(*c == PLAIN_STYLE || (unsigned char)*c >= firstPass2Style))) {
     	    if (pos < minPos) minPos = pos;
     	    if (pos+1 > maxPos) maxPos = pos+1;
     	}
@@ -1502,32 +1522,118 @@ static int lastModified(textBuffer *styleBuf)
     return 0;
 }
 
+/*
+** Compute the distance between two colors.
+*/
+
+static double colorDistance(const XColor *c1, const XColor *c2)
+{
+    /* This is done in RGB space, which is close, but not optimal.  It's
+       probably better to do it in HSV or YIQ space, however, that means
+       a whole lot of extra conversions.  This would allow us to weight
+       the coordinates differently, e.g, prefer to match hue over
+       brightness. */
+
+    static const double scale = 65535;
+
+    double tred   = c1->red   / scale - c2->red   / scale;
+    double tgreen = c1->green / scale - c2->green / scale;
+    double tblue  = c1->blue  / scale - c2->blue  / scale;
+
+    /* use square Euclidian distance */
+    return tred * tred + tgreen * tgreen + tblue * tblue;
+}
+
 /*
 ** Allocate a read-only (shareable) colormap cell for a named color, from the
 ** the default colormap of the screen on which the widget (w) is displayed. If
-** the colormap is full and there's no suitable substiture, print an error on
+** the colormap is full and there's no suitable substitute, print an error on
 ** stderr, and return the widget's foreground color as a backup.
 */
-static Pixel allocColor(Widget w, char *colorName)
+
+static Pixel allocColor(Widget w, const char *colorName)
 {
-    XColor colorDef;
-    Display *display = XtDisplay(w);
-    int screenNum = XScreenNumberOfScreen(XtScreen(w));
-    Colormap cMap = DefaultColormap(display, screenNum);
-    Pixel foreground;
+    XColor       colorDef;
+    XColor      *allColorDefs;
+    Display     *display = XtDisplay(w);
+    Colormap     cMap;
+    Pixel        foreground, bestPixel;
+    double       small = 1.0e9;
+    int          depth;
+    unsigned int ncolors;
+    unsigned long i, best = 0;    /* pixel value */
+    
+    /* Get the correct colormap for compatability with the "best" visual
+       feature in 5.2.  Default visual of screen is no good here. */
 
-    /* Allocate and return the color cell, or print an error and fall through */
-    if (XParseColor(display, cMap,  colorName, &colorDef)) {
-	if (XAllocColor(display, cMap, &colorDef))
-	    return colorDef.pixel;
-    	else
-	    fprintf(stderr, "NEdit: Can't allocate color: %s\n", colorName);
-    } else
-	fprintf(stderr, "NEdit: Color name %s not in database\n",  colorName);
+    XtVaGetValues(w,
+                  XtNcolormap,   &cMap,
+                  XtNdepth,      &depth,
+                  XtNforeground, &foreground,
+                  NULL);
 
-    /* Color cell couldn't be allocated, return the widget's foreground color */
-    XtVaGetValues(w, XmNforeground, &foreground, 0);
-    return foreground;
+    bestPixel = foreground; /* Our last fallback */
+
+    /* First, check for valid syntax */        
+    if (! XParseColor(display, cMap, colorName, &colorDef)) {
+        fprintf(stderr, "NEdit: Color name %s not in database\n",  colorName);
+        return foreground;
+    }
+
+    /* Attempt allocation of the exact color. */
+    if (XAllocColor(display, cMap, &colorDef))
+        return colorDef.pixel;
+
+    /* ---------- Allocation failed, the colormap may be full. ---------- */
+
+#if 0
+    printf("Couldn't allocate %d %d %d\n", colorDef.red, colorDef.green, colorDef.blue);
+#endif
+ 
+    /* We can't do the nearest-match on other than 8 bit visuals because
+       it just takes too long.  */
+
+    if (depth > 8)               /* Oh no! */
+        return foreground;
+
+    /* Get the entire colormap so we can find the closet one. */
+    ncolors = pow(2, depth);
+    allColorDefs = malloc(ncolors * sizeof(XColor));
+    memset(allColorDefs, 0, ncolors * sizeof(XColor));
+    
+    for (i = 0; i < ncolors; i++)
+        allColorDefs[i].pixel = i;
+
+    XQueryColors(display, cMap, allColorDefs, ncolors);
+
+    /* Scan through each color, looking for the closest one. */
+    for (i = 0; i < ncolors; i++)
+    {
+        double dist = colorDistance(&allColorDefs[i], &colorDef);
+
+        if (dist < small)
+        {
+            best = i;
+            small = dist;
+        }
+    }
+
+    /* Legally try to acquire the shared color- we should loop through
+       the shortest distances here.  We could sort the map in order
+       of decreasing distances and loop through it until one works. */
+
+    if (XAllocColor(display, cMap, &allColorDefs[best]))
+        bestPixel = allColorDefs[best].pixel;
+
+#if 0
+    printf("Got %d %d %d, ", allColorDefs[best].red,
+                             allColorDefs[best].green,
+                             allColorDefs[best].blue);
+    printf("That's %f off\n", small);
+#endif
+
+    free(allColorDefs);
+    return bestPixel;
 }
 
 /*
@@ -1541,12 +1647,12 @@ static char getPrevChar(textBuffer *buf, int pos)
 /*
 ** compile a regular expression and present a user friendly dialog on failure.
 */
-regexp *compileREAndWarn(Widget parent, char *re)
+static regexp *compileREAndWarn(Widget parent, const char *re)
 {
     regexp *compiledRE;
     char *compileMsg;
     
-    compiledRE = CompileRE(re, &compileMsg);
+    compiledRE = CompileRE(re, &compileMsg, REDFLT_STANDARD);
     if (compiledRE == NULL) {
    	DialogF(DF_WARN, parent, 1,
    	       "Error in syntax highlighting regular expression:\n%s\n%s",
@@ -1556,115 +1662,171 @@ regexp *compileREAndWarn(Widget parent, char *re)
     return compiledRE;
 }
 
-/*
-** Begin with character at pos and scan backwards through "styleBuf" looking
-** for the next change in style.  Returns the position 1st character of the
-** style pointed to by "pos"
-*/
-static int startOfMajorStyle(char *majorStyles, textBuffer *styleBuf, int pos,
-    	int limit)
+static int parentStyleOf(const char *parentStyles, int style)
 {
-    int i;
-    char style, startStyle;
-    
-    /* Find the style of the character at pos */
-    startStyle = BufGetCharacter(styleBuf, pos);
-    
-    if (IS_PLAIN(startStyle))
-    	return pos;
+    return parentStyles[style-'A'];
+}
 
-    /* Find the top level style corresponding to startStyle */
-    startStyle = majorStyles[startStyle - 'A'];
+static int isParentStyle(const char *parentStyles, int style1, int style2)
+{
+    int p;
     
-    /* Scan backwards, comparing major (top level) styles in hierarchy,
-       until the style changes */
-    for (i=pos-1; i>=limit; i--) {
-    	style = BufGetCharacter(styleBuf, i);
-    	if (majorStyles[style-'A'] != startStyle)
-    	    return i + 1;
-    }
-    return limit;
+    for (p = parentStyleOf(parentStyles, style2); p != '\0';
+	    p = parentStyleOf(parentStyles, p))
+	if (style1 == p)
+	    return TRUE;
+    return FALSE;
 }
 
 /*
-** Back up position pointed to by "pos" enough that parsing from that point on
-** will satisfy context gurantees for pattern matching for modifications at
-** pos.  If that can't be done without extending beyond 2X the requested
-** context because a styled region extends back from with the context region,
-** return False and set pos to the first position outside of the context
-** region from which parsing of the styled region can safely be resumed using
-** the patterns for the major (top-level) style at that position.
-**
-** The requirement that parsing can resume using major style patterns means
-** that the returned position may be considerably farther than two context
-** distances back from pos.  This means there can be significant performance
-** penalties for patterns which allow sub-styles (anything below major styles)
-** to match significant ranges of text.  A comment in incremental re-parse
-** explains this decision further.
+** Discriminates patterns which can be used with parseString from those which
+** can't.  Leaf patterns are not suitable for parsing, because patterns
+** contain the expressions used for parsing within the context of their own
+** operation, i.e. the parent pattern initiates, and leaf patterns merely
+** confirm and color.  Returns TRUE if the pattern is suitable for parsing.
 */
-static int moveBackwardToEnsureContext(textBuffer *buf, textBuffer *styleBuf,
-    	reparseContext *context, char *majorStyles, int *pos)
+static int patternIsParsable(highlightDataRec *pattern)
 {
-    int style, overlapStyle, overlapMajorStyle, checkBackTo;
-    int safeParseStart, maxSafeParseStart, i;
+    return pattern != NULL && pattern->subPatternRE != NULL;
+}
+
+/*
+** Back up position pointed to by "pos" enough that parsing from that point
+** on will satisfy context gurantees for pattern matching for modifications
+** at pos.  Returns the style with which to begin parsing.  The caller is
+** guranteed that parsing may safely BEGIN with that style, but not that it
+** will continue at that level.
+**
+** This routine can be fooled if a continuous style run of more than one
+** context distance in length is produced by multiple pattern matches which
+** abut, rather than by a single continuous match.  In this  case the
+** position returned by this routine may be a bad starting point which will
+** result in an incorrect re-parse.  However this will happen very rarely,
+** and, if it does, is unlikely to result in incorrect highlighting.
+*/
+static int findSafeParseRestartPos(textBuffer *buf,
+    	windowHighlightData *highlightData, int *pos)
+{
+    int style, startStyle, runningStyle, checkBackTo, safeParseStart, i;
+    char *parentStyles = highlightData->parentStyles;
+    highlightDataRec *pass1Patterns = highlightData->pass1Patterns;
+    reparseContext *context = &highlightData->contextRequirements;
     
-    /* Back up far enough to ensure that expressions can match any characters
-       within the pattern's guranteed context back from the change */
+    /* We must begin at least one context distance back from the change */
     *pos = backwardOneContext(buf, context, *pos);
     
     /* If the new position is outside of any styles or at the beginning of
        the buffer, this is a safe place to begin parsing, and we're done */
     if (*pos == 0)
-    	return True;
-    overlapStyle = BufGetCharacter(styleBuf, *pos);
-    if (IS_PLAIN(overlapStyle))
-    	return True;
+    	return PLAIN_STYLE;
+    startStyle = BufGetCharacter(highlightData->styleBuffer, *pos);
+    if (IS_PLAIN(startStyle))
+    	return PLAIN_STYLE;
     
     /*
-    ** The new position is inside of a styled region, meaning, its
-    ** pattern could potentially be affected by the modification.
+    ** The new position is inside of a styled region, meaning, its pattern
+    ** could potentially be affected by the modification.
     **
     ** Follow the style back by enough context to ensure that if we don't find
     ** its beginning, at least we've found a safe place to begin parsing
-    ** WITHIN the encroaching style.
+    ** within the styled region.
     **
-    ** A safe starting position within a style is one which is preceded by a
-    ** the major style (i.e. not the middle of a sub-style), and is far enough
-    ** from the beginning and end of the style to gurantee that it's not
-    ** within the start or end expression match.
-    */   
-    checkBackTo = 0;
-    maxSafeParseStart = backwardOneContext(buf, context, *pos);
-    safeParseStart = 0;
-    overlapMajorStyle = majorStyles[overlapStyle-'A'];
+    ** A safe starting position within a style is either at a style
+    ** boundary, or far enough from the beginning and end of the style run
+    ** to ensure that it's not within the start or end expression match
+    ** (unfortunately, abutting styles can produce false runs so we're not
+    ** really ensuring it, just making it likely).
+    */ 
+    if (patternIsParsable(patternOfStyle(pass1Patterns, startStyle))) {
+    	safeParseStart = backwardOneContext(buf, context, *pos);
+    	checkBackTo = backwardOneContext(buf, context, safeParseStart);
+    } else {
+	safeParseStart = 0;
+	checkBackTo = 0;
+    }
+    runningStyle = startStyle;
     for (i = *pos-1; ; i--) {
     	
-    	/* The start of buffer is certainly a safe place to parse from */
+    	/* The start of the buffer is certainly a safe place to parse from */
     	if (i == 0) {
     	    *pos = 0;
-    	    return True;
+    	    return PLAIN_STYLE;
     	}
     	
-    	/* If the start of the style is encountered, no internal parsing
-    	   of the style is required */
-    	style = BufGetCharacter(styleBuf, i);
-    	if (overlapMajorStyle != majorStyles[style-'A']) {
-    	    *pos = i + 1;
-    	    return True;
+    	/* If the style is preceded by a parent style, it's safe to parse
+	   with the parent style, provided that the parent is parsable. */
+    	style = BufGetCharacter(highlightData->styleBuffer, i);
+	if (isParentStyle(parentStyles, style, runningStyle)) {
+	    if (patternIsParsable(patternOfStyle(pass1Patterns, style))) {
+		*pos = i + 1;
+		return style;
+	    } else {
+		/* The parent is not parsable, so well have to continue
+		   searching. The parent now becomes the running style. */
+	    	runningStyle = style;
+	    }
     	}
-    	
-    	/* A point preceded by the major style is safe for parsing witin the
-    	   style, but only after we've ensured that it's far enough from the
-    	   beginning and end of the style that it's not in the middle of the
-    	   start or end expression matches */
-    	if (i <= maxSafeParseStart && safeParseStart == 0 &&
-    	    	style == overlapMajorStyle) {
-    	    safeParseStart = i + 1;
-    	    checkBackTo = backwardOneContext(buf, context, i);
+	
+	/* If the style is preceded by a child style, it's safe to resume
+	   parsing with the running style, provided that the running
+	   style is parsable. */
+    	else if (isParentStyle(parentStyles, runningStyle, style)) {
+	    if (patternIsParsable
+		   (patternOfStyle(pass1Patterns, runningStyle))) {
+		*pos = i + 1;
+		return runningStyle;
+	    }
+	    /* Else: keep searching; it's no use switching to the child style
+	       because even the running one is not parsable. */
     	}
+	
+	/* If the style is preceded by a sibling style, it's safe to resume
+	   parsing with the common ancestor style, provided that the ancestor
+	   is parsable. Checking for siblings is very hard; checking whether
+	   the style has the same parent will probably catch 99% of the cases
+	   in practice. */
+	else if (runningStyle != style &&
+	    isParentStyle(parentStyles,
+			  parentStyleOf(parentStyles, runningStyle), style)) {
+	    int parentStyle = parentStyleOf(parentStyles, runningStyle);
+	    if (patternIsParsable(patternOfStyle(pass1Patterns, parentStyle))) {
+		*pos = i + 1;
+		return parentStyle;
+	    } else {
+		/* Switch to the new style */
+		runningStyle = style;
+	    }
+    	}
+
+	/* If the style is preceded by an unrelated style, it's safe to
+	   resume parsing with PLAIN_STYLE. (Actually, it isn't, because
+	   we didn't really check for all possible sibling relations; but
+	   it will be ok in practice.) */
+	else if (runningStyle != style) {
+	    *pos = i + 1;
+    	    return PLAIN_STYLE;
+    	}
+	
+	/* If the style is parsable and didn't change for one whole context
+	   distance on either side of safeParseStart, safeParseStart is a
+	   reasonable guess at a place to start parsing.
+	   Note: No 'else' here! We may come from one of the 'fall-through
+	   cases' above. */
     	if (i == checkBackTo) {
     	    *pos = safeParseStart;
-    	    return False;
+
+	    /* We should never return a non-parsable style, because it will
+	       result in an internal error. If the current style is not
+	       parsable, the pattern set most probably contains a context
+	       distance violation. In that case we can only avoid internal
+	       errors (by climbing the pattern hierarchy till we find a
+	       parsable ancestor) and hope that the highlighting errors are
+	       minor. */
+	    while (!patternIsParsable
+			(patternOfStyle(pass1Patterns, runningStyle)))
+	       runningStyle = parentStyleOf(parentStyles, runningStyle);
+
+    	    return runningStyle;
     	}
     }
 }
@@ -1741,6 +1903,8 @@ static highlightDataRec *patternOfStyle(highlightDataRec *patterns, int style)
     for (i=0; patterns[i].style!=0; i++)
     	if (patterns[i].style == style)
     	    return &patterns[i];
+    if (style == PLAIN_STYLE || style == UNFINISHED_STYLE)
+	return &patterns[0];
     return NULL;
 }
 
@@ -1755,7 +1919,7 @@ static int min(int i1, int i2)
 }
 
 static int indexOfNamedPattern(highlightPattern *patList, int nPats,
-    	char *patName)
+    	const char *patName)
 {
     int i;
     
@@ -1773,9 +1937,12 @@ static int findTopLevelParentIndex(highlightPattern *patList, int nPats,
     int topIndex;
     
     topIndex = index;
-    while (patList[topIndex].subPatternOf != NULL)
+    while (patList[topIndex].subPatternOf != NULL) {
     	topIndex = indexOfNamedPattern(patList, nPats,
     	    	patList[topIndex].subPatternOf);
+	if (index==topIndex)
+	   return -1; /* amai: circular dependency ?! */
+    }
     return topIndex;
 }
 
@@ -1794,12 +1961,12 @@ static void updateWindowHeight(WindowInfo *window, int oldFontHeight)
     	
     /* Decompose the window height into the part devoted to displaying
        text (textHeight) and the non-text part (boderHeight) */
-    XtVaGetValues(window->shell, XmNheight, &windowHeight, 0);
+    XtVaGetValues(window->shell, XmNheight, &windowHeight, NULL);
     XtVaGetValues(window->textArea, XmNheight, &textAreaHeight,
-    	    textNmarginHeight, &marginHeight, 0);
+    	    textNmarginHeight, &marginHeight, NULL);
     textHeight = textAreaHeight - 2*marginHeight;
     for (i=0; i<window->nPanes; i++) {
-    	XtVaGetValues(window->textPanes[i], XmNheight, &textAreaHeight, 0);
+    	XtVaGetValues(window->textPanes[i], XmNheight, &textAreaHeight, NULL);
     	textHeight += textAreaHeight - 2*marginHeight;
     }
     borderHeight = windowHeight - textHeight;
@@ -1812,10 +1979,10 @@ static void updateWindowHeight(WindowInfo *window, int oldFontHeight)
        requests.  Our height increment is probably wrong because it is still
        set for the previous font.  Set the new height in advance, before
        attempting to resize. */
-    XtVaSetValues(window->shell, XmNheightInc, getFontHeight(window), 0);
+    XtVaSetValues(window->shell, XmNheightInc, getFontHeight(window), NULL);
     
     /* Re-size the window */
-    XtVaSetValues(window->shell, XmNheight, newWindowHeight, 0);
+    XtVaSetValues(window->shell, XmNheight, newWindowHeight, NULL);
 }
 
 /*
