@@ -1,3 +1,4 @@
+static const char CVSID[] = "$Id: nedit.c,v 1.18.2.1 2001/09/20 14:24:21 tringali Exp $";
 /*******************************************************************************
 *									       *
 * nedit.c -- Nirvana Editor main program				       *
@@ -11,7 +12,7 @@
 * 									       *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
-* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
 * for more details.							       *
 * 									       *
 * You should have received a copy of the GNU General Public License along with *
@@ -45,12 +46,10 @@
 #include "../util/VMSparam.h"
 #include "../util/VMSUtils.h"
 #else
+#ifndef __MVS__
 #include <sys/param.h>
+#endif
 #endif /*VMS*/
-#include "../util/misc.h"
-#include "../util/printUtils.h"
-#include "../util/fileUtils.h"
-#include "../util/getfiles.h"
 #include "textBuf.h"
 #include "nedit.h"
 #include "file.h"
@@ -61,17 +60,28 @@
 #include "menu.h"
 #include "macro.h"
 #include "server.h"
+#include "rbTree.h"
 #include "interpret.h"
 #include "parse.h"
+#include "help.h"
+#include "../util/misc.h"
+#include "../util/printUtils.h"
+#include "../util/fileUtils.h"
+#include "../util/getfiles.h"
+
 
 static void nextArg(int argc, char **argv, int *argIndex);
-static int checkDoMacroArg(char *macro);
-void maskArgvKeywords(int argc, char **argv, char **maskArgs);
-void unmaskArgvKeywords(int argc, char **argv, char **maskArgs);
+static int checkDoMacroArg(const char *macro);
+static void maskArgvKeywords(int argc, char **argv, const char **maskArgs);
+static void unmaskArgvKeywords(int argc, char **argv, const char **maskArgs);
 
 WindowInfo *WindowList = NULL;
-Display *TheDisplay;
-char *ArgV0;
+Display *TheDisplay = NULL;
+char *ArgV0 = NULL;
+Boolean IsServer = False;
+
+#define NEDIT_DEFAULT_FONT "-*-helvetica-medium-r-normal-*-12-*-*-*-*-*-*"
+#define NEDIT_FIXED_FONT   "-*-courier-medium-r-normal-*-12-*-*-*-*-*-*"
 
 static char *fallbackResources[] = {
     "*menuBar.marginHeight: 0",
@@ -79,29 +89,29 @@ static char *fallbackResources[] = {
     "*pane.sashHeight: 11",
     "*pane.sashWidth: 11",
     "*text.selectionArrayCount: 3",
-    "*fontList:-adobe-helvetica-medium-r-normal-*-12-*-*-*-*-*-*",
-    "*XmList.fontList:-adobe-courier-medium-r-normal-*-12-*-*-*-*-*-*",
-    "*XmText.fontList:-adobe-courier-medium-r-normal-*-12-*-*-*-*-*-*",
-    /* This should not be necessary, but some default in lesstif is
+    "nedit*fontList: " NEDIT_DEFAULT_FONT,
+    "nedit*XmList.fontList: " NEDIT_FIXED_FONT,
+    /* This should not be necessary, but some default in LessTif is
        overriding the resource above, and specifying the app-name fixes it */
-    "nedit*XmText.fontList:-adobe-courier-medium-r-normal-*-12-*-*-*-*-*-*",
-    /* Same with this, both Solaris Motif and Lesstif seem to have some
+    "nedit*XmText.fontList: " NEDIT_FIXED_FONT,
+    /* Same with this, both Solaris Motif and LessTif seem to have some
        very specific defaults for file selection box fonts */
-    "nedit*FileSelect*XmList.fontList:-adobe-courier-medium-r-normal-*-12-*-*-*-*-*-*",
-    "*XmTextField.fontList:-adobe-courier-medium-r-normal-*-12-*-*-*-*-*-*",
-    "*background: #b3b3b3",
-    "*foreground: black",
-    "*text.lineNumForeground: #777777",
-    "*text.background: #e5e5e5",
-    "*text.foreground: black",
-    "*text.highlightBackground: red",
-    "*text.highlightForeground: black",
-    "*XmText*foreground: black",
-    "*XmText*background: #cccccc",
-    "*helpText.background: #cccccc",
-    "*helpText.foreground: black",
-    "*helpText.selectBackground: #b3b3b3",
-    "*helpText.font: -adobe-courier-medium-r-normal-*-12-*-*-*-*-*-*",
+    "nedit*XmFileSelectionBox*XmList.fontList: " NEDIT_FIXED_FONT,
+    "nedit*XmTextField.fontList: " NEDIT_FIXED_FONT,
+    "nedit*background: #b3b3b3",
+    "nedit*foreground: black",
+    "nedit*text.lineNumForeground: #777777",
+    "nedit*text.background: #e5e5e5",
+    "nedit*text.foreground: black",
+    "nedit*text.highlightBackground: red",
+    "nedit*text.highlightForeground: black",
+    "nedit*XmText*foreground: black",
+    "nedit*XmText*background: #cccccc",
+    "nedit*helpText.background: #cccccc",
+    "nedit*helpText.foreground: black",
+    "nedit*helpText.selectBackground: #b3b3b3",
+    "nedit*statsLine.background: #b3b3b3",
+    "nedit*helpText.font: " NEDIT_FIXED_FONT,
     "*XmText.translations: #override \
 Ctrl~Alt~Meta<KeyPress>v: paste-clipboard()\\n\
 Ctrl~Alt~Meta<KeyPress>c: copy-clipboard()\\n\
@@ -112,10 +122,10 @@ Ctrl~Alt~Meta<KeyPress>v: paste-clipboard()\\n\
 Ctrl~Alt~Meta<KeyPress>c: copy-clipboard()\\n\
 Ctrl~Alt~Meta<KeyPress>x: cut-clipboard()\\n\
 Ctrl~Alt~Meta<KeyPress>u: delete-to-start-of-line()\\n",
-    "*XmList*foreground: black",
-    "*XmList*background: #cccccc",
-    "*XmTextField*background: #cccccc",
-    "*XmTextField*foreground: black",
+    "nedit*XmList*foreground: black",
+    "nedit*XmList*background: #cccccc",
+    "nedit*XmTextField*background: #cccccc",
+    "nedit*XmTextField*foreground: black",
     "*iSearchForm*highlightThickness: 1",
     "*fileMenu.tearOffModel: XmTEAR_OFF_ENABLED",
     "*editMenu.tearOffModel: XmTEAR_OFF_ENABLED",
@@ -190,9 +200,15 @@ Ctrl~Alt~Meta<KeyPress>u: delete-to-start-of-line()\\n",
     "*searchMenu.replace.accelerator: Ctrl<Key>r",
     "*searchMenu.replace.acceleratorText: [Shift]Ctrl+R",
     "*searchMenu.replaceShift.accelerator: Shift Ctrl<Key>r",
-    "*searchMenu.replaceAgain.accelerator: Ctrl<Key>t",
-    "*searchMenu.replaceAgain.acceleratorText: [Shift]Ctrl+T",
-    "*searchMenu.replaceAgainShift.accelerator: Shift Ctrl<Key>t",
+    "*searchMenu.findReplace.accelerator: Ctrl<Key>r",
+    "*searchMenu.findReplace.acceleratorText: [Shift]Ctrl+R",
+    "*searchMenu.findReplaceShift.accelerator: Shift Ctrl<Key>r",
+    "*searchMenu.replaceFindAgain.accelerator: Ctrl<Key>t",
+    "*searchMenu.replaceFindAgain.acceleratorText: [Shift]Ctrl+T",
+    "*searchMenu.replaceFindAgainShift.accelerator: Shift Ctrl<Key>t",
+    "*searchMenu.replaceAgain.accelerator: Alt<Key>t",
+    "*searchMenu.replaceAgain.acceleratorText: [Shift]Alt+T",
+    "*searchMenu.replaceAgainShift.accelerator: Shift Alt<Key>t",
     "*searchMenu.gotoLineNumber.accelerator: Ctrl<Key>l",
     "*searchMenu.gotoLineNumber.acceleratorText: Ctrl+L",
     "*searchMenu.gotoSelected.accelerator: Ctrl<Key>e",
@@ -210,8 +226,7 @@ Ctrl~Alt~Meta<KeyPress>u: delete-to-start-of-line()\\n",
     "*preferencesMenu.mnemonic: P",
     "*preferencesMenu.statisticsLine.accelerator: Alt<Key>a",
     "*preferencesMenu.statisticsLine.acceleratorText: Alt+A",
-    "*preferencesMenu.overtype.accelerator: Ctrl<Key>b",
-    "*preferencesMenu.overtype.acceleratorText: Ctrl+B",
+    "*preferencesMenu.overtype.acceleratorText: Insert",
     "*shellMenu.mnemonic: l",
     "*shellMenu.filterSelection.accelerator: Alt<Key>r",
     "*shellMenu.filterSelection.acceleratorText: Alt+R",
@@ -241,7 +256,7 @@ Ctrl~Alt~Meta<KeyPress>u: delete-to-start-of-line()\\n",
     0
 };
 
-static char cmdLineHelp[] =
+static const char cmdLineHelp[] =
 #ifndef VMS
 "Usage:  nedit [-read] [-create] [-line n | +n] [-server] [-do command]\n\
 	      [-tags file] [-tabs n] [-wrap] [-nowrap] [-autowrap]\n\
@@ -258,42 +273,23 @@ static char cmdLineHelp[] =
 int main(int argc, char **argv)
 {
     int i, lineNum, nRead, fileSpecified = FALSE, editFlags = CREATE;
-    int isServer = FALSE, gotoLine = False, macroFileRead = False;
+    int gotoLine = False, macroFileRead = False;
     int iconic = False;
     char *toDoCommand = NULL, *geometry = NULL, *langMode = NULL;
     char filename[MAXPATHLEN], pathname[MAXPATHLEN];
     XtAppContext context;
     XrmDatabase prefDB;
-    static char *protectedKeywords[] = {"-iconic", "-icon", "-geometry", "-g",
-	    "-rv", "-reverse", "-bd", "-bordercolor", "-borderwidth", "-bw",
-	    "-title", NULL};
+    static const char *protectedKeywords[] = {"-iconic", "-icon", "-geometry",
+            "-g", "-rv", "-reverse", "-bd", "-bordercolor", "-borderwidth",
+	    "-bw", "-title", NULL};
     
     /* Save the command which was used to invoke nedit for restart command */
     ArgV0 = argv[0];
-    
-#ifndef NO_XMIM
-    /* Set local for C library and X, and Motif input functions */
-    if (setlocale(LC_CTYPE, "") == NULL) {
-	fprintf(stderr, "NEdit: Locale not supported by C library.\n");
-	if(setlocale(LC_CTYPE, NULL)==NULL) {
-            fprintf(stderr,"NEdit: cannot continue.\n");
-            exit(0);
-	} else
-            fprintf(stderr,"NEdit: Using %s locale instead.\n",
-            	    setlocale(LC_CTYPE, NULL));
-    }
-    if (!XSupportsLocale()) {
-    	fprintf(stderr, "NEdit: Xlib: locale %s not supported\n.",
-    	    	setlocale(LC_CTYPE, NULL));
-    	exit(0);
-    }
-    if (XSetLocaleModifiers("") == NULL)
-        fprintf(stderr,"NEdit: cannot set locale modifiers.\n");
-#else
-    /* Set locale for C library functions */
-    setlocale(LC_CTYPE, "");
-#endif
-    
+
+    /* Set locale for C library, X, and Motif input functions. 
+       Reverts to "C" if requested locale not available. */
+    XtSetLanguageProc(NULL, NULL, NULL);
+ 
     /* Initialize X toolkit (does not open display yet) */
     XtToolkitInitialize();
     context = XtCreateApplicationContext();
@@ -313,6 +309,10 @@ int main(int argc, char **argv)
     /* Convert the command line to Unix style (This is not an ideal solution) */
     ConvertVMSCommandLine(&argc, &argv);
 #endif /*VMS*/
+#ifdef __EMX__
+    /* expand wildcards if necessary */
+    _wildcard(&argc, &argv);
+#endif
     
     /* Read the preferences file and command line into a database */
     prefDB = CreateNEditPrefDB(&argc, argv);
@@ -328,7 +328,7 @@ int main(int argc, char **argv)
     unmaskArgvKeywords(argc, argv, protectedKeywords);
     if (!TheDisplay) {
 	XtWarning ("NEdit: Can't open display\n");
-	exit(0);
+	exit(EXIT_FAILURE);
     }
     
     /* Initialize global symbols and subroutines used in the macro language */
@@ -403,7 +403,7 @@ int main(int argc, char **argv)
     	    else
     	    	gotoLine = True;
     	} else if (!strcmp(argv[i], "-server")) {
-    	    isServer = True;
+    	    IsServer = True;
 	} else if (!strcmp(argv[i], "-iconic") || !strcmp(argv[i], "-icon")) {
     	    iconic = True;
 	} else if (!strcmp(argv[i], "-noiconic")) {
@@ -416,13 +416,16 @@ int main(int argc, char **argv)
     	    langMode = argv[i];
 	} else if (!strcmp(argv[i], "-import")) {
 	    nextArg(argc, argv, &i); /* already processed, skip */
-    	} else if (*argv[i] == '-') {
+	} else if (!strcmp(argv[i], "-V") || !strcmp(argv[i], "-version")) {
+	    PrintVersion();
+	    exit(EXIT_SUCCESS);
+	} else if (*argv[i] == '-') {
 #ifdef VMS
 	    *argv[i] = '/';
 #endif /*VMS*/
     	    fprintf(stderr, "NEdit: Unrecognized option %s\n%s", argv[i],
     	    	    cmdLineHelp);
-    	    exit(0);
+    	    exit(EXIT_FAILURE);
     	} else {
 #ifdef VMS
 	    int numFiles, j;
@@ -471,7 +474,7 @@ int main(int argc, char **argv)
     
     /* If no file to edit was specified, open a window to edit "Untitled" */
     if (!fileSpecified) {
-    	EditNewFile(geometry, iconic, langMode);
+    	EditNewFile(geometry, iconic, langMode, NULL);
 	ReadMacroInitFile(WindowList);
 	if (toDoCommand != NULL)
 	    DoMacro(WindowList, toDoCommand, "-do macro");
@@ -481,17 +484,17 @@ int main(int argc, char **argv)
     AddLastCommandActionHook(context);
 
     /* Set up communication port and write ~/.nedit_server_process file */
-    if (isServer)
+    if (IsServer)
     	InitServerCommunication();
 
     /* Process events. */
-    if (isServer)
+    if (IsServer)
     	ServerMainLoop(context);
     else
     	XtAppMainLoop(context);
 
     /* Not reached but this keeps some picky compilers happy */
-    return 0;
+    return EXIT_SUCCESS;
 }
 
 static void nextArg(int argc, char **argv, int *argIndex)
@@ -502,7 +505,7 @@ static void nextArg(int argc, char **argv, int *argIndex)
 #endif /*VMS*/
     	fprintf(stderr, "NEdit: %s requires an argument\n%s", argv[*argIndex],
     	        cmdLineHelp);
-    	exit(0);
+    	exit(EXIT_FAILURE);
     }
     (*argIndex)++;
 }
@@ -510,7 +513,7 @@ static void nextArg(int argc, char **argv, int *argIndex)
 /*
 ** Return True if -do macro is valid, otherwise write an error on stderr
 */
-static int checkDoMacroArg(char *macro)
+static int checkDoMacroArg(const char *macro)
 {
     Program *prog;
     char *errMsg, *stoppedAt, *tMacro;
@@ -543,17 +546,21 @@ static int checkDoMacroArg(char *macro)
 ** it from consuming arguments which are meant to apply per-window, like
 ** -geometry and -iconic.
 */
-void maskArgvKeywords(int argc, char **argv, char **maskArgs)
+static void maskArgvKeywords(int argc, char **argv, const char **maskArgs)
 {
     int i, k;
+
     for (i=1; i<argc; i++)
 	for (k=0; maskArgs[k]!=NULL; k++)
 	    if (!strcmp(argv[i], maskArgs[k]))
     		argv[i][0] = ' ';
 }
-void unmaskArgvKeywords(int argc, char **argv, char **maskArgs)
+
+
+static void unmaskArgvKeywords(int argc, char **argv, const char **maskArgs)
 {
     int i, k;
+
     for (i=1; i<argc; i++)
 	for (k=0; maskArgs[k]!=NULL; k++)
 	    if (argv[i][0]==' ' && !strcmp(&argv[i][1], &maskArgs[k][1]))
