@@ -1,3 +1,4 @@
+/* $Id: nedit.h,v 1.17 2001/05/31 08:49:52 amai Exp $ */
 /*******************************************************************************
 *									       *
 * nedit.h -- Nirvana Editor common include file				       *
@@ -51,7 +52,12 @@
 
 enum indentStyle {NO_AUTO_INDENT, AUTO_INDENT, SMART_INDENT};
 enum wrapStyle {NO_WRAP, NEWLINE_WRAP, CONTINUOUS_WRAP};
-enum fileFormats {UNIX_FILE_FORMAT, DOS_FILE_FORMAT};
+enum fileFormats {UNIX_FILE_FORMAT, DOS_FILE_FORMAT, MAC_FILE_FORMAT};
+enum showMatchingStyle {NO_FLASH, FLASH_DELIMIT, FLASH_RANGE};
+
+#define NO_FLASH_STRING		"off"
+#define FLASH_DELIMIT_STRING	"delimiter"
+#define FLASH_RANGE_STRING	"range"
 
 #define CHARSET (XmStringCharSet)XmSTRING_DEFAULT_CHARSET
 
@@ -72,6 +78,31 @@ enum fileFormats {UNIX_FILE_FORMAT, DOS_FILE_FORMAT};
     XtGetValues(widget, args, 1); \
 }
 
+/* This handles all the different reasons files can be locked */
+#define USER_LOCKED_BIT     0
+#define PERM_LOCKED_BIT     1
+#define FORCE_LOCKED_BIT    2
+#define TOO_MUCH_BINARY_DATA_LOCKED_BIT 3
+
+#define LOCKED_BIT_TO_MASK(bitNum) (1 << (bitNum))
+#define SET_LOCKED_BY_REASON(reasons, onOrOff, reasonBit) ((onOrOff) ? \
+                    ((reasons) |= LOCKED_BIT_TO_MASK(reasonBit)) : \
+                    ((reasons) &= ~LOCKED_BIT_TO_MASK(reasonBit)))
+
+#define IS_USER_LOCKED(reasons) (((reasons) & LOCKED_BIT_TO_MASK(USER_LOCKED_BIT)) != 0)
+#define SET_USER_LOCKED(reasons, onOrOff) SET_LOCKED_BY_REASON(reasons, onOrOff, USER_LOCKED_BIT)
+#define IS_PERM_LOCKED(reasons) (((reasons) & LOCKED_BIT_TO_MASK(PERM_LOCKED_BIT)) != 0)
+#define SET_PERM_LOCKED(reasons, onOrOff) SET_LOCKED_BY_REASON(reasons, onOrOff, PERM_LOCKED_BIT)
+#define IS_FORCE_LOCKED(reasons) (((reasons) & LOCKED_BIT_TO_MASK(FORCE_LOCKED_BIT)) != 0)
+#define SET_FORCE_LOCKED(reasons, onOrOff) SET_LOCKED_BY_REASON(reasons, onOrOff, FORCE_LOCKED_BIT)
+#define IS_TMBD_LOCKED(reasons) (((reasons) & LOCKED_BIT_TO_MASK(TOO_MUCH_BINARY_DATA_LOCKED_BIT)) != 0)
+#define SET_TMBD_LOCKED(reasons, onOrOff) SET_LOCKED_BY_REASON(reasons, onOrOff, TOO_MUCH_BINARY_DATA_LOCKED_BIT)
+
+#define IS_ANY_LOCKED_IGNORING_USER(reasons) (((reasons) & ~LOCKED_BIT_TO_MASK(USER_LOCKED_BIT)) != 0)
+#define IS_ANY_LOCKED_IGNORING_PERM(reasons) (((reasons) & ~LOCKED_BIT_TO_MASK(PERM_LOCKED_BIT)) != 0)
+#define IS_ANY_LOCKED(reasons) ((reasons) != 0)
+#define CLEAR_ALL_LOCKS(reasons) ((reasons) = 0)
+
 /* Record on undo list */
 typedef struct _UndoInfo {
     struct _UndoInfo *next;		/* pointer to the next undo record */
@@ -107,35 +138,41 @@ typedef struct _WindowInfo {
     Widget	statsLine;		/* file stats information display */
     Widget  	iSearchForm;	    	/* incremental search line widgets */
     Widget  	iSearchText;
-    Widget  	iSearchREToggle;
+    Widget  	iSearchRegexToggle;
     Widget  	iSearchCaseToggle;
-    Widget  	iSearchLiteralToggle;
     Widget  	iSearchRevToggle;
     Widget	menuBar;    	    	/* the main menu bar */
     Widget	replaceDlog;		/* replace dialog */
     Widget	replaceText;		/* replace dialog settable widgets... */
     Widget	replaceWithText;
-    Widget	replaceLiteralBtn;
-    Widget	replaceCaseBtn;
-    Widget	replaceRegExpBtn;
-    Widget	replaceFwdBtn;
-    Widget	replaceRevBtn;
+    Widget    	replaceCaseToggle;
+    Widget	replaceWordToggle;    
+    Widget	replaceRegexToggle;    
+    Widget	replaceRevToggle;
     Widget	replaceKeepBtn;
     Widget	replaceBtns;
     Widget	replaceBtn;
+    Widget	replaceAllBtn;
+#ifndef REPLACE_SCOPE
+    Widget      replaceInWinBtn;
     Widget	replaceInSelBtn;
+#endif
     Widget	replaceSearchTypeBox;
+    Widget	replaceFindBtn;
+    Widget	replaceAndFindBtn;
     Widget	findDlog;		/* find dialog */
     Widget	findText;		/* find dialog settable widgets... */
-    Widget	findLiteralBtn;
-    Widget	findCaseBtn;
-    Widget	findRegExpBtn;
-    Widget	findFwdBtn;
-    Widget	findRevBtn;
+    Widget      findCaseToggle;
+    Widget      findWordToggle;    
+    Widget      findRegexToggle;    
+    Widget	findRevToggle;
     Widget	findKeepBtn;
     Widget	findBtns;
     Widget	findBtn;
     Widget	findSearchTypeBox;
+    Widget	replaceMultiFileDlog;	/* Replace in multiple files */
+    Widget	replaceMultiFileList;
+    Widget	replaceMultiFilePathBtn;
     Widget	fontDialog;		/* NULL, unless font dialog is up */
     Widget	readOnlyItem;		/* menu bar settable widgets... */
     Widget	autoSaveItem;
@@ -154,7 +191,14 @@ typedef struct _WindowInfo {
     Widget  	noWrapItem;
     Widget  	newlineWrapItem;
     Widget  	continuousWrapItem;
-    Widget  	highlightItem;
+    Widget	statsLineItem;
+    Widget	iSearchLineItem;
+    Widget	lineNumsItem;
+    Widget	showMatchingOffItem;
+    Widget	showMatchingDelimitItem;
+    Widget	showMatchingRangeItem;
+    Widget	overtypeModeItem;
+    Widget	highlightItem;
     Widget	windowMenuPane;
     Widget	shellMenuPane;
     Widget	macroMenuPane;
@@ -172,11 +216,15 @@ typedef struct _WindowInfo {
     Widget	noWrapDefItem;
     Widget	newlineWrapDefItem;
     Widget	contWrapDefItem;
-    Widget	showMatchingDefItem;
+    Widget	showMatchingOffDefItem;
+    Widget	showMatchingDelimitDefItem;
+    Widget	showMatchingRangeDefItem;
     Widget	highlightOffDefItem;
     Widget	highlightDefItem;
     Widget	searchDlogsDefItem;
+    Widget      beepOnSearchWrapDefItem;
     Widget	keepSearchDlogsDefItem;
+    Widget	searchWrapsDefItem;
     Widget	sortOpenPrevDefItem;
     Widget	allTagsDefItem;
     Widget	smartTagsDefItem;
@@ -184,11 +232,20 @@ typedef struct _WindowInfo {
     Widget	statsLineDefItem;
     Widget	iSearchLineDefItem;
     Widget	lineNumsDefItem;
+    Widget	pathInWindowsMenuDefItem;
     Widget  	modWarnDefItem;
     Widget  	exitWarnDefItem;
     Widget	searchLiteralDefItem;
     Widget	searchCaseSenseDefItem;
+    Widget	searchLiteralWordDefItem;
+    Widget	searchCaseSenseWordDefItem;
+    Widget	searchRegexNoCaseDefItem;
     Widget	searchRegexDefItem;
+#ifdef REPLACE_SCOPE
+    Widget	replScopeWinDefItem;
+    Widget	replScopeSelDefItem;
+    Widget	replScopeSmartDefItem;
+#endif
     Widget	size24x80DefItem;
     Widget	size40x80DefItem;
     Widget	size60x80DefItem;
@@ -246,18 +303,20 @@ typedef struct _WindowInfo {
     					   of selection related menu items */
     Boolean	filenameSet;		/* is the window still "Untitled"? */ 
     Boolean	fileChanged;		/* has window been modified? */
-    Boolean	readOnly;		/* is current file read only? */
-    Boolean	lockWrite;		/* is Read Only selected in menu? */
+    int         lockReasons;            /* all ways a file can be locked */
     Boolean	autoSave;		/* is autosave turned on? */
     Boolean	saveOldVersion;		/* keep old version in filename.bck */
     char	indentStyle;		/* whether/how to auto indent */
     char	wrapMode;		/* line wrap style: NO_WRAP,
     	    	    	    	    	   NEWLINE_WRAP or CONTINUOUS_WRAP */
     Boolean	overstrike;		/* is overstrike mode turned on ? */
-    Boolean	showMatching;		/* is paren matching mode on? */
+    char 	showMatchingStyle; 	/* How to show matching parens:
+					   NO_FLASH, FLASH_DELIMIT, or
+					   FLASH_RANGE */
     Boolean	showStats;		/* is stats line supposed to be shown */
     Boolean 	showISearchLine;    	/* is incr. search line to be shown */
     Boolean 	showLineNumbers;    	/* is the line number display shown */
+    Boolean 	showPathInWindowsMenu; 	/* is the path shown in windows menu */
     Boolean	highlightSyntax;	/* is syntax highlighting turned on? */
     Boolean	modeMessageDisplayed;	/* special stats line banner for learn
     					   and shell command executing modes */
@@ -267,6 +326,7 @@ typedef struct _WindowInfo {
     int		rHistIndex, fHistIndex;	/* history placeholders for */
     int     	iSearchHistIndex;	/*   find and replace dialogs */
     int     	iSearchStartPos;    	/* start pos. of current incr. search */
+    int       	iSearchLastBeginPos;    /* beg. pos. last match of current i.s.*/
     int     	nMarks;     	    	/* number of active bookmarks */
     XtIntervalId markTimeoutID;	    	/* backup timer for mark event handler*/
     Bookmark	markTable[MAX_MARKS];	/* marked locations in window */
@@ -277,8 +337,33 @@ typedef struct _WindowInfo {
     void    	*smartIndentData;   	/* compiled macros for smart indent */
     int    	languageMode;	    	/* identifies language mode currently
     	    	    	    	    	   selected in the window */
+    Boolean	multiFileReplSelected;	/* selected during last multi-window 
+					   replacement operation (history) */
+    struct _WindowInfo**		/* temporary list of writable windows */
+		writableWindows;	/* used during multi-file replacements */
+    int		nWritableWindows;	/* number of elements in the list */
+    Bool 	multiFileBusy;		/* suppresses multiple beeps/dialogs
+					   during multi-file replacements */
+    Bool 	replaceFailed;		/* flags replacements failures during
+					   multi-file replacements */
+    Bool	replaceLastRegexCase;   /* last state of the case sense button
+                                           in regex mode for replace dialog */
+    Bool	replaceLastLiteralCase; /* idem, for literal mode */
+    Bool	iSearchLastRegexCase;   /* idem, for regex mode in 
+                                           incremental search bar */
+    Bool	iSearchLastLiteralCase; /* idem, for literal mode */
+    Bool	findLastRegexCase; 	/* idem, for regex mode in find dialog */
+    Bool	findLastLiteralCase;    /* idem, for literal mode */
+    
+#ifdef REPLACE_SCOPE
+    int		replaceScope;		/* Current scope for replace dialog */
+    Widget	replaceScopeWinToggle;	/* Scope for replace = window */
+    Widget	replaceScopeSelToggle;	/* Scope for replace = selection */
+    Widget	replaceScopeMultiToggle;/* Scope for replace = multiple files */
+#endif
 } WindowInfo;
 
 extern WindowInfo *WindowList;
 extern Display *TheDisplay;
 extern char *ArgV0;
+extern Boolean IsServer;
