@@ -1,3 +1,4 @@
+static const char CVSID[] = "$Id: server.c,v 1.9 2001/08/14 08:37:16 jlous Exp $";
 /*******************************************************************************
 *									       *
 * server.c -- Nirvana Editor edit-server component			       *
@@ -11,7 +12,7 @@
 * 									       *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
-* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
 * for more details.							       *
 * 									       *
 * You should have received a copy of the GNU General Public License along with *
@@ -34,9 +35,11 @@
 #include "../util/VMSparam.h"
 #include "../util/VMSutils.h"
 #else
-#include <sys/utsname.h>
-#include <sys/param.h>
 #include <sys/types.h>
+#include <sys/utsname.h>
+#ifndef __MVS__
+#include <sys/param.h>
+#endif
 #include <unistd.h>
 #include <pwd.h>
 #endif
@@ -66,8 +69,8 @@
 
 static void processServerCommand(void);
 static void cleanUpServerCommunication(void);
-static char *getUserName(void);
-static void getHostName(char *hostname);
+static const char *getUserName(void);
+static const char *getHostName(void);
 static void processServerCommandString(char *string);
 
 static Atom ServerRequestAtom = 0;
@@ -80,15 +83,15 @@ static Atom ServerExistsAtom = 0;
 void InitServerCommunication(void)
 {
     Window rootWindow = RootWindow(TheDisplay, DefaultScreen(TheDisplay));
-    char *userName, propName[24+MAXNODENAMELEN+MAXUSERNAMELEN+MAXPATHLEN];
-    char hostName[MAXNODENAMELEN+1];
+    const char *userName, *hostName;
+    char propName[24+MAXNODENAMELEN+MAXUSERNAMELEN+MAXPATHLEN];
     char serverName[MAXPATHLEN+2];
 
     /* Create server property atoms.  Atom names are generated by
        concatenating NEDIT_SERVER_REQUEST_, and NEDIT_SERVER_EXITS_
        with hostname and user name. */
     userName = getUserName();
-    getHostName(hostName);
+    hostName = getHostName();
     if (GetPrefServerName()[0] != '\0') {
     	serverName[0] = '_';
     	strcpy(&serverName[1], GetPrefServerName());
@@ -179,7 +182,7 @@ static void processServerCommand(void)
 ** Return a pointer to the username of the current user in a statically
 ** allocated string.
 */
-static char *getUserName(void)
+static const char *getUserName(void)
 {
 #ifdef VMS
     return cuserid(NULL);
@@ -191,7 +194,12 @@ static char *getUserName(void)
        results in the user-name of the original terminal being used, which is
        not correct when the user uses the su command.  Now, getpwuid only: */
     struct passwd *passwdEntry;
+
     passwdEntry = getpwuid(getuid());
+    if (!passwdEntry) {
+       perror("NEdit: getpwuid() failed ");
+       exit(EXIT_FAILURE);
+    }
     return passwdEntry->pw_name;
 #endif
 }
@@ -199,7 +207,7 @@ static char *getUserName(void)
 /*
 ** Writes the hostname of the current system in string "hostname".
 */
-static void getHostName(char *hostname)
+static const char *getHostName(void)
 {
 #ifdef VMS
     /* This should be simple, but uname is not supported in the DEC C RTL and
@@ -211,6 +219,7 @@ static void getHostName(char *hostname)
     unsigned long int syiItemCode = SYI$_NODENAME;	/* get Nodename */
     unsigned long int unused = 0;
     unsigned short int hostnameLen = MAXNODENAMELEN+1;
+    static char hostname[MAXNODENAMELEN+1];
     
     hostnameDesc = NulStrWrtDesc(hostname, MAXNODENAMELEN+1);
     syi_status = lib$getsyi(&syiItemCode, &unused, hostnameDesc, &hostnameLen,
@@ -221,11 +230,19 @@ static void getHostName(char *hostname)
     } else
     	hostname[hostnameLen] = '\0';
     FreeStrDesc(hostnameDesc);
+    return hostname;
 #else
     struct utsname nameStruct;
-
-    uname(&nameStruct);
+    static char hostname[MAXNODENAMELEN+1];
+    int rc;
+    
+    rc=uname(&nameStruct);
+    if (rc<0) {
+       perror("NEdit: uname() failed ");
+       exit(EXIT_FAILURE);
+    }   
     strcpy(hostname, nameStruct.nodename);
+    return hostname;
 #endif
 }
 
@@ -245,7 +262,7 @@ static void processServerCommandString(char *string)
     	    if (!window->filenameSet && !window->fileChanged)
     	    	break;
     	if (window == NULL) {
-    	    EditNewFile(NULL, False, NULL);
+    	    EditNewFile(NULL, False, NULL, NULL);
     	    CheckCloseDim();
     	} else
     	    XMapRaised(TheDisplay, XtWindow(window->shell));
