@@ -1,32 +1,36 @@
+static const char CVSID[] = "$Id: shift.c,v 1.8 2001/08/14 08:37:16 jlous Exp $";
 /*******************************************************************************
 *									       *
 * shift.c -- Nirvana Editor built-in filter commands			       *
 *									       *
-* Copyright (c) 1991 Universities Research Association, Inc.		       *
-* All rights reserved.							       *
+* Copyright (C) 1999 Mark Edel						       *
+*									       *
+* This is free software; you can redistribute it and/or modify it under the    *
+* terms of the GNU General Public License as published by the Free Software    *
+* Foundation; either version 2 of the License, or (at your option) any later   *
+* version.							               *
 * 									       *
-* This material resulted from work developed under a Government Contract and   *
-* is subject to the following license:  The Government retains a paid-up,      *
-* nonexclusive, irrevocable worldwide license to reproduce, prepare derivative *
-* works, perform publicly and display publicly by or for the Government,       *
-* including the right to distribute to other Government contractors.  Neither  *
-* the United States nor the United States Department of Energy, nor any of     *
-* their employees, makes any warrenty, express or implied, or assumes any      *
-* legal liability or responsibility for the accuracy, completeness, or         *
-* usefulness of any information, apparatus, product, or process disclosed, or  *
-* represents that its use would not infringe privately owned rights.           *
-*                                        				       *
-* Fermilab Nirvana GUI Library						       *
+* This software is distributed in the hope that it will be useful, but WITHOUT *
+* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
+* for more details.							       *
+* 									       *
+* You should have received a copy of the GNU General Public License along with *
+* software; if not, write to the Free Software Foundation, Inc., 59 Temple     *
+* Place, Suite 330, Boston, MA  02111-1307 USA		                       *
+*									       *
+* Nirvana Text Editor	    						       *
 * June 18, 1991								       *
 *									       *
 * Written by Mark Edel							       *
 *									       *
 *******************************************************************************/
-static char SCCSID[] = "@(#)shift.c	1.10     8/3/94";
 #ifdef VMS
 #include "../util/VMSparam.h"
 #else
+#ifndef __MVS__
 #include <sys/param.h>
+#endif
 #endif /*VMS*/
 #include <limits.h>
 #include <ctype.h>
@@ -38,17 +42,19 @@ static char SCCSID[] = "@(#)shift.c	1.10     8/3/94";
 #include "shift.h"
 #include "window.h"
 
-static char *shiftText(char *text, int direction, int tabsAllowed, int tabDist,
-	int nChars, int *newLen);
 static void shiftRect(WindowInfo *window, int direction, int byTab,
 	int selStart, int selEnd, int rectStart, int rectEnd);
 static void changeCase(WindowInfo *window, int makeUpper);
-static void shiftLineRight(char *line, int tabsAllowed, int tabDist,
-	int nChars);
-static void shiftLineLeft(char *line, int tabDist, int nChars);
-static int findLeftMargin(char *text, int tabDist);
-static char *fillParagraph(char *text, int addIndent, int width,
-	int startCol, int tabDist, int allowTabs, int *filledLen);
+static char *shiftLineRight(char *line, int lineLen, int tabsAllowed,
+	int tabDist, int nChars);
+static char *shiftLineLeft(char *line, int lineLen, int tabDist, int nChars);
+static int findLeftMargin(char *text, int length, int tabDist);
+static char *fillParagraphs(char *text, int rightMargin, int tabDist,
+	int useTabs, char nullSubsChar, int *filledLen, int alignWithFirst);
+static char *fillParagraph(char *text, int leftMargin, int firstLineIndent,
+	int rightMargin, int tabDist, int allowTabs, char nullSubsChar,
+	int *filledLen);
+char *makeIndentString(int indent, int tabDist, int allowTabs, int *nChars);
 static int atTabStop(int pos, int tabDist);
 static int nextTab(int pos, int tabDist);
 static int countLines(char *text);
@@ -99,11 +105,11 @@ void ShiftSelection(WindowInfo *window, int direction, int byTab)
     
     /* shift the text by the appropriate distance */
     if (byTab) {
-    	XtVaGetValues(window->textArea, textNemulateTabs, &emTabDist, 0);
+    	XtVaGetValues(window->textArea, textNemulateTabs, &emTabDist, NULL);
     	shiftDist = emTabDist == 0 ? buf->tabDist : emTabDist;
     } else
     	shiftDist = 1;
-    shiftedText = shiftText(text, direction, buf->useTabs, buf->tabDist,
+    shiftedText = ShiftText(text, direction, buf->useTabs, buf->tabDist,
     	    shiftDist, &shiftedLen);
     XtFree(text);
     BufReplaceSelected(buf, shiftedText);
@@ -126,7 +132,7 @@ static void shiftRect(WindowInfo *window, int direction, int byTab,
     
     /* Calculate the the left/right offset for the new rectangle */
     if (byTab) {
-    	XtVaGetValues(window->textArea, textNemulateTabs, &emTabDist, 0);
+    	XtVaGetValues(window->textArea, textNemulateTabs, &emTabDist, NULL);
     	offset = emTabDist == 0 ? buf->tabDist : emTabDist;
     } else
     	offset = 1;
@@ -188,12 +194,14 @@ static void changeCase(WindowInfo *window, int makeUpper)
     	    return;
 	}
     	*bufChar = BufGetCharacter(buf, cursorPos-1);
-    	*bufChar = makeUpper ? toupper(*bufChar) : tolower(*bufChar);
+    	*bufChar = makeUpper ? toupper((unsigned char)*bufChar) :
+		tolower((unsigned char)*bufChar);
     	BufReplace(buf, cursorPos-1, cursorPos, bufChar);
     } else {
 	text = BufGetSelectionText(buf);
 	for (c=text; *c!='\0'; c++)
-    	    *c = makeUpper ? toupper(*c) : tolower(*c);
+    	    *c = makeUpper ? toupper((unsigned char)*c) :
+		    tolower((unsigned char)*c);
 	BufReplaceSelected(buf, text);
 	XtFree(text);
 	if (isRect)
@@ -206,9 +214,9 @@ static void changeCase(WindowInfo *window, int makeUpper)
 void FillSelection(WindowInfo *window)
 {
     textBuffer *buf = window->buffer;
-    char *c, *text, *filledText;
-    int left, right, nCols, leftMargin, len, isRect, rectStart, rectEnd;
-    int rightMargin, wrapMargin, nLines;
+    char *text, *filledText;
+    int left, right, nCols, len, isRect, rectStart, rectEnd;
+    int rightMargin, wrapMargin;
     int insertPos = TextGetCursorPos(window->lastFocus);
     int hasSelection = window->buffer->primary.selected;
     
@@ -227,7 +235,6 @@ void FillSelection(WindowInfo *window)
     } else if (isRect) {
     	left = BufStartOfLine(buf, left);
     	right = BufEndOfLine(buf, right);
-    	nLines = BufCountLines(buf, left, right) + 1;
     	text = BufGetTextInRect(buf, left, right, rectStart, INT_MAX);
     } else {
 	left = BufStartOfLine(buf, left);
@@ -239,33 +246,22 @@ void FillSelection(WindowInfo *window)
     	BufSelect(buf, left, right);
     	text = BufGetRange(buf, left, right);
     }
-
-    /* Find the margins either as specified in the rectangular selection, or
+    
+    /* Find right margin either as specified in the rectangular selection, or
        by measuring the text and querying the window's wrap margin (or width) */
     if (hasSelection && isRect) {
-    	leftMargin = 0;
     	rightMargin = rectEnd - rectStart;
     } else {
-    	leftMargin = findLeftMargin(text, buf->tabDist);
 	XtVaGetValues(window->textArea, textNcolumns, &nCols,
-    		textNwrapMargin, &wrapMargin, 0);
+    		textNwrapMargin, &wrapMargin, NULL);
 	rightMargin = (wrapMargin == 0 ? nCols : wrapMargin) - 1;
     }
     
     /* Fill the text */
-    filledText = fillParagraph(text, leftMargin, rightMargin-leftMargin,
-    	    (hasSelection && isRect) ? rectStart : 0, buf->tabDist, 
-    	    buf->useTabs, &len);
+    filledText = fillParagraphs(text, rightMargin, buf->tabDist, buf->useTabs,
+	    buf->nullSubsChar, &len, False);
     XtFree(text);
-    
-    /* If this is a rectangular fill, trim the trailing newlines */
-    if (hasSelection && isRect) {
-    	for (c=&filledText[len-1]; c>=filledText && *c=='\n'; c--) {
-    	    *c = '\0';
-    	    len--;
-    	}
-    }
-    
+        
     /* Replace the text in the window */
     if (hasSelection && isRect) {
         BufReplaceRect(buf, left, right, rectStart, INT_MAX, filledText);
@@ -292,12 +288,11 @@ void FillSelection(WindowInfo *window)
 ** shift lines left and right in a multi-line text string.  Returns the
 ** shifted text in memory that must be freed by the caller with XtFree.
 */
-static char *shiftText(char *text, int direction, int tabsAllowed, int tabDist,
+char *ShiftText(char *text, int direction, int tabsAllowed, int tabDist,
 	int nChars, int *newLen)
 {
-    char line[MAX_LINE_LENGTH];
-    char *shiftedText;
-    char *textPtr, *linePtr, *shiftedPtr;
+    char *shiftedText, *shiftedLine;
+    char *textPtr, *lineStartPtr, *shiftedPtr;
     int bufLen;
     
     /*
@@ -312,20 +307,21 @@ static char *shiftText(char *text, int direction, int tabsAllowed, int tabDist,
     shiftedText = (char *)XtMalloc(bufLen + 1);
     
     /*
-    ** break into lines and call shiftLine on each
+    ** break into lines and call shiftLine(Left/Right) on each
     */
-    linePtr = line;
+    lineStartPtr = text;
     textPtr = text;
     shiftedPtr = shiftedText;
     while (TRUE) {
 	if (*textPtr=='\n' || *textPtr=='\0') {
-	    *linePtr = '\0';
-	    if (direction == SHIFT_RIGHT)
-		shiftLineRight(line, tabsAllowed, tabDist, nChars);
-	    else
-		shiftLineLeft(line, tabDist, nChars);
-	    strcpy(shiftedPtr, line);
-	    shiftedPtr += strlen(line);
+	    shiftedLine = (direction == SHIFT_RIGHT) ?
+		    shiftLineRight(lineStartPtr, textPtr-lineStartPtr,
+		        tabsAllowed, tabDist, nChars) :
+	    	    shiftLineLeft(lineStartPtr, textPtr-lineStartPtr, tabDist,
+			nChars);
+	    strcpy(shiftedPtr, shiftedLine);
+	    shiftedPtr += strlen(shiftedLine);
+	    XtFree(shiftedLine);
 	    if (*textPtr == '\0') {
 	        /* terminate string & exit loop at end of text */
 	    	*shiftedPtr = '\0';
@@ -335,31 +331,30 @@ static char *shiftText(char *text, int direction, int tabsAllowed, int tabDist,
 		*shiftedPtr++ = *textPtr++;
 	    }
 	    /* start line over */
-	    linePtr = line;
-	} else {
-	    /* build up line */
-	    *linePtr++ = *textPtr++;
-	}
+	    lineStartPtr = textPtr;
+	} else
+	    textPtr++;
     }
     *newLen = shiftedPtr - shiftedText;
     return shiftedText;
 }
 
-static void shiftLineRight(char *line, int tabsAllowed, int tabDist, int nChars)
+static char *shiftLineRight(char *line, int lineLen, int tabsAllowed,
+	int tabDist, int nChars)
 {
-    char lineIn[MAX_LINE_LENGTH];
+    char *lineOut;
     char *lineInPtr, *lineOutPtr;
     int whiteWidth, i;
     
-    strcpy(lineIn, line);
-    lineInPtr = lineIn;
-    lineOutPtr = line;
+    lineInPtr = line;
+    lineOut = XtMalloc(lineLen + nChars + 1);
+    lineOutPtr = lineOut;
     whiteWidth = 0;
     while (TRUE) {
-        if (*lineInPtr == '\0') {
+        if (*lineInPtr == '\0' || (lineInPtr - line) >= lineLen) {
 	    /* nothing on line, wipe it out */
-	    *line = '\0';
-	    return;
+	    *lineOut = '\0';
+	    return lineOut;
         } else if (*lineInPtr == ' ') {
 	    /* white space continues with tab, advance to next tab stop */
 	    whiteWidth++;
@@ -380,30 +375,30 @@ static void shiftLineRight(char *line, int tabsAllowed, int tabDist, int nChars)
 		}
 	    }
 	    /* move remainder of line */
-    	    while (*lineInPtr!='\0')
+    	    while (*lineInPtr!='\0' && (lineInPtr - line) < lineLen)
 		*lineOutPtr++ = *lineInPtr++;
 	    *lineOutPtr = '\0';
-	    return;
+	    return lineOut;
 	}
     }
 }
 
-static void shiftLineLeft(char *line, int tabDist, int nChars)
+static char *shiftLineLeft(char *line, int lineLen, int tabDist, int nChars)
 {
-    char lineIn[MAX_LINE_LENGTH];
+    char *lineOut;
     int i, whiteWidth, lastWhiteWidth, whiteGoal;
     char *lineInPtr, *lineOutPtr;
     
-    strcpy(lineIn, line);
-    lineInPtr = lineIn;
-    lineOutPtr = line;
+    lineInPtr = line;
+    lineOut = XtMalloc(lineLen + tabDist + 1);
+    lineOutPtr = lineOut;
     whiteWidth = 0;
     lastWhiteWidth = 0;
     while (TRUE) {
-        if (*lineInPtr == '\0') {
+        if (*lineInPtr == '\0' || (lineInPtr - line) >= lineLen) {
 	    /* nothing on line, wipe it out */
-	    *line = '\0';
-	    return;
+	    *lineOut = '\0';
+	    return lineOut;
         } else if (*lineInPtr == ' ') {
 	    /* white space continues with space, advance one character */
 	    whiteWidth++;
@@ -417,7 +412,7 @@ static void shiftLineLeft(char *line, int tabDist, int nChars)
 	} else {
 	    /* end of white space, remove nChars characters */
 	    for (i=1; i<=nChars; i++) {
-		if (lineOutPtr != line) {
+		if (lineOutPtr > lineOut) {
 		    if (*(lineOutPtr-1) == ' ') {
 			/* end of white space is a space, just remove it */
 			lineOutPtr--;
@@ -435,11 +430,11 @@ static void shiftLineLeft(char *line, int tabDist, int nChars)
 		}
 	    }
 	    /* move remainder of line */
-    	    while (*lineInPtr!='\0')
+    	    while (*lineInPtr!='\0' && (lineInPtr - line) < lineLen)
 		*lineOutPtr++ = *lineInPtr++;
 	    /* add a null */
 	    *lineOutPtr = '\0';
-	    return;
+	    return lineOut;
 	}
     }
 }
@@ -468,21 +463,23 @@ static int countLines(char *text)
 
 /*
 ** Find the implied left margin of a text string (the number of columns to the
-** first non-whitespace character on any line).
+** first non-whitespace character on any line) up to either the terminating
+** null character at the end of the string, or "length" characters, whever
+** comes first.
 */
-static int findLeftMargin(char *text, int tabDist)
+static int findLeftMargin(char *text, int length, int tabDist)
 {
     char *c;
     int col = 0, leftMargin = INT_MAX;
     int inMargin = True;
     
-    for (c=text; *c!='\0'; c++) {
+    for (c=text; *c!='\0' && c-text<length; c++) {
     	if (*c == '\t') {
-    	    col += tabDist - (col % tabDist);
+    	    col += BufCharWidth('\t', col, tabDist, '\0');
     	} else if (*c == ' ') {
     	    col++;
     	} else if (*c == '\n') {
-    	    col = 0;
+	    col = 0;
     	    inMargin = True;
     	} else {
     	    /* non-whitespace */
@@ -492,7 +489,7 @@ static int findLeftMargin(char *text, int tabDist)
     	}
     }
     
-    /* if no text is found, the leftMargin will never be set */
+    /* if no non-white text is found, the leftMargin will never be set */
     if (leftMargin == INT_MAX)
     	return 0;
     
@@ -500,14 +497,96 @@ static int findLeftMargin(char *text, int tabDist)
 }
 
 /*
-** Trim leading space, arrange text to fill width "width", and add back
-** "addIndent"
+** Fill multiple paragraphs between rightMargin and an implied left margin
+** and first line indent determined by analyzing the text.  alignWithFirst
+** aligns subsequent paragraphs with the margins of the first paragraph (a
+** capability not currently used in NEdit, but carried over from code for
+** previous versions which did all paragraphs together).
 */
-static char *fillParagraph(char *text, int addIndent, int width,
-	int startCol, int tabDist, int allowTabs, int *filledLen)
+static char *fillParagraphs(char *text, int rightMargin, int tabDist,
+	int useTabs, char nullSubsChar, int *filledLen, int alignWithFirst)
 {
-    char *cleanedText, *outText, *indentString, *outPtr, *c, *b;
-    int i, col, cleanedLen, indentLen, nLines = 1;
+    int paraStart, paraEnd, fillEnd;
+    char *c, ch, *secondLineStart, *paraText, *filledText;
+    int firstLineLen, firstLineIndent, leftMargin, len;
+    textBuffer *buf;
+    
+    /* Create a buffer to accumulate the filled paragraphs */
+    buf = BufCreate();
+    BufSetAll(buf, text);
+    
+    /*
+    ** Loop over paragraphs, filling each one, and accumulating the results
+    ** in buf
+    */
+    paraStart = 0;
+    for (;;) {
+	
+	/* Skip over white space */
+	while (paraStart < buf->length) {
+	    ch = BufGetCharacter(buf, paraStart);
+	    if (ch != ' ' && ch != '\t' && ch != '\n')
+	    	break;
+	    paraStart++;
+	}
+	if (paraStart >= buf->length)
+	    break;
+	paraStart = BufStartOfLine(buf, paraStart);
+	
+	/* Find the end of the paragraph */
+	paraEnd = findParagraphEnd(buf, paraStart);
+	
+	/* Operate on either the one paragraph, or to make them all identical,
+	   do all of them together (fill paragraph can format all the paragraphs
+	   it finds with identical specs if it gets passed more than one) */
+	fillEnd = alignWithFirst ? buf->length :  paraEnd;
+
+	/* Get the paragraph in a text string (or all of the paragraphs if
+	   we're making them all the same) */
+	paraText = BufGetRange(buf, paraStart, fillEnd);
+	
+	/* Find separate left margins for the first and for the first line of
+	   the paragraph, and for rest of the remainder of the paragraph */
+	for (c=paraText ; *c!='\0' && *c!='\n'; c++);
+	firstLineLen = c - paraText;
+	secondLineStart = *c == '\0' ? paraText : c + 1;
+	firstLineIndent = findLeftMargin(paraText, firstLineLen, tabDist);
+	leftMargin = findLeftMargin(secondLineStart, paraEnd - paraStart -
+		(secondLineStart - paraText), tabDist);
+
+	/* Fill the paragraph */
+	filledText = fillParagraph(paraText, leftMargin, firstLineIndent,
+		rightMargin, tabDist, useTabs, nullSubsChar, &len);
+	XtFree(paraText);
+	
+	/* Replace it in the buffer */
+	BufReplace(buf, paraStart, fillEnd, filledText);
+	XtFree(filledText);
+	
+	/* move on to the next paragraph */
+	paraStart += len;
+    }
+    
+    /* Free the buffer and return its contents */
+    filledText = BufGetAll(buf);
+    *filledLen = buf->length;
+    BufFree(buf);
+    return filledText;
+}
+
+/*
+** Trim leading space, and arrange text to fill between leftMargin and
+** rightMargin (except for the first line which fills from firstLineIndent),
+** re-creating whitespace to the left of the text using tabs (if allowTabs is
+** True) calculated using tabDist, and spaces.  Returns a newly allocated
+** string as the function result, and the length of the new string in filledLen.
+*/
+static char *fillParagraph(char *text, int leftMargin, int firstLineIndent,
+	int rightMargin, int tabDist, int allowTabs, char nullSubsChar,
+	int *filledLen)
+{
+    char *cleanedText, *outText, *indentString, *leadIndentStr, *outPtr, *c, *b;
+    int col, cleanedLen, indentLen, leadIndentLen, nLines = 1;
     int inWhitespace, inMargin;
     
     /* remove leading spaces, convert newlines to spaces */
@@ -541,20 +620,20 @@ static char *fillParagraph(char *text, int addIndent, int width,
        Algorithm: scan through characters, counting columns, and when the
        margin width is exceeded, search backward for beginning of the word
        and convert the last whitespace character into a newline */
-    col = startCol;
+    col = firstLineIndent;
     for (c=cleanedText; *c!='\0'; c++) {
     	if (*c == '\n')
-    	    col = startCol;
+    	    col = leftMargin;
     	else
-    	    col += BufCharWidth(*c, col, tabDist);
-    	if (col-startCol-1 > width) {
+    	    col += BufCharWidth(*c, col, tabDist, nullSubsChar);
+    	if (col-1 > rightMargin) {
     	    inWhitespace = True;
     	    for (b=c; b>=cleanedText && *b!='\n'; b--) {
     	    	if (*b == '\t' || *b == ' ') {
     	    	    if (!inWhitespace) {
     	    		*b = '\n';
     	    		c = b;
-    	    		col = startCol;
+    	    		col = leftMargin;
      	    		nLines++;
    	    		break;
     	    	    }
@@ -564,29 +643,20 @@ static char *fillParagraph(char *text, int addIndent, int width,
     	}
     }
     nLines++;
-    
+
     /* produce a string to prepend to lines to indent them to the left margin */
-    indentString = XtMalloc(sizeof(char) * addIndent + 1);
-    outPtr = indentString;
-    if (allowTabs) {
-	for (i=0; i<addIndent/tabDist; i++)
-    	    *outPtr++ = '\t';
-	for (i=0; i<addIndent%tabDist; i++)
-    	    *outPtr++ = ' ';
-    } else {
-    	for (i=0; i<addIndent; i++)
-    	    *outPtr++ = ' ';
-    }
-    indentLen = outPtr - indentString;
-    *outPtr = '\0';
-    
+    leadIndentStr = makeIndentString(firstLineIndent, tabDist,
+	    allowTabs, &leadIndentLen);
+    indentString = makeIndentString(leftMargin, tabDist, allowTabs, &indentLen);
+        
     /* allocate memory for the finished string */
-    outText = XtMalloc(sizeof(char) * (cleanedLen + indentLen * nLines + 1));
+    outText = XtMalloc(sizeof(char) * (cleanedLen + leadIndentLen +
+	    indentLen * (nLines-1) + 1));
     outPtr = outText;
     
     /* prepend the indent string to each line of the filled text */
-    strncpy(outPtr, indentString, indentLen);
-    outPtr += indentLen;
+    strncpy(outPtr, leadIndentStr, leadIndentLen);
+    outPtr += leadIndentLen;
     for (c=cleanedText; *c!='\0'; c++) {
     	*outPtr++ = *c;
     	if (*c == '\n') {
@@ -602,11 +672,32 @@ static char *fillParagraph(char *text, int addIndent, int width,
     
     /* clean up, return result */
     XtFree(cleanedText);
+    XtFree(leadIndentStr);
     XtFree(indentString);
     *filledLen = outPtr - outText;
     return outText;
 }
 
+char *makeIndentString(int indent, int tabDist, int allowTabs, int *nChars)
+{
+    char *indentString, *outPtr;
+    int i;
+    
+    outPtr = indentString = XtMalloc(sizeof(char) * indent + 1);
+    if (allowTabs) {
+	for (i=0; i<indent/tabDist; i++)
+    	    *outPtr++ = '\t';
+	for (i=0; i<indent%tabDist; i++)
+    	    *outPtr++ = ' ';
+    } else {
+    	for (i=0; i<indent; i++)
+    	    *outPtr++ = ' ';
+    }
+    *outPtr = '\0';
+    *nChars = outPtr - indentString;
+    return indentString;
+}
+
 /*
 ** Find the boundaries of the paragraph containing pos
 */
