@@ -1,3 +1,4 @@
+static const char CVSID[] = "$Id: smartIndent.c,v 1.12 2001/08/14 08:37:16 jlous Exp $";
 /*******************************************************************************
 *									       *
 * smartIndent.c -- Maintain, and allow user to edit, macros for smart indent   *
@@ -11,7 +12,7 @@
 * 									       *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
-* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
 * for more details.							       *
 * 									       *
 * You should have received a copy of the GNU General Public License along with *
@@ -25,12 +26,16 @@
 *									       *
 *******************************************************************************/
 #include <stdio.h>
+#include <string.h>
 #include <limits.h>
+
 #include <Xm/Xm.h>
 #ifdef VMS
 #include "../util/VMSparam.h"
 #else
+#ifndef __MVS__
 #include <sys/param.h>
+#endif
 #endif /*VMS*/
 #include <Xm/Xm.h>
 #include <Xm/Form.h>
@@ -40,12 +45,14 @@
 #include <Xm/RowColumn.h>
 #include <Xm/SeparatoG.h>
 #include <Xm/PanedW.h>
+
 #include "../util/DialogF.h"
 #include "../util/misc.h"
 #include "textBuf.h"
 #include "nedit.h"
 #include "text.h"
 #include "preferences.h"
+#include "rbTree.h"
 #include "interpret.h"
 #include "macro.h"
 #include "window.h"
@@ -65,7 +72,9 @@ typedef struct {
 
 typedef struct {
     Program *newlineMacro;
+    int inNewLineMacro;
     Program *modMacro;
+    int inModMacro;
 } windowSmartIndentData;
 
 /* Smart indent macros dialog information */
@@ -707,12 +716,16 @@ Preferences -> Language Mode.", "Dismiss", modeName);
     	    return;
 	initialized = True;
     }
-    if (!ReadMacroString(window, indentMacros->initMacro,
-	    "smart indent initialization macro"))
-    	return;
+    if (indentMacros->initMacro != NULL) {
+	if (!ReadMacroString(window, indentMacros->initMacro,
+    	    	"smart indent initialization macro"))
+    	    return;
+    }
     
     /* Compile the newline and modify macros and attach them to the window */
     winData = (windowSmartIndentData *)XtMalloc(sizeof(windowSmartIndentData));
+    winData->inNewLineMacro = 0;
+    winData->inModMacro = 0;
     winData->newlineMacro = ParseMacro(indentMacros->newlineMacro, &errMsg,
     	    &stoppedAt);
     if (winData->newlineMacro == NULL) {
@@ -785,6 +798,7 @@ static void executeNewlineMacro(WindowInfo *window, smartIndentCBStruct *cbInfo)
 {
     windowSmartIndentData *winData =
     	    (windowSmartIndentData *)window->smartIndentData;
+    /* posValue probably shouldn't be static due to re-entrance issues <slobasso> */
     static DataValue posValue = {INT_TAG, {0}};
     DataValue result;
     RestartData *continuation;
@@ -793,6 +807,7 @@ static void executeNewlineMacro(WindowInfo *window, smartIndentCBStruct *cbInfo)
    
     /* Call newline macro with the position at which to add newline/indent */
     posValue.val.n = cbInfo->pos;
+    ++(winData->inNewLineMacro);
     stat = ExecuteMacro(window, winData->newlineMacro, 1, &posValue, &result,
     	    &continuation, &errMsg);
     
@@ -800,6 +815,7 @@ static void executeNewlineMacro(WindowInfo *window, smartIndentCBStruct *cbInfo)
     while (stat == MACRO_TIME_LIMIT)
     	stat = ContinueMacro(continuation, &result, &errMsg);
     
+    --(winData->inNewLineMacro);
     /* Collect Garbage.  Note that the mod macro does not collect garbage,
        (because collecting per-line is more efficient than per-character)
        but GC now depends on the newline macro being mandatory */
@@ -825,6 +841,14 @@ static void executeNewlineMacro(WindowInfo *window, smartIndentCBStruct *cbInfo)
     cbInfo->indentRequest = result.val.n;
 }
 
+
+Boolean InSmartIndentMacros(WindowInfo *window) {
+    windowSmartIndentData *winData =
+    	    (windowSmartIndentData *)window->smartIndentData;
+
+	return((winData && (winData->inModMacro || winData->inModMacro)));
+}
+
 /*
 ** Run the modification macro with information from the smart-indent callback
 ** structure passed by the widget
@@ -833,7 +857,9 @@ static void executeModMacro(WindowInfo *window,smartIndentCBStruct *cbInfo)
 {
     windowSmartIndentData *winData =
     	    (windowSmartIndentData *)window->smartIndentData;
+    /* args probably shouldn't be static due to future re-entrance issues <slobasso> */
     static DataValue args[2] = {{INT_TAG, {0}}, {STRING_TAG, {0}}};
+    /* after 5.2 release remove inModCB and use new winData->inModMacro value */
     static int inModCB = False;
     DataValue result;
     RestartData *continuation;
@@ -851,11 +877,16 @@ static void executeModMacro(WindowInfo *window,smartIndentCBStruct *cbInfo)
     args[0].val.n = cbInfo->pos;
     args[1].val.str = AllocString(strlen(cbInfo->charsTyped) + 1);
     strcpy(args[1].val.str, cbInfo->charsTyped);
+
     inModCB = True;
-	stat = ExecuteMacro(window, winData->modMacro, 3, args, &result,
-    		&continuation, &errMsg);
-	while (stat == MACRO_TIME_LIMIT)
-    	    stat = ContinueMacro(continuation, &result, &errMsg);
+    ++(winData->inModMacro);
+
+    stat = ExecuteMacro(window, winData->modMacro, 3, args, &result,
+        &continuation, &errMsg);
+    while (stat == MACRO_TIME_LIMIT)
+        stat = ContinueMacro(continuation, &result, &errMsg);
+
+    --(winData->inModMacro);
     inModCB = False;
     
     /* Process errors in macro execution */
@@ -904,12 +935,12 @@ void EditSmartIndentMacros(WindowInfo *window)
     XtSetArg(args[n], XmNdeleteResponse, XmDO_NOTHING); n++;
     XtSetArg(args[n], XmNiconName, "Smart Indent Macros"); n++;
     XtSetArg(args[n], XmNtitle, "Smart Indent Macros"); n++;
-    SmartIndentDialog.shell = XtAppCreateShell(APP_NAME, APP_CLASS,
+    SmartIndentDialog.shell = CreateShellWithBestVis(APP_NAME, APP_CLASS,
 	    applicationShellWidgetClass, TheDisplay, args, n);
     AddSmallIcon(SmartIndentDialog.shell);
     form = XtVaCreateManagedWidget("editSmartIndentMacros", xmFormWidgetClass,
 	    SmartIndentDialog.shell, XmNautoUnmanage, False,
-	    XmNresizePolicy, XmRESIZE_NONE, 0);
+	    XmNresizePolicy, XmRESIZE_NONE, NULL);
     XtAddCallback(form, XmNdestroyCallback, destroyCB, NULL);
     AddMotifCloseCallback(SmartIndentDialog.shell, dismissCB, NULL);
        
@@ -920,7 +951,7 @@ void EditSmartIndentMacros(WindowInfo *window)
 	    XmNtopAttachment, XmATTACH_POSITION,
 	    XmNtopPosition, 1,
 	    XmNrightAttachment, XmATTACH_POSITION,
-	    XmNrightPosition, 99, 0);
+	    XmNrightPosition, 99, NULL);
  
     SmartIndentDialog.lmPulldown = CreateLanguageModeMenu(lmForm, langModeCB,
     	    NULL);
@@ -944,14 +975,14 @@ void EditSmartIndentMacros(WindowInfo *window)
 	    XmNrightPosition, 50,
 	    XmNtopAttachment, XmATTACH_FORM,
 	    XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET,
-	    XmNbottomWidget, lmOptMenu, 0);
+	    XmNbottomWidget, lmOptMenu, NULL);
     XmStringFree(s1);
     
     lmBtn = XtVaCreateManagedWidget("lmBtn", xmPushButtonWidgetClass, lmForm,
     	    XmNlabelString, s1=MKSTRING("Add / Modify\nLanguage Mode..."),
     	    XmNmnemonic, 'A',
     	    XmNrightAttachment, XmATTACH_FORM,
-    	    XmNtopAttachment, XmATTACH_FORM, 0);
+    	    XmNtopAttachment, XmATTACH_FORM, NULL);
     XtAddCallback(lmBtn, XmNactivateCallback, lmDialogCB, NULL);
     XmStringFree(s1);
     
@@ -960,7 +991,7 @@ void EditSmartIndentMacros(WindowInfo *window)
     	    XmNlabelString, s1=MKSTRING("Common / Shared\nInitialization..."),
     	    XmNmnemonic, 'C',
     	    XmNleftAttachment, XmATTACH_FORM,
-    	    XmNtopAttachment, XmATTACH_FORM, 0);
+    	    XmNtopAttachment, XmATTACH_FORM, NULL);
     XtAddCallback(commonBtn, XmNactivateCallback, commonDialogCB, NULL);
     XmStringFree(s1);
     
@@ -971,7 +1002,7 @@ void EditSmartIndentMacros(WindowInfo *window)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 13,
     	    XmNbottomAttachment, XmATTACH_FORM,
-    	    XmNbottomOffset, BORDER, 0);
+    	    XmNbottomOffset, BORDER, NULL);
     XtAddCallback(okBtn, XmNactivateCallback, okCB, NULL);
     XmStringFree(s1);
     
@@ -983,7 +1014,7 @@ void EditSmartIndentMacros(WindowInfo *window)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 26,
     	    XmNbottomAttachment, XmATTACH_FORM,
-    	    XmNbottomOffset, BORDER, 0);
+    	    XmNbottomOffset, BORDER, NULL);
     XtAddCallback(applyBtn, XmNactivateCallback, applyCB, NULL);
     XmStringFree(s1);
     
@@ -995,7 +1026,7 @@ void EditSmartIndentMacros(WindowInfo *window)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 39,
     	    XmNbottomAttachment, XmATTACH_FORM,
-    	    XmNbottomOffset, BORDER, 0);
+    	    XmNbottomOffset, BORDER, NULL);
     XtAddCallback(checkBtn, XmNactivateCallback, checkCB, NULL);
     XmStringFree(s1);
     
@@ -1007,7 +1038,7 @@ void EditSmartIndentMacros(WindowInfo *window)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 52,
     	    XmNbottomAttachment, XmATTACH_FORM,
-    	    XmNbottomOffset, BORDER, 0);
+    	    XmNbottomOffset, BORDER, NULL);
     XtAddCallback(deleteBtn, XmNactivateCallback, deleteCB, NULL);
     XmStringFree(s1);
     
@@ -1019,7 +1050,7 @@ void EditSmartIndentMacros(WindowInfo *window)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 73,
     	    XmNbottomAttachment, XmATTACH_FORM,
-    	    XmNbottomOffset, BORDER, 0);
+    	    XmNbottomOffset, BORDER, NULL);
     XtAddCallback(restoreBtn, XmNactivateCallback, restoreCB, NULL);
     XmStringFree(s1);
     
@@ -1031,7 +1062,7 @@ void EditSmartIndentMacros(WindowInfo *window)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 86,
     	    XmNbottomAttachment, XmATTACH_FORM,
-    	    XmNbottomOffset, BORDER, 0);
+    	    XmNbottomOffset, BORDER, NULL);
     XtAddCallback(dismissBtn, XmNactivateCallback, dismissCB, NULL);
     XmStringFree(s1);
     
@@ -1044,7 +1075,7 @@ void EditSmartIndentMacros(WindowInfo *window)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 99,
     	    XmNbottomAttachment, XmATTACH_FORM,
-    	    XmNbottomOffset, BORDER, 0);
+    	    XmNbottomOffset, BORDER, NULL);
     XtAddCallback(helpBtn, XmNactivateCallback, helpCB, NULL);
     XmStringFree(s1);
     
@@ -1056,16 +1087,16 @@ void EditSmartIndentMacros(WindowInfo *window)
 	    XmNtopAttachment, XmATTACH_WIDGET,
 	    XmNtopWidget, lmForm,
 	    XmNbottomAttachment, XmATTACH_WIDGET,
-	    XmNbottomWidget, okBtn, 0);
+	    XmNbottomWidget, okBtn, NULL);
      	    /* XmNmarginWidth, 0, XmNmarginHeight, 0, XmNseparatorOn, False,
     	    XmNspacing, 3, XmNsashIndent, -2, */
 
     initForm = XtVaCreateManagedWidget("initForm", xmFormWidgetClass,
-	    pane, 0);
+	    pane, NULL);
     initLbl = XtVaCreateManagedWidget("initLbl", xmLabelGadgetClass, initForm,
     	    XmNlabelString, s1=XmStringCreateSimple(
     	     "Language Specific Initialization Macro Commands and Definitions"),
-    	    XmNmnemonic, 'I', 0);
+    	    XmNmnemonic, 'I', NULL);
     XmStringFree(s1);
     n = 0;
     XtSetArg(args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
@@ -1080,20 +1111,20 @@ void EditSmartIndentMacros(WindowInfo *window)
     	    "initMacro", args, n);
     XtManageChild(SmartIndentDialog.initMacro);
     RemapDeleteKey(SmartIndentDialog.initMacro);
-    XtVaSetValues(initLbl, XmNuserData, SmartIndentDialog.initMacro, 0);
+    XtVaSetValues(initLbl, XmNuserData, SmartIndentDialog.initMacro, NULL);
 
     newlineForm = XtVaCreateManagedWidget("newlineForm", xmFormWidgetClass,
-	    pane, 0);
+	    pane, NULL);
     newlineLbl = XtVaCreateManagedWidget("newlineLbl", xmLabelGadgetClass,
     	    newlineForm,
     	    XmNlabelString, s1=XmStringCreateSimple("Newline Macro"),
-    	    XmNmnemonic, 'N', 0);
+    	    XmNmnemonic, 'N', NULL);
     XmStringFree(s1);
     XtVaCreateManagedWidget("newlineArgsLbl", xmLabelGadgetClass,
     	    newlineForm, XmNalignment, XmALIGNMENT_END,
     	    XmNlabelString, s1=XmStringCreateSimple(
 	       "($1 is insert position, return indent request or -1)"),
-	    XmNrightAttachment, XmATTACH_FORM, 0);
+	    XmNrightAttachment, XmATTACH_FORM, NULL);
     XmStringFree(s1);
     n = 0;
     XtSetArg(args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
@@ -1108,19 +1139,19 @@ void EditSmartIndentMacros(WindowInfo *window)
     	    "newlineMacro", args, n);
     XtManageChild(SmartIndentDialog.newlineMacro);
     RemapDeleteKey(SmartIndentDialog.newlineMacro);
-    XtVaSetValues(newlineLbl, XmNuserData, SmartIndentDialog.newlineMacro, 0);
+    XtVaSetValues(newlineLbl, XmNuserData, SmartIndentDialog.newlineMacro,NULL);
 
     modifyForm = XtVaCreateManagedWidget("modifyForm", xmFormWidgetClass,
-	    pane, 0);
+	    pane, NULL);
     modifyLbl = XtVaCreateManagedWidget("modifyLbl", xmLabelGadgetClass,
     	    modifyForm, XmNlabelString,s1=XmStringCreateSimple("Type-in Macro"),
-    	    XmNmnemonic, 'M', 0);
+    	    XmNmnemonic, 'M', NULL);
     XmStringFree(s1);
     XtVaCreateManagedWidget("modifyArgsLbl", xmLabelGadgetClass,
     	    modifyForm, XmNalignment, XmALIGNMENT_END,
     	    XmNlabelString, s1=XmStringCreateSimple(
 	        "($1 is position, $2 is character just inserted)"),
-	    XmNrightAttachment, XmATTACH_FORM, 0);
+	    XmNrightAttachment, XmATTACH_FORM, NULL);
     XmStringFree(s1);
     n = 0;
     XtSetArg(args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
@@ -1135,14 +1166,14 @@ void EditSmartIndentMacros(WindowInfo *window)
     	    "modifyMacro", args, n);
     XtManageChild(SmartIndentDialog.modMacro);
     RemapDeleteKey(SmartIndentDialog.modMacro);
-    XtVaSetValues(modifyLbl, XmNuserData, SmartIndentDialog.modMacro, 0);
+    XtVaSetValues(modifyLbl, XmNuserData, SmartIndentDialog.modMacro, NULL);
 
     /* Set initial default button */
-    XtVaSetValues(form, XmNdefaultButton, okBtn, 0);
-    XtVaSetValues(form, XmNcancelButton, dismissBtn, 0);
+    XtVaSetValues(form, XmNdefaultButton, okBtn, NULL);
+    XtVaSetValues(form, XmNcancelButton, dismissBtn, NULL);
     
     /* Handle mnemonic selection of buttons and focus to dialog */
-    AddDialogMnemonicHandler(form);
+    AddDialogMnemonicHandler(form, FALSE);
     
     /* Fill in the dialog information for the selected language mode */
     setSmartIndentDialogData(findIndentSpec(lmName));
@@ -1166,7 +1197,7 @@ static void langModeCB(Widget w, XtPointer clientData, XtPointer callData)
     smartIndentRec *oldMacros, *newMacros;
 	    
     /* Get the newly selected mode name.  If it's the same, do nothing */
-    XtVaGetValues(w, XmNuserData, &modeName, 0);
+    XtVaGetValues(w, XmNuserData, &modeName, NULL);
     if (!strcmp(modeName, SmartIndentDialog.langModeName))
     	return;
 
@@ -1182,7 +1213,7 @@ static void langModeCB(Widget w, XtPointer clientData, XtPointer callData)
     if (indentSpecsDiffer(oldMacros, newMacros)) {
 	resp = DialogF(DF_QUES, SmartIndentDialog.shell, 3,
       "Smart indent macros for language mode\n%s were changed.  Apply changes?",
-		"Apply", "Discard", "Cancel", oldMacros->lmName);
+		"Apply", "Discard", "Cancel", SmartIndentDialog.langModeName);
 	if (resp == 3) {
 	    SetLangModeMenu(SmartIndentDialog.lmOptMenu,
 		    SmartIndentDialog.langModeName);
@@ -1431,12 +1462,12 @@ void EditCommonSmartIndentMacro(void)
     XtSetArg(args[n], XmNdeleteResponse, XmDO_NOTHING); n++;
     XtSetArg(args[n], XmNiconName, "Common Smart Indent Macros"); n++;
     XtSetArg(args[n], XmNtitle, "Common Smart Indent Macros"); n++;
-    CommonDialog.shell = XtAppCreateShell(APP_NAME, APP_CLASS,
+    CommonDialog.shell = CreateShellWithBestVis(APP_NAME, APP_CLASS,
 	    applicationShellWidgetClass, TheDisplay, args, n);
     AddSmallIcon(CommonDialog.shell);
     form = XtVaCreateManagedWidget("editCommonSIMacros", xmFormWidgetClass,
 	    CommonDialog.shell, XmNautoUnmanage, False,
-	    XmNresizePolicy, XmRESIZE_NONE, 0);
+	    XmNresizePolicy, XmRESIZE_NONE, NULL);
     XtAddCallback(form, XmNdestroyCallback, comDestroyCB, NULL);
     AddMotifCloseCallback(CommonDialog.shell, comDismissCB, NULL);
     
@@ -1447,7 +1478,7 @@ void EditCommonSmartIndentMacro(void)
 	    XmNtopAttachment, XmATTACH_FORM,
 	    XmNtopOffset, VERT_BORDER,
 	    XmNleftAttachment, XmATTACH_POSITION,
-	    XmNleftPosition, 1, 0);
+	    XmNleftPosition, 1, NULL);
 
     okBtn = XtVaCreateManagedWidget("ok", xmPushButtonWidgetClass, form,
     	    XmNlabelString, s1=XmStringCreateSimple("OK"),
@@ -1456,7 +1487,7 @@ void EditCommonSmartIndentMacro(void)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 18,
     	    XmNbottomAttachment, XmATTACH_FORM,
-    	    XmNbottomOffset, VERT_BORDER, 0);
+    	    XmNbottomOffset, VERT_BORDER, NULL);
     XtAddCallback(okBtn, XmNactivateCallback, comOKCB, NULL);
     XmStringFree(s1);
     
@@ -1468,7 +1499,7 @@ void EditCommonSmartIndentMacro(void)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 35,
     	    XmNbottomAttachment, XmATTACH_FORM,
-    	    XmNbottomOffset, VERT_BORDER, 0);
+    	    XmNbottomOffset, VERT_BORDER, NULL);
     XtAddCallback(applyBtn, XmNactivateCallback, comApplyCB, NULL);
     XmStringFree(s1);
     
@@ -1480,7 +1511,7 @@ void EditCommonSmartIndentMacro(void)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 52,
     	    XmNbottomAttachment, XmATTACH_FORM,
-    	    XmNbottomOffset, VERT_BORDER, 0);
+    	    XmNbottomOffset, VERT_BORDER, NULL);
     XtAddCallback(checkBtn, XmNactivateCallback, comCheckCB, NULL);
     XmStringFree(s1);
     
@@ -1493,7 +1524,7 @@ void EditCommonSmartIndentMacro(void)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 77,
     	    XmNbottomAttachment, XmATTACH_FORM,
-    	    XmNbottomOffset, VERT_BORDER, 0);
+    	    XmNbottomOffset, VERT_BORDER, NULL);
     XtAddCallback(restoreBtn, XmNactivateCallback, comRestoreCB, NULL);
     XmStringFree(s1);
     
@@ -1505,7 +1536,7 @@ void EditCommonSmartIndentMacro(void)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 94,
     	    XmNbottomAttachment, XmATTACH_FORM,
-    	    XmNbottomOffset, VERT_BORDER, 0);
+    	    XmNbottomOffset, VERT_BORDER, NULL);
     XtAddCallback(dismissBtn, XmNactivateCallback, comDismissCB, NULL);
     XmStringFree(s1);
     
@@ -1526,14 +1557,14 @@ void EditCommonSmartIndentMacro(void)
     CommonDialog.text = XmCreateScrolledText(form, "commonText", args, n);
     XtManageChild(CommonDialog.text);
     RemapDeleteKey(CommonDialog.text);
-    XtVaSetValues(topLbl, XmNuserData, CommonDialog.text, 0);
+    XtVaSetValues(topLbl, XmNuserData, CommonDialog.text, NULL);
 
     /* Set initial default button */
-    XtVaSetValues(form, XmNdefaultButton, okBtn, 0);
-    XtVaSetValues(form, XmNcancelButton, dismissBtn, 0);
+    XtVaSetValues(form, XmNdefaultButton, okBtn, NULL);
+    XtVaSetValues(form, XmNcancelButton, dismissBtn, NULL);
     
     /* Handle mnemonic selection of buttons and focus to dialog */
-    AddDialogMnemonicHandler(form);
+    AddDialogMnemonicHandler(form, FALSE);
     
     /* Realize all of the widgets in the new dialog */
     RealizeWithoutForcingPosition(CommonDialog.shell);
@@ -1769,7 +1800,6 @@ int LoadSmartIndentString(char *inString)
 	}
 	
 	/* read the modify macro */
-	inPtr + strspn(inPtr, " \t\n");
 	is.modMacro = readSIMacro(&inPtr);
 	if (is.modMacro == NULL) {
     	    XtFree(is.lmName);
