@@ -1,3 +1,4 @@
+static const char CVSID[] = "$Id: textBuf.c,v 1.11 2001/08/14 08:37:16 jlous Exp $";
 /*******************************************************************************
 *                                                                              *
 * textBuf.c - Manage source text for one or more text areas                    *
@@ -11,7 +12,7 @@
 * 									       *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
-* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
 * for more details.							       *
 * 									       *
 * You should have received a copy of the GNU General Public License along with *
@@ -26,33 +27,36 @@
 *******************************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <ctype.h>
+
 #include <Xm/Xm.h>
+
 #include "textBuf.h"
 
 #define PREFERRED_GAP_SIZE 80	/* Initial size for the buffer gap (empty space
                                    in the buffer where text might be inserted
                                    if the user is typing sequential chars) */
 
-static void histogramCharacters(char *string, int length, char hist[256],
+static void histogramCharacters(const char *string, int length, char hist[256],
 	int init);
 static void subsChars(char *string, int length, char fromChar, char toChar);
 static char chooseNullSubsChar(char hist[256]);
-static int insert(textBuffer *buf, int pos, char *text);
+static int insert(textBuffer *buf, int pos, const char *text);
 static void delete(textBuffer *buf, int start, int end);
 static void deleteRect(textBuffer *buf, int start, int end, int rectStart,
 	int rectEnd, int *replaceLen, int *endPos);
-static void insertCol(textBuffer *buf, int column, int startPos, char *insText,
+static void insertCol(textBuffer *buf, int column, int startPos, const char *insText,
 	int *nDeleted, int *nInserted, int *endPos);
 static void overlayRect(textBuffer *buf, int startPos, int rectStart,
     	int rectEnd, char *insText, int *nDeleted, int *nInserted, int *endPos);
-static void insertColInLine(char *line, char *insLine, int column, int insWidth,
+static void insertColInLine(const char *line, const char *insLine, int column, int insWidth,
 	int tabDist, int useTabs, char nullSubsChar, char *outStr, int *outLen,
 	int *endOffset);
-static void deleteRectFromLine(char *line, int rectStart, int rectEnd,
+static void deleteRectFromLine(const char *line, int rectStart, int rectEnd,
 	int tabDist, int useTabs, char nullSubsChar, char *outStr, int *outLen,
 	int *endOffset);
-static void overlayRectInLine(char *line, char *insLine, int rectStart,
+static void overlayRectInLine(const char *line, const char *insLine, int rectStart,
     	int rectEnd, int tabDist, int useTabs, char nullSubsChar, char *outStr,
     	int *outLen, int *endOffset);
 static void callModifyCBs(textBuffer *buf, int pos, int nDeleted,
@@ -72,32 +76,44 @@ static int getSelectionPos(selection *sel, int *start, int *end,
         int *isRect, int *rectStart, int *rectEnd);
 static char *getSelectionText(textBuffer *buf, selection *sel);
 static void removeSelected(textBuffer *buf, selection *sel);
-static void replaceSelected(textBuffer *buf, selection *sel, char *text);
+static void replaceSelected(textBuffer *buf, selection *sel, const char *text);
 static void addPadding(char *string, int startIndent, int toIndent,
 	int tabDist, int useTabs, char nullSubsChar, int *charsAdded);
 static int searchForward(textBuffer *buf, int startPos, char searchChar,
 	int *foundPos);
 static int searchBackward(textBuffer *buf, int startPos, char searchChar,
 	int *foundPos);
-static char *copyLine(char *text, int *lineLen);
-static int countLines(char *string);
-static int textWidth(char *text, int tabDist, char nullSubsChar);
+static char *copyLine(const char *text, int *lineLen);
+static int countLines(const char *string);
+static int textWidth(const char *text, int tabDist, char nullSubsChar);
 static void findRectSelBoundariesForCopy(textBuffer *buf, int lineStartPos,
 	int rectStart, int rectEnd, int *selStart, int *selEnd);
-static char *realignTabs(char *text, int origIndent, int newIndent,
+static char *realignTabs(const char *text, int origIndent, int newIndent,
 	int tabDist, int useTabs, char nullSubsChar, int *newLength);
-static char *expandTabs(char *text, int startIndent, int tabDist,
+static char *expandTabs(const char *text, int startIndent, int tabDist,
 	char nullSubsChar, int *newLen);
-static char *unexpandTabs(char *text, int startIndent, int tabDist,
+static char *unexpandTabs(const char *text, int startIndent, int tabDist,
 	char nullSubsChar, int *newLen);
 static int max(int i1, int i2);
 static int min(int i1, int i2);
 
-static char *ControlCodeTable[32] = {
+#ifdef __MVS__
+static const char *ControlCodeTable[64] = {
+     "nul", "soh", "stx", "etx", "sel", "ht", "rnl", "del",
+     "ge", "sps", "rpt", "vt", "ff", "cr", "so", "si",
+     "dle", "dc1", "dc2", "dc3", "res", "nl", "bs", "poc",
+     "can", "em", "ubs", "cu1", "ifs", "igs", "irs", "ius",
+     "ds", "sos", "fs", "wus", "byp", "lf", "etb", "esc",
+     "sa", "sfe", "sm", "csp", "mfa", "enq", "ack", "bel",
+     "x30", "x31", "syn", "ir", "pp", "trn", "nbs", "eot",
+     "sbs", "it", "rff", "cu3", "dc4", "nak", "x3e", "sub"};
+#else
+static const char *ControlCodeTable[32] = {
      "nul", "soh", "stx", "etx", "eot", "enq", "ack", "bel",
      "bs", "ht", "nl", "vt", "np", "cr", "so", "si",
      "dle", "dc1", "dc2", "dc3", "dc4", "nak", "syn", "etb",
      "can", "em", "sub", "esc", "fs", "gs", "rs", "us"};
+#endif
 
 /*
 ** Create an empty text buffer
@@ -174,7 +190,7 @@ char *BufGetAll(textBuffer *buf)
 /*
 ** Replace the entire contents of the text buffer
 */
-void BufSetAll(textBuffer *buf, char *text)
+void BufSetAll(textBuffer *buf, const char *text)
 {
     int length, deletedLength;
     char *deletedText;
@@ -261,7 +277,7 @@ char BufGetCharacter(textBuffer *buf, int pos)
 /*
 ** Insert null-terminated string "text" at position "pos" in "buf"
 */
-void BufInsert(textBuffer *buf, int pos, char *text)
+void BufInsert(textBuffer *buf, int pos, const char *text)
 {
     int nInserted;
     
@@ -279,7 +295,7 @@ void BufInsert(textBuffer *buf, int pos, char *text)
 ** Delete the characters between "start" and "end", and insert the
 ** null-terminated string "text" in their place in in "buf"
 */
-void BufReplace(textBuffer *buf, int start, int end, char *text)
+void BufReplace(textBuffer *buf, int start, int end, const char *text)
 {
     char *deletedText;
     int nInserted;
@@ -357,7 +373,7 @@ void BufCopyFromBuf(textBuffer *fromBuf, textBuffer *toBuf, int fromStart,
 ** number of characters inserted and deleted in the operation (beginning
 ** at startPos) are returned in these arguments
 */
-void BufInsertCol(textBuffer *buf, int column, int startPos, char *text,
+void BufInsertCol(textBuffer *buf, int column, int startPos, const char *text,
     	int *charsInserted, int *charsDeleted)
 {
     int nLines, lineStartPos, nDeleted, insertDeleted, nInserted;
@@ -415,9 +431,10 @@ void BufOverlayRect(textBuffer *buf, int startPos, int rectStart,
 ** rectangle, add extra lines to make room for it.
 */
 void BufReplaceRect(textBuffer *buf, int start, int end, int rectStart,
-	int rectEnd, char *text)
+	int rectEnd, const char *text)
 {
-    char *deletedText, *insText, *insPtr;
+    char *deletedText;
+    char *insText=NULL;
     int i, nInsertedLines, nDeletedLines, insLen, hint;
     int insertDeleted, insertInserted, deleteInserted;
     int linesPadded = 0;
@@ -436,6 +453,8 @@ void BufReplaceRect(textBuffer *buf, int start, int end, int rectStart,
     nInsertedLines = countLines(text);
     nDeletedLines = BufCountLines(buf, start, end);
     if (nInsertedLines < nDeletedLines) {
+        char *insPtr;
+
     	insLen = strlen(text);
     	insText = XtMalloc(insLen + nDeletedLines - nInsertedLines + 1);
     	strcpy(insText, text);
@@ -447,25 +466,28 @@ void BufReplaceRect(textBuffer *buf, int start, int end, int rectStart,
     	linesPadded = nInsertedLines-nDeletedLines;
     	for (i=0; i<linesPadded; i++)
     	    insert(buf, end, "\n");
-	insText = text;
-    } else /* nDeletedLines == nInsertedLines */
-	insText = text;
+    } else /* nDeletedLines == nInsertedLines */ {
+    }
     
     /* Save a copy of the text which will be modified for the modify CBs */
     deletedText = BufGetRange(buf, start, end);
     	  
     /* Delete then insert */
     deleteRect(buf, start, end, rectStart, rectEnd, &deleteInserted, &hint);
-    insertCol(buf, rectStart, start, insText, &insertDeleted, &insertInserted,
-    	    &buf->cursorPosHint);
+    if (insText) {
+    	insertCol(buf, rectStart, start, insText, &insertDeleted, &insertInserted,
+    		    &buf->cursorPosHint);
+        XtFree(insText);
+    }
+    else
+    	insertCol(buf, rectStart, start, text, &insertDeleted, &insertInserted,
+    		    &buf->cursorPosHint);
     
     /* Figure out how many chars were inserted and call modify callbacks */
     if (insertDeleted != deleteInserted + linesPadded)
     	fprintf(stderr, "NEdit: internal consistency check repl1 failed\n");
     callModifyCBs(buf, start, end-start, insertInserted, 0, deletedText);
     XtFree(deletedText);
-    if (nInsertedLines < nDeletedLines)
-    	XtFree(insText);
 }
 
 /*
@@ -847,18 +869,26 @@ int BufExpandCharacter(char c, int indent, char *outStr, int tabDist,
 	return nSpaces;
     }
     
-    /* Convert control codes to readable character sequences */
-    /*... is this safe with international character sets? */
+    /* Convert ASCII (and EBCDIC in the __MVS__ (OS/390) case) control
+       codes to readable character sequences */
+    if (c == nullSubsChar) {
+	sprintf(outStr, "<nul>");
+    	return 5;
+    }
+#ifdef __MVS__
+    if (((unsigned char)c) <= 63) {
+    	sprintf(outStr, "<%s>", ControlCodeTable[(unsigned char)c]);
+    	return strlen(outStr);
+    }
+#else
     if (((unsigned char)c) <= 31) {
-    	sprintf(outStr, "<%s>", ControlCodeTable[c]);
+    	sprintf(outStr, "<%s>", ControlCodeTable[(unsigned char)c]);
     	return strlen(outStr);
     } else if (c == 127) {
     	sprintf(outStr, "<del>");
     	return 5;
-    } else if (c == nullSubsChar) {
-	sprintf(outStr, "<nul>");
-    	return 5;
     }
+#endif
     
     /* Otherwise, just return the character */
     *outStr = c;
@@ -875,14 +905,14 @@ int BufExpandCharacter(char c, int indent, char *outStr, int tabDist,
 int BufCharWidth(char c, int indent, int tabDist, char nullSubsChar)
 {
     /* Note, this code must parallel that in BufExpandCharacter */
-    if (c == '\t')
+    if (c == nullSubsChar)
+    	return 5;
+    else if (c == '\t')
 	return tabDist - (indent % tabDist);
     else if (((unsigned char)c) <= 31)
-    	return strlen(ControlCodeTable[c]) + 2;
+    	return strlen(ControlCodeTable[(unsigned char)c]) + 2;
     else if (c == 127)
     	return 5;
-    else if (c == nullSubsChar)
-    	return 5;
     return 1;
 }
 
@@ -1119,8 +1149,10 @@ int BufSubstituteNullChars(char *string, int length, textBuffer *buf)
 	bufString = BufGetAll(buf);
 	histogramCharacters(bufString, buf->length, histogram, False);
 	newSubsChar = chooseNullSubsChar(histogram);
-	if (newSubsChar == '\0')
+	if (newSubsChar == '\0') {
+	    XtFree(bufString);
 	    return False;
+        }
 	subsChars(bufString, buf->length, buf->nullSubsChar, newSubsChar);
 	delete(buf, 0, buf->length);
 	insert(buf, 0, bufString);
@@ -1158,11 +1190,11 @@ void BufUnsubstituteNullChars(char *string, textBuffer *buf)
 ** with a 1).  If init is true, initialize the histogram before acumulating.
 ** if not, add the new data to an existing histogram.
 */
-static void histogramCharacters(char *string, int length, char hist[256],
+static void histogramCharacters(const char *string, int length, char hist[256],
 	int init)
 {
     int i;
-    char *c;
+    const char *c;
 
     if (init)
 	for (i=0; i<256; i++)
@@ -1195,7 +1227,7 @@ static char chooseNullSubsChar(char hist[256])
 	    20,21,22,23,24,25,26,28,29,30,31,11,7};
     int i;
     for (i = 0; i < N_REPLACEMENTS; i++)
-	if (hist[replacements[i]] == 0)
+	if (hist[(unsigned char)replacements[i]] == 0)
 	    return replacements[i];
     return '\0';
 }
@@ -1207,7 +1239,7 @@ static char chooseNullSubsChar(char hist[256])
 ** on to call redisplay).  pos must be contiguous with the existing text in
 ** the buffer (i.e. not past the end).
 */
-static int insert(textBuffer *buf, int pos, char *text)
+static int insert(textBuffer *buf, int pos, const char *text)
 {
     int length = strlen(text);
 
@@ -1263,12 +1295,13 @@ static void delete(textBuffer *buf, int start, int end)
 ** position of the lower left edge of the inserted column (as a hint for
 ** routines which need to set a cursor position).
 */
-static void insertCol(textBuffer *buf, int column, int startPos, char *insText,
+static void insertCol(textBuffer *buf, int column, int startPos, const char *insText,
 	int *nDeleted, int *nInserted, int *endPos)
 {
     int nLines, start, end, insWidth, lineStart, lineEnd;
     int expReplLen, expInsLen, len, endOffset;
-    char *c, *outStr, *outPtr, *line, *replText, *expText, *insLine, *insPtr;
+    char *outStr, *outPtr, *line, *replText, *expText, *insLine;
+    const char *insPtr;
 
     if (column < 0)
     	column = 0;
@@ -1298,10 +1331,8 @@ static void insertCol(textBuffer *buf, int column, int startPos, char *insText,
     outStr = XtMalloc(expReplLen + expInsLen +
     	    nLines * (column + insWidth + MAX_EXP_CHAR_LEN) + 1);
     
-    /* Loop over all lines in the buffer between start and end removing the
-       text between rectStart and rectEnd and padding appropriately.  Trim
-       trailing space from line (whitespace at the ends of lines otherwise
-       tends to multiply, since additional padding is added to maintain it */
+    /* Loop over all lines in the buffer between start and end inserting
+       text at column, splitting tabs and adding padding appropriately */
     outPtr = outStr;
     lineStart = start;
     insPtr = insText;
@@ -1314,8 +1345,16 @@ static void insertCol(textBuffer *buf, int column, int startPos, char *insText,
     		buf->useTabs, buf->nullSubsChar, outPtr, &len, &endOffset);
     	XtFree(line);
     	XtFree(insLine);
-    	for (c=outPtr+len-1; c>outPtr && isspace((unsigned char)*c); c--)
-    	    len--;
+#if 0   /* Earlier comments claimed that trailing whitespace could multiply on
+        the ends of lines, but insertColInLine looks like it should never
+        add space unnecessarily, and this trimming interfered with
+        paragraph filling, so lets see if it works without it. MWE */
+        {
+            char *c;
+            for (c=outPtr+len-1; c>outPtr && isspace((unsigned char)*c); c--)
+                len--;
+        }
+#endif
 	outPtr += len;
 	*outPtr++ = '\n';
     	lineStart = lineEnd < buf->length ? lineEnd + 1 : buf->length;
@@ -1467,11 +1506,12 @@ static void overlayRect(textBuffer *buf, int startPos, int rectStart,
 ** the right edge of the inserted text (as a hint for routines which need
 ** to position the cursor).
 */
-static void insertColInLine(char *line, char *insLine, int column, int insWidth,
+static void insertColInLine(const char *line, const char *insLine, int column, int insWidth,
 	int tabDist, int useTabs, char nullSubsChar, char *outStr, int *outLen,
 	int *endOffset)
 {
-    char *c, *linePtr, *outPtr, *retabbedStr;
+    char *c, *outPtr, *retabbedStr;
+    const char *linePtr;
     int indent, toIndent, len, postColIndent;
         
     /* copy the line up to "column" */ 
@@ -1557,12 +1597,13 @@ static void insertColInLine(char *line, char *insLine, int column, int insWidth,
 ** the beginning of the string to the point where the characters were
 ** deleted (as a hint for routines which need to position the cursor).
 */
-static void deleteRectFromLine(char *line, int rectStart, int rectEnd,
+static void deleteRectFromLine(const char *line, int rectStart, int rectEnd,
 	int tabDist, int useTabs, char nullSubsChar, char *outStr, int *outLen,
 	int *endOffset)
 {
     int indent, preRectIndent, postRectIndent, len;
-    char *c, *outPtr;
+    const char *c;
+    char *outPtr;
     char *retabbedStr;
     
     /* copy the line up to rectStart */
@@ -1617,11 +1658,12 @@ static void deleteRectFromLine(char *line, int rectStart, int rectEnd,
 ** the right edge of the inserted text (as a hint for routines which need
 ** to position the cursor).
 */
-static void overlayRectInLine(char *line, char *insLine, int rectStart,
+static void overlayRectInLine(const char *line, const char *insLine, int rectStart,
     	int rectEnd, int tabDist, int useTabs, char nullSubsChar, char *outStr,
     	int *outLen, int *endOffset)
 {
-    char *c, *linePtr, *outPtr, *retabbedStr;
+    char *c, *outPtr, *retabbedStr;
+    const char *linePtr;
     int inIndent, outIndent, len, postRectIndent;
         
     /* copy the line up to "rectStart" */ 
@@ -1774,7 +1816,7 @@ static void removeSelected(textBuffer *buf, selection *sel)
         BufRemove(buf, start, end);
 }
 
-static void replaceSelected(textBuffer *buf, selection *sel, char *text)
+static void replaceSelected(textBuffer *buf, selection *sel, const char *text)
 {
     int start, end, isRect, rectStart, rectEnd;
     selection oldSelection = *sel;
@@ -2060,10 +2102,11 @@ static int searchBackward(textBuffer *buf, int startPos, char searchChar,
 ** and return the copy as the function value, and the length of the line in
 ** "lineLen"
 */
-static char *copyLine(char *text, int *lineLen)
+static char *copyLine(const char *text, int *lineLen)
 {
     int len = 0;
-    char *c, *outStr;
+    const char *c;
+    char *outStr;
     
     for (c=text; *c!='\0' && *c!='\n'; c++)
     	len++;
@@ -2077,9 +2120,9 @@ static char *copyLine(char *text, int *lineLen)
 /*
 ** Count the number of newlines in a null-terminated text string;
 */
-static int countLines(char *string)
+static int countLines(const char *string)
 {
-    char *c;
+    const char *c;
     int lineCount = 0;
     
     for (c=string; *c!='\0'; c++)
@@ -2090,10 +2133,10 @@ static int countLines(char *string)
 /*
 ** Measure the width in displayed characters of string "text"
 */
-static int textWidth(char *text, int tabDist, char nullSubsChar)
+static int textWidth(const char *text, int tabDist, char nullSubsChar)
 {
     int width = 0, maxWidth = 0;
-    char *c;
+    const char *c;
     
     for (c=text; *c!='\0'; c++) {
     	if (*c == '\n') {
@@ -2167,7 +2210,7 @@ static void findRectSelBoundariesForCopy(textBuffer *buf, int lineStartPos,
 ** "origIndent" to starting at "newIndent".  Returns an allocated string
 ** which must be freed by the caller with XtFree.
 */
-static char *realignTabs(char *text, int origIndent, int newIndent,
+static char *realignTabs(const char *text, int origIndent, int newIndent,
 	int tabDist, int useTabs, char nullSubsChar, int *newLength)
 {
     char *expStr, *outStr;
@@ -2199,10 +2242,11 @@ static char *realignTabs(char *text, int origIndent, int newIndent,
 ** "startIndent" if nonzero, indicates that the text is a rectangular selection
 ** beginning at column "startIndent"
 */
-static char *expandTabs(char *text, int startIndent, int tabDist,
+static char *expandTabs(const char *text, int startIndent, int tabDist,
 	char nullSubsChar, int *newLen)
 {
-    char *outStr, *outPtr, *c;
+    char *outStr, *outPtr;
+    const char *c;
     int indent, len, outLen = 0;
 
     /* rehearse the expansion to figure out length for output string */
@@ -2248,10 +2292,11 @@ static char *expandTabs(char *text, int startIndent, int tabDist,
 ** when 3 or more spaces can be converted into a single tab, this avoids
 ** converting double spaces after a period withing a block of text.
 */
-static char *unexpandTabs(char *text, int startIndent, int tabDist,
+static char *unexpandTabs(const char *text, int startIndent, int tabDist,
 	char nullSubsChar, int *newLen)
 {
-    char *outStr, *outPtr, *c, expandedChar[MAX_EXP_CHAR_LEN];
+    char *outStr, *outPtr, expandedChar[MAX_EXP_CHAR_LEN];
+    const char *c;
     int indent, len;
     
     outStr = XtMalloc(strlen(text)+1);
