@@ -1,3 +1,4 @@
+static const char CVSID[] = "$Id: userCmds.c,v 1.17 2001/08/21 14:29:37 tringali Exp $";
 /*******************************************************************************
 *									       *
 * userCmds.c -- Nirvana Editor shell and macro command dialogs 		       *
@@ -11,7 +12,7 @@
 * 									       *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
-* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
 * for more details.							       *
 * 									       *
 * You should have received a copy of the GNU General Public License along with *
@@ -26,12 +27,16 @@
 *******************************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <ctype.h>
 #ifdef VMS
 #include "../util/VMSparam.h"
 #else
+#ifndef __MVS__
 #include <sys/param.h>
+#endif
 #endif /*VMS*/
+
 #include <Xm/Xm.h>
 #include <X11/keysym.h>
 #include <Xm/Text.h>
@@ -43,6 +48,7 @@
 #include <Xm/SelectioB.h>
 #include <Xm/RowColumn.h>
 #include <Xm/CascadeB.h>
+
 #include "../util/DialogF.h"
 #include "../util/misc.h"
 #include "../util/managedList.h"
@@ -55,8 +61,10 @@
 #include "shell.h"
 #include "macro.h"
 #include "file.h"
+#include "rbTree.h"
 #include "interpret.h"
 #include "parse.h"
+#include "userCmds.h"
 
 /* max number of user programmable menu commands allowed per each of the
    macro, shell, and bacground menus */
@@ -176,8 +184,8 @@ static void genAccelEventName(char *text, unsigned int modifiers,
 	KeySym keysym);
 static int parseAcceleratorString(char *string, unsigned int *modifiers,
 	KeySym *keysym);
-static int parseError(char *message);
-static char *copyMacroToEnd(char **inPtr, char *itemName);
+static int parseError(const char *message);
+static char *copyMacroToEnd(char **inPtr);
 static void addTerminatingNewline(char **string);
 
 /*
@@ -214,12 +222,12 @@ void EditShellMenu(WindowInfo *window)
     XtSetArg(args[ac], XmNdeleteResponse, XmDO_NOTHING); ac++;
     XtSetArg(args[ac], XmNiconName, "Shell Commands"); ac++;
     XtSetArg(args[ac], XmNtitle, "Shell Commands"); ac++;
-    ucd->dlogShell = XtAppCreateShell(APP_NAME, APP_CLASS,
+    ucd->dlogShell = CreateShellWithBestVis(APP_NAME, APP_CLASS,
 	    applicationShellWidgetClass, TheDisplay, args, ac);
     AddSmallIcon(ucd->dlogShell);
     form = XtVaCreateManagedWidget("editShellCommands", xmFormWidgetClass,
 	    ucd->dlogShell, XmNautoUnmanage, False,
-	    XmNresizePolicy, XmRESIZE_NONE, 0);
+	    XmNresizePolicy, XmRESIZE_NONE, NULL);
     ShellCmdDialog = ucd->dlogShell;
     XtAddCallback(form, XmNdestroyCallback, destroyCB, ucd);
     AddMotifCloseCallback(ucd->dlogShell, dismissCB, ucd);
@@ -248,7 +256,7 @@ void EditShellMenu(WindowInfo *window)
 	    XmNrightAttachment, XmATTACH_POSITION,
 	    XmNrightPosition, RIGHT_MARGIN_POS,
 	    XmNbottomAttachment, XmATTACH_POSITION,
-	    XmNbottomPosition, SHELL_CMD_TOP, 0);
+	    XmNbottomPosition, SHELL_CMD_TOP, NULL);
     XmStringFree(s1);
     ucd->saveFirstBtn = XtVaCreateManagedWidget("saveFirstBtn",
     	    xmToggleButtonWidgetClass, form,
@@ -261,7 +269,7 @@ void EditShellMenu(WindowInfo *window)
 	    XmNrightAttachment, XmATTACH_POSITION,
 	    XmNrightPosition, RIGHT_MARGIN_POS,
 	    XmNbottomAttachment, XmATTACH_WIDGET,
-	    XmNbottomWidget, ucd->loadAfterBtn, 0);
+	    XmNbottomWidget, ucd->loadAfterBtn, NULL);
     XmStringFree(s1);
     ucd->repInpBtn = XtVaCreateManagedWidget("repInpBtn",
     	    xmToggleButtonWidgetClass, form,
@@ -274,7 +282,7 @@ void EditShellMenu(WindowInfo *window)
 	    XmNrightAttachment, XmATTACH_POSITION,
 	    XmNrightPosition, RIGHT_MARGIN_POS,
 	    XmNbottomAttachment, XmATTACH_WIDGET,
-	    XmNbottomWidget, ucd->saveFirstBtn, 0);
+	    XmNbottomWidget, ucd->saveFirstBtn, NULL);
     XmStringFree(s1);
     outBox = XtVaCreateManagedWidget("outBox", xmRowColumnWidgetClass, form,
 	    XmNpacking, XmPACK_TIGHT,
@@ -287,14 +295,14 @@ void EditShellMenu(WindowInfo *window)
 	    XmNrightPosition, RIGHT_MARGIN_POS,
 	    XmNbottomAttachment, XmATTACH_WIDGET,
 	    XmNbottomWidget, ucd->repInpBtn,
-	    XmNbottomOffset, 4, 0);
+	    XmNbottomOffset, 4, NULL);
     ucd->sameOutBtn = XtVaCreateManagedWidget("sameOutBtn",
     	    xmToggleButtonWidgetClass, outBox,
     	    XmNlabelString, s1=MKSTRING("same window"),
     	    XmNmnemonic, 'm',
     	    XmNalignment, XmALIGNMENT_BEGINNING,
     	    XmNmarginHeight, 0,
-    	    XmNset, True, 0);
+    	    XmNset, True, NULL);
     XmStringFree(s1);
     XtAddCallback(ucd->sameOutBtn, XmNvalueChangedCallback, sameOutCB, ucd);
     ucd->dlogOutBtn = XtVaCreateManagedWidget("dlogOutBtn",
@@ -303,7 +311,7 @@ void EditShellMenu(WindowInfo *window)
     	    XmNmnemonic, 'g',
     	    XmNalignment, XmALIGNMENT_BEGINNING,
     	    XmNmarginHeight, 0,
-    	    XmNset, False, 0);
+    	    XmNset, False, NULL);
     XmStringFree(s1);
     ucd->winOutBtn = XtVaCreateManagedWidget("winOutBtn", xmToggleButtonWidgetClass,
     	    outBox,
@@ -311,7 +319,7 @@ void EditShellMenu(WindowInfo *window)
     	    XmNmnemonic, 'n',
     	    XmNalignment, XmALIGNMENT_BEGINNING,
     	    XmNmarginHeight, 0,
-    	    XmNset, False, 0);
+    	    XmNset, False, NULL);
     XmStringFree(s1);
     outLabel = XtVaCreateManagedWidget("outLabel", xmLabelGadgetClass, form,
     	    XmNlabelString, s1=MKSTRING("Command Output:"),
@@ -322,7 +330,7 @@ void EditShellMenu(WindowInfo *window)
 	    XmNrightAttachment, XmATTACH_POSITION,
 	    XmNrightPosition, RIGHT_MARGIN_POS,
 	    XmNbottomAttachment, XmATTACH_WIDGET,
-	    XmNbottomWidget, outBox, 0);
+	    XmNbottomWidget, outBox, NULL);
     XmStringFree(s1);
 
     inpBox = XtVaCreateManagedWidget("inpBox", xmRowColumnWidgetClass, form,
@@ -335,14 +343,14 @@ void EditShellMenu(WindowInfo *window)
 	    XmNrightAttachment, XmATTACH_POSITION,
 	    XmNrightPosition, RIGHT_MARGIN_POS,
 	    XmNbottomAttachment, XmATTACH_WIDGET,
-	    XmNbottomWidget, outLabel, 0);
+	    XmNbottomWidget, outLabel, NULL);
     ucd->selInpBtn = XtVaCreateManagedWidget("selInpBtn", xmToggleButtonWidgetClass,
     	    inpBox,
     	    XmNlabelString, s1=MKSTRING("selection"),
     	    XmNmnemonic, 's',
     	    XmNalignment, XmALIGNMENT_BEGINNING,
     	    XmNmarginHeight, 0,
-    	    XmNset, True, 0);
+    	    XmNset, True, NULL);
     XmStringFree(s1);
     ucd->winInpBtn = XtVaCreateManagedWidget("winInpBtn",
     	    xmToggleButtonWidgetClass, inpBox,
@@ -350,7 +358,7 @@ void EditShellMenu(WindowInfo *window)
     	    XmNmnemonic, 'w',
     	    XmNalignment, XmALIGNMENT_BEGINNING,
     	    XmNmarginHeight, 0,
-    	    XmNset, False, 0);
+    	    XmNset, False, NULL);
     XmStringFree(s1);
     ucd->eitherInpBtn = XtVaCreateManagedWidget("eitherInpBtn",
     	    xmToggleButtonWidgetClass, inpBox,
@@ -358,7 +366,7 @@ void EditShellMenu(WindowInfo *window)
     	    XmNmnemonic, 't',
     	    XmNalignment, XmALIGNMENT_BEGINNING,
     	    XmNmarginHeight, 0,
-    	    XmNset, False, 0);
+    	    XmNset, False, NULL);
     XmStringFree(s1);
     ucd->noInpBtn = XtVaCreateManagedWidget("noInpBtn",
     	    xmToggleButtonWidgetClass, inpBox,
@@ -366,7 +374,7 @@ void EditShellMenu(WindowInfo *window)
     	    XmNmnemonic, 'o',
     	    XmNalignment, XmALIGNMENT_BEGINNING,
     	    XmNmarginHeight, 0,
-    	    XmNset, False, 0);
+    	    XmNset, False, NULL);
     XmStringFree(s1);
     inpLabel = XtVaCreateManagedWidget("inpLabel", xmLabelGadgetClass, form,
     	    XmNlabelString, s1=MKSTRING("Command Input:"),
@@ -377,7 +385,7 @@ void EditShellMenu(WindowInfo *window)
 	    XmNrightAttachment, XmATTACH_POSITION,
 	    XmNrightPosition, RIGHT_MARGIN_POS,
 	    XmNbottomAttachment, XmATTACH_WIDGET,
-	    XmNbottomWidget, inpBox, 0);
+	    XmNbottomWidget, inpBox, NULL);
     XmStringFree(s1);
  
     ucd->mneTextW = XtVaCreateManagedWidget("mne", xmTextWidgetClass, form,
@@ -388,7 +396,7 @@ void EditShellMenu(WindowInfo *window)
 	    XmNrightAttachment, XmATTACH_POSITION,
 	    XmNrightPosition, RIGHT_MARGIN_POS,
 	    XmNbottomAttachment, XmATTACH_WIDGET,
-    	    XmNbottomWidget, inpLabel, 0);
+    	    XmNbottomWidget, inpLabel, NULL);
     RemapDeleteKey(ucd->mneTextW);
 
     ucd->accTextW = XtVaCreateManagedWidget("acc", xmTextWidgetClass, form,
@@ -400,7 +408,7 @@ void EditShellMenu(WindowInfo *window)
 	    XmNrightAttachment, XmATTACH_POSITION,
 	    XmNrightPosition, RIGHT_MARGIN_POS-15,
 	    XmNbottomAttachment, XmATTACH_WIDGET,
-    	    XmNbottomWidget, inpLabel, 0);
+    	    XmNbottomWidget, inpLabel, NULL);
     XtAddEventHandler(ucd->accTextW, KeyPressMask, False,
     	    (XtEventHandler)accKeyCB, ucd);
     XtAddCallback(ucd->accTextW, XmNfocusCallback, accFocusCB, ucd);
@@ -416,7 +424,7 @@ void EditShellMenu(WindowInfo *window)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, LIST_RIGHT + 24,
     	    XmNbottomAttachment, XmATTACH_WIDGET,
-    	    XmNbottomWidget, ucd->mneTextW, 0);
+    	    XmNbottomWidget, ucd->mneTextW, NULL);
     XmStringFree(s1);
 
     XtVaCreateManagedWidget("mneLabel", xmLabelGadgetClass, form,
@@ -430,7 +438,7 @@ void EditShellMenu(WindowInfo *window)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, RIGHT_MARGIN_POS,
     	    XmNbottomAttachment, XmATTACH_WIDGET,
-    	    XmNbottomWidget, ucd->mneTextW, 0);
+    	    XmNbottomWidget, ucd->mneTextW, NULL);
     XmStringFree(s1);
     
     ucd->nameTextW = XtVaCreateManagedWidget("name", xmTextWidgetClass, form,
@@ -439,7 +447,7 @@ void EditShellMenu(WindowInfo *window)
 	    XmNrightAttachment, XmATTACH_POSITION,
 	    XmNrightPosition, RIGHT_MARGIN_POS,
 	    XmNbottomAttachment, XmATTACH_WIDGET,
-    	    XmNbottomWidget, accLabel, 0);
+    	    XmNbottomWidget, accLabel, NULL);
     RemapDeleteKey(ucd->nameTextW);
  
     nameLabel = XtVaCreateManagedWidget("nameLabel", xmLabelGadgetClass, form,
@@ -451,7 +459,7 @@ void EditShellMenu(WindowInfo *window)
     	    XmNleftAttachment, XmATTACH_POSITION,
     	    XmNleftPosition, LIST_RIGHT,
     	    XmNbottomAttachment, XmATTACH_WIDGET,
-    	    XmNbottomWidget, ucd->nameTextW, 0);
+    	    XmNbottomWidget, ucd->nameTextW, NULL);
     XmStringFree(s1);
  
     XtVaCreateManagedWidget("nameNotes", xmLabelGadgetClass, form,
@@ -463,7 +471,7 @@ void EditShellMenu(WindowInfo *window)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, RIGHT_MARGIN_POS,
     	    XmNbottomAttachment, XmATTACH_WIDGET,
-    	    XmNbottomWidget, ucd->nameTextW, 0);
+    	    XmNbottomWidget, ucd->nameTextW, NULL);
     XmStringFree(s1);
 
     XtVaCreateManagedWidget("topLabel", xmLabelGadgetClass, form,
@@ -477,7 +485,7 @@ Select \"New\" to add a new command to the menu."),
 	    XmNrightAttachment, XmATTACH_POSITION,
 	    XmNrightPosition, RIGHT_MARGIN_POS,
 	    XmNbottomAttachment, XmATTACH_WIDGET,
-    	    XmNbottomWidget, nameLabel, 0);
+    	    XmNbottomWidget, nameLabel, NULL);
     XmStringFree(s1);
  
     cmdLabel = XtVaCreateManagedWidget("cmdLabel", xmLabelGadgetClass, form,
@@ -488,7 +496,7 @@ Select \"New\" to add a new command to the menu."),
     	    XmNtopAttachment, XmATTACH_POSITION,
     	    XmNtopPosition, SHELL_CMD_TOP,
     	    XmNleftAttachment, XmATTACH_POSITION,
-    	    XmNleftPosition, LEFT_MARGIN_POS, 0);
+    	    XmNleftPosition, LEFT_MARGIN_POS, NULL);
     XmStringFree(s1);
     XtVaCreateManagedWidget("cmdLabel", xmLabelGadgetClass, form,
     	    XmNlabelString, s1=MKSTRING("(% expands to current filename)"),
@@ -499,7 +507,7 @@ Select \"New\" to add a new command to the menu."),
     	    XmNleftAttachment, XmATTACH_WIDGET,
     	    XmNleftWidget, cmdLabel,
     	    XmNrightAttachment, XmATTACH_POSITION,
-    	    XmNrightPosition, RIGHT_MARGIN_POS, 0);
+    	    XmNrightPosition, RIGHT_MARGIN_POS, NULL);
     XmStringFree(s1);
 
     okBtn = XtVaCreateManagedWidget("ok",xmPushButtonWidgetClass,form,
@@ -509,7 +517,7 @@ Select \"New\" to add a new command to the menu."),
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 29,
     	    XmNbottomAttachment, XmATTACH_POSITION,
-    	    XmNbottomPosition, 99, 0);
+    	    XmNbottomPosition, 99, NULL);
     XtAddCallback(okBtn, XmNactivateCallback, okCB, ucd);
     XmStringFree(s1);
 
@@ -521,7 +529,7 @@ Select \"New\" to add a new command to the menu."),
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 58,
     	    XmNbottomAttachment, XmATTACH_POSITION,
-    	    XmNbottomPosition, 99, 0);
+    	    XmNbottomPosition, 99, NULL);
     XtAddCallback(applyBtn, XmNactivateCallback, applyCB, ucd);
     XmStringFree(s1);
 
@@ -532,7 +540,7 @@ Select \"New\" to add a new command to the menu."),
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 87,
     	    XmNbottomAttachment, XmATTACH_POSITION,
-    	    XmNbottomPosition, 99, 0);
+    	    XmNbottomPosition, 99, NULL);
     XtAddCallback(dismissBtn, XmNactivateCallback, dismissCB, ucd);
     XmStringFree(s1);
     
@@ -553,7 +561,7 @@ Select \"New\" to add a new command to the menu."),
     XtManageChild(ucd->cmdTextW);
     MakeSingleLineTextW(ucd->cmdTextW);
     RemapDeleteKey(ucd->cmdTextW);
-    XtVaSetValues(cmdLabel, XmNuserData, ucd->cmdTextW, 0); /* for mnemonic */
+    XtVaSetValues(cmdLabel, XmNuserData, ucd->cmdTextW, NULL); /* for mnemonic */
    
     /* Disable text input for the accelerator key widget, let the
        event handler manage it instead */
@@ -563,11 +571,11 @@ Select \"New\" to add a new command to the menu."),
     updateDialogFields(NULL, ucd); 
     
     /* Set initial default button */
-    XtVaSetValues(form, XmNdefaultButton, okBtn, 0);
-    XtVaSetValues(form, XmNcancelButton, dismissBtn, 0);
+    XtVaSetValues(form, XmNdefaultButton, okBtn, NULL);
+    XtVaSetValues(form, XmNcancelButton, dismissBtn, NULL);
     
     /* Handle mnemonic selection of buttons and focus to dialog */
-    AddDialogMnemonicHandler(form);
+    AddDialogMnemonicHandler(form, FALSE);
     
     /* realize all of the widgets in the new window */
     RealizeWithoutForcingPosition(ucd->dlogShell);
@@ -630,12 +638,12 @@ static void editMacroOrBGMenu(WindowInfo *window, int dialogType)
     XtSetArg(args[ac], XmNdeleteResponse, XmDO_NOTHING); ac++;
     XtSetArg(args[ac], XmNiconName, title); ac++;
     XtSetArg(args[ac], XmNtitle, title); ac++;
-    ucd->dlogShell = XtAppCreateShell(APP_NAME, APP_CLASS,
+    ucd->dlogShell = CreateShellWithBestVis(APP_NAME, APP_CLASS,
 	    applicationShellWidgetClass, TheDisplay, args, ac);
     AddSmallIcon(ucd->dlogShell);
     form = XtVaCreateManagedWidget("editMacroCommands", xmFormWidgetClass,
 	    ucd->dlogShell, XmNautoUnmanage, False,
-	    XmNresizePolicy, XmRESIZE_NONE, 0);
+	    XmNresizePolicy, XmRESIZE_NONE, NULL);
     XtAddCallback(form, XmNdestroyCallback, destroyCB, ucd);
     AddMotifCloseCallback(ucd->dlogShell, dismissCB, ucd);
  
@@ -662,7 +670,7 @@ static void editMacroOrBGMenu(WindowInfo *window, int dialogType)
 	    XmNleftAttachment, XmATTACH_POSITION,
 	    XmNleftPosition, LIST_RIGHT,
 	    XmNbottomAttachment, XmATTACH_POSITION,
-    	    XmNbottomPosition, MACRO_CMD_TOP, 0);
+    	    XmNbottomPosition, MACRO_CMD_TOP, NULL);
     XmStringFree(s1);
  
     ucd->mneTextW = XtVaCreateManagedWidget("mne", xmTextWidgetClass, form,
@@ -674,7 +682,7 @@ static void editMacroOrBGMenu(WindowInfo *window, int dialogType)
 	    XmNrightPosition, RIGHT_MARGIN_POS-21,
 	    XmNbottomAttachment, XmATTACH_WIDGET,
     	    XmNbottomWidget, ucd->selInpBtn,
-	    XmNbottomOffset, 5, 0);
+	    XmNbottomOffset, 5, NULL);
     RemapDeleteKey(ucd->mneTextW);
 
     ucd->accTextW = XtVaCreateManagedWidget("acc", xmTextWidgetClass, form,
@@ -687,7 +695,7 @@ static void editMacroOrBGMenu(WindowInfo *window, int dialogType)
 	    XmNrightPosition, RIGHT_MARGIN_POS-20-10,
 	    XmNbottomAttachment, XmATTACH_WIDGET,
     	    XmNbottomWidget, ucd->selInpBtn,
-	    XmNbottomOffset, 5, 0);
+	    XmNbottomOffset, 5, NULL);
     XtAddEventHandler(ucd->accTextW, KeyPressMask, False,
     	    (XtEventHandler)accKeyCB, ucd);
     XtAddCallback(ucd->accTextW, XmNfocusCallback, accFocusCB, ucd);
@@ -704,7 +712,7 @@ static void editMacroOrBGMenu(WindowInfo *window, int dialogType)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, LIST_RIGHT + 22,
     	    XmNbottomAttachment, XmATTACH_WIDGET,
-    	    XmNbottomWidget, ucd->mneTextW, 0);
+    	    XmNbottomWidget, ucd->mneTextW, NULL);
     XmStringFree(s1);
 
     XtVaCreateManagedWidget("mneLabel", xmLabelGadgetClass, form,
@@ -718,7 +726,7 @@ static void editMacroOrBGMenu(WindowInfo *window, int dialogType)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, RIGHT_MARGIN_POS-21,
     	    XmNbottomAttachment, XmATTACH_WIDGET,
-    	    XmNbottomWidget, ucd->mneTextW, 0);
+    	    XmNbottomWidget, ucd->mneTextW, NULL);
     XmStringFree(s1);
 
     pasteReplayBtn = XtVaCreateManagedWidget("pasteReplay",
@@ -726,14 +734,12 @@ static void editMacroOrBGMenu(WindowInfo *window, int dialogType)
     	    XmNlabelString, s1=MKSTRING("Paste Learn/\nReplay Macro"),
     	    XmNmnemonic, 'P',
     	    XmNsensitive, GetReplayMacro() != NULL,
-    	    XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET,
-	    XmNtopWidget, ucd->accTextW,
      	    XmNleftAttachment, XmATTACH_POSITION,
    	    XmNleftPosition, RIGHT_MARGIN_POS-20,
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, RIGHT_MARGIN_POS,
 	    XmNbottomAttachment, XmATTACH_POSITION,
-    	    XmNbottomPosition, MACRO_CMD_TOP, 0);
+    	    XmNbottomPosition, MACRO_CMD_TOP, NULL);
     XtAddCallback(pasteReplayBtn, XmNactivateCallback,
     	    pasteReplayCB, ucd);
     XmStringFree(s1);
@@ -744,7 +750,7 @@ static void editMacroOrBGMenu(WindowInfo *window, int dialogType)
 	    XmNrightAttachment, XmATTACH_POSITION,
 	    XmNrightPosition, RIGHT_MARGIN_POS,
 	    XmNbottomAttachment, XmATTACH_WIDGET,
-    	    XmNbottomWidget, accLabel, 0);
+    	    XmNbottomWidget, accLabel, NULL);
     RemapDeleteKey(ucd->nameTextW);
  
     nameLabel = XtVaCreateManagedWidget("nameLabel", xmLabelGadgetClass, form,
@@ -756,7 +762,7 @@ static void editMacroOrBGMenu(WindowInfo *window, int dialogType)
     	    XmNleftAttachment, XmATTACH_POSITION,
     	    XmNleftPosition, LIST_RIGHT,
     	    XmNbottomAttachment, XmATTACH_WIDGET,
-    	    XmNbottomWidget, ucd->nameTextW, 0);
+    	    XmNbottomWidget, ucd->nameTextW, NULL);
     XmStringFree(s1);
  
     XtVaCreateManagedWidget("nameNotes", xmLabelGadgetClass, form,
@@ -768,7 +774,7 @@ static void editMacroOrBGMenu(WindowInfo *window, int dialogType)
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, RIGHT_MARGIN_POS,
     	    XmNbottomAttachment, XmATTACH_WIDGET,
-    	    XmNbottomWidget, ucd->nameTextW, 0);
+    	    XmNbottomWidget, ucd->nameTextW, NULL);
     XmStringFree(s1);
 
     XtVaCreateManagedWidget("topLabel", xmLabelGadgetClass, form,
@@ -782,7 +788,7 @@ Select \"New\" to add a new command to the menu."),
 	    XmNrightAttachment, XmATTACH_POSITION,
 	    XmNrightPosition, RIGHT_MARGIN_POS,
 	    XmNbottomAttachment, XmATTACH_WIDGET,
-    	    XmNbottomWidget, nameLabel, 0);
+    	    XmNbottomWidget, nameLabel, NULL);
     XmStringFree(s1);
  
     cmdLabel = XtVaCreateManagedWidget("cmdLabel", xmLabelGadgetClass, form,
@@ -793,7 +799,7 @@ Select \"New\" to add a new command to the menu."),
     	    XmNtopAttachment, XmATTACH_POSITION,
     	    XmNtopPosition, MACRO_CMD_TOP,
     	    XmNleftAttachment, XmATTACH_POSITION,
-    	    XmNleftPosition, LEFT_MARGIN_POS, 0);
+    	    XmNleftPosition, LEFT_MARGIN_POS, NULL);
     XmStringFree(s1);
 
     okBtn = XtVaCreateManagedWidget("ok",xmPushButtonWidgetClass,form,
@@ -803,7 +809,7 @@ Select \"New\" to add a new command to the menu."),
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 23,
     	    XmNbottomAttachment, XmATTACH_POSITION,
-    	    XmNbottomPosition, 99, 0);
+    	    XmNbottomPosition, 99, NULL);
     XtAddCallback(okBtn, XmNactivateCallback, okCB, ucd);
     XmStringFree(s1);
 
@@ -815,7 +821,7 @@ Select \"New\" to add a new command to the menu."),
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 46,
     	    XmNbottomAttachment, XmATTACH_POSITION,
-    	    XmNbottomPosition, 99, 0);
+    	    XmNbottomPosition, 99, NULL);
     XtAddCallback(applyBtn, XmNactivateCallback, applyCB, ucd);
     XmStringFree(s1);
 
@@ -827,7 +833,7 @@ Select \"New\" to add a new command to the menu."),
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 69,
     	    XmNbottomAttachment, XmATTACH_POSITION,
-    	    XmNbottomPosition, 99, 0);
+    	    XmNbottomPosition, 99, NULL);
     XtAddCallback(applyBtn, XmNactivateCallback, checkCB, ucd);
     XmStringFree(s1);
 
@@ -838,7 +844,7 @@ Select \"New\" to add a new command to the menu."),
     	    XmNrightAttachment, XmATTACH_POSITION,
     	    XmNrightPosition, 92,
     	    XmNbottomAttachment, XmATTACH_POSITION,
-    	    XmNbottomPosition, 99, 0);
+    	    XmNbottomPosition, 99, NULL);
     XtAddCallback(dismissBtn, XmNactivateCallback, dismissCB, ucd);
     XmStringFree(s1);
     
@@ -856,7 +862,7 @@ Select \"New\" to add a new command to the menu."),
     ucd->cmdTextW = XmCreateScrolledText(form, "name", args, ac);
     XtManageChild(ucd->cmdTextW);
     RemapDeleteKey(ucd->cmdTextW);
-    XtVaSetValues(cmdLabel, XmNuserData, ucd->cmdTextW, 0); /* for mnemonic */
+    XtVaSetValues(cmdLabel, XmNuserData, ucd->cmdTextW, NULL); /* for mnemonic */
    
     /* Disable text input for the accelerator key widget, let the
        event handler manage it instead */
@@ -866,11 +872,11 @@ Select \"New\" to add a new command to the menu."),
     updateDialogFields(NULL, ucd); 
       
     /* Set initial default button */
-    XtVaSetValues(form, XmNdefaultButton, okBtn, 0);
-    XtVaSetValues(form, XmNcancelButton, dismissBtn, 0);
+    XtVaSetValues(form, XmNdefaultButton, okBtn, NULL);
+    XtVaSetValues(form, XmNcancelButton, dismissBtn, NULL);
     
     /* Handle mnemonic selection of buttons and focus to dialog */
-    AddDialogMnemonicHandler(form);
+    AddDialogMnemonicHandler(form, FALSE);
     
     /* Make widgets for top level shell and paste-replay buttons available
        to other functions */
@@ -935,18 +941,19 @@ static void dimSelDepItemsInMenu(Widget menuPane, menuItemRec **menuList,
 {
     WidgetList items;
     Widget subMenu;
-    XtArgVal userData;
-    int n, nItems, index;
+    XtPointer userData;
+    int n, index;
+    Cardinal nItems;
     
-    XtVaGetValues(menuPane, XmNchildren, &items, XmNnumChildren, &nItems,0);
+    XtVaGetValues(menuPane, XmNchildren, &items, XmNnumChildren, &nItems, NULL);
     for (n=0; n<nItems; n++) {
-	XtVaGetValues(items[n], XmNuserData, &userData, 0);
-    	if (userData != PERMANENT_MENU_ITEM) {
+	XtVaGetValues(items[n], XmNuserData, &userData, NULL);
+    	if (userData !=  (XtPointer)PERMANENT_MENU_ITEM) {
     	    if (XtClass(items[n]) == xmCascadeButtonWidgetClass) {
-	    	XtVaGetValues(items[n], XmNsubMenuId, &subMenu, 0);
+	    	XtVaGetValues(items[n], XmNsubMenuId, &subMenu, NULL);
 		dimSelDepItemsInMenu(subMenu, menuList, nMenuItems, sensitive);
 	    } else {
-		index = userData - 10;
+		index = (int)userData - 10;
 		if (index <0 || index >= nMenuItems)
     		    return;
 		if (menuList[index]->input == FROM_SELECTION)
@@ -995,6 +1002,7 @@ char *WriteMacroCmdsString(void)
 {
     return writeMenuItemString(MacroMenuItems, NMacroMenuItems, MACRO_CMDS);
 }
+
 char *WriteBGMenuCmdsString(void)
 {
     return writeMenuItemString(BGMenuItems, NBGMenuItems, BG_MENU_CMDS);
@@ -1019,6 +1027,7 @@ int LoadMacroCmdsString(char *inString)
     return loadMenuItemString(inString, MacroMenuItems, &NMacroMenuItems,
     	    MACRO_CMDS);
 }
+
 int LoadBGMenuCmdsString(char *inString)
 {
     return loadMenuItemString(inString, BGMenuItems, &NBGMenuItems,
@@ -1030,7 +1039,7 @@ int LoadBGMenuCmdsString(char *inString)
 ** name "itemName".  Returns True on successs and False on failure.
 */
 #ifndef VMS
-int DoNamedShellMenuCmd(WindowInfo *window, char *itemName, int fromMacro)
+int DoNamedShellMenuCmd(WindowInfo *window, const char *itemName, int fromMacro)
 {
     int i;
     
@@ -1055,7 +1064,7 @@ int DoNamedShellMenuCmd(WindowInfo *window, char *itemName, int fromMacro)
 ** with menu item name "itemName".  Returns True on successs and False on
 ** failure.
 */
-int DoNamedMacroMenuCmd(WindowInfo *window, char *itemName)
+int DoNamedMacroMenuCmd(WindowInfo *window, const char *itemName)
 {
     int i;
     
@@ -1067,7 +1076,8 @@ int DoNamedMacroMenuCmd(WindowInfo *window, char *itemName)
     }
     return False;
 }
-int DoNamedBGMenuCmd(WindowInfo *window, char *itemName)
+
+int DoNamedBGMenuCmd(WindowInfo *window, const char *itemName)
 {
     int i;
     
@@ -1177,6 +1187,7 @@ static void updateMenu(WindowInfo *window, int menuType)
 		    window->bgMenuUndoItem = btn;
 		else if (menuType == BG_MENU_CMDS && !strcmp(f->cmd,"redo()\n"))
 		    window->bgMenuRedoItem = btn;
+		UpdateAccelLockPatch(window->splitPane, btn);
 		break;
 	    }
 	    hierName = copySubstring(strippedName, subSep - strippedName);
@@ -1277,9 +1288,11 @@ static Widget createUserMenuItem(Widget menuPane, char *name, menuItemRec *f,
     
     generateAcceleratorString(accText, f->modifiers, f->keysym);
     genAccelEventName(accKeys, f->modifiers, f->keysym);
+    st1=XmStringCreateSimple(name);
+    st2=XmStringCreateSimple(accText);
     btn = XtVaCreateManagedWidget("cmd", xmPushButtonWidgetClass, menuPane, 
-    	    XmNlabelString, st1=XmStringCreateSimple(name),
-    	    XmNacceleratorText, st2=XmStringCreateSimple(accText),
+    	    XmNlabelString, st1,
+    	    XmNacceleratorText, st2,
     	    XmNaccelerator, accKeys,
     	    XmNmnemonic, f->mnemonic,
     	    XmNuserData, index+10, NULL);
@@ -1301,10 +1314,10 @@ static Widget createUserSubMenu(Widget parent, char *label)
     XmString st1;
     static Arg args[1] = {{XmNuserData, (XtArgVal)TEMPORARY_MENU_ITEM}};
    
-    menu = XmCreatePulldownMenu(parent, "userPulldown", args, 1);
+    menu = CreatePulldownMenu(parent, "userPulldown", args, 1);
     XtVaCreateManagedWidget("userCascade", xmCascadeButtonWidgetClass, parent, 
     	    XmNlabelString, st1=XmStringCreateSimple(label),
-    	    XmNsubMenuId, menu, XmNuserData, TEMPORARY_MENU_ITEM, 0);
+    	    XmNsubMenuId, menu, XmNuserData, TEMPORARY_MENU_ITEM, NULL);
     XmStringFree(st1);
     return menu;
 }
@@ -1313,21 +1326,23 @@ static void removeMenuItems(Widget menuPane)
 {
     WidgetList items, itemList;
     Widget subMenuID;
-    XtArgVal userData;
-    int n, nItems;
+    XtPointer userData;
+    int n;
+    Cardinal nItems;
     
     /* Fetch the list of children from the menu pane, and make a copy
        (because the widget alters this list as you delete widgets) */
-    XtVaGetValues(menuPane, XmNchildren, &itemList, XmNnumChildren, &nItems,0);
+    XtVaGetValues(menuPane, XmNchildren, &itemList, XmNnumChildren, &nItems,
+	    NULL);
     items = (WidgetList)XtMalloc(sizeof(Widget) * nItems);
     memcpy(items, itemList, sizeof(Widget) * nItems);
     
     /* Delete all of the widgets not marked as PERMANENT_MENU_ITEM */
     for (n=0; n<nItems; n++) {
-	XtVaGetValues(items[n], XmNuserData, &userData, 0);
-    	if (userData != PERMANENT_MENU_ITEM) {
+	XtVaGetValues(items[n], XmNuserData, &userData, NULL);
+    	if (userData !=  (XtPointer)PERMANENT_MENU_ITEM) {
     	    if (XtClass(items[n]) == xmCascadeButtonWidgetClass) {
-		XtVaGetValues(items[n], XmNsubMenuId, &subMenuID, 0);
+		XtVaGetValues(items[n], XmNsubMenuId, &subMenuID, NULL);
 		removeMenuItems(subMenuID);
 #if XmVersion < 2000  /* Skipping this creates a memory and server resource
 		   leak (though both are reclaimed on window closing).  In
@@ -1338,7 +1353,7 @@ static void removeMenuItems(Widget menuPane)
 		XtDestroyWidget(subMenuID);
 #endif
 	    } else /* remove accel. before destroy or lose it forever */
-    		XtVaSetValues(items[n], XmNaccelerator, NULL, 0);
+    		XtVaSetValues(items[n], XmNaccelerator, NULL, NULL);
     	    /* unmanaging before destroying stops parent from displaying */
     	    XtUnmanageChild(items[n]);
     	    XtDestroyWidget(items[n]);
@@ -1514,7 +1529,7 @@ static void accLoseFocusCB(Widget w, XtPointer clientData, XtPointer callData)
 {
     userCmdDialog *ucd = (userCmdDialog *)clientData;
 
-    AddDialogMnemonicHandler(XtParent(ucd->accTextW));
+    AddDialogMnemonicHandler(XtParent(ucd->accTextW), FALSE);
 }
 
 static void accKeyCB(Widget w, XtPointer clientData, XKeyEvent *event)
@@ -1573,8 +1588,8 @@ static void shellMenuCB(Widget w, WindowInfo *window, XtPointer callData)
     char *params[1];
 
     /* get the index of the shell command and verify that it's in range */
-    XtVaGetValues(w, XmNuserData, &userData, 0);
-    index = userData - 10;
+    XtVaGetValues(w, XmNuserData, &userData, NULL);
+    index = (int)userData - 10;
     if (index <0 || index >= NShellMenuItems)
     	return;
     
@@ -1601,8 +1616,8 @@ static void macroMenuCB(Widget w, WindowInfo *window, XtPointer callData)
 	return;
     
     /* get the index of the macro command and verify that it's in range */
-    XtVaGetValues(w, XmNuserData, &userData, 0);
-    index = userData - 10;
+    XtVaGetValues(w, XmNuserData, &userData, NULL);
+    index = (int)userData - 10;
     if (index <0 || index >= NMacroMenuItems)
     	return;
     
@@ -1618,8 +1633,8 @@ static void bgMenuCB(Widget w, WindowInfo *window, XtPointer callData)
     char *params[1];
 
     /* get the index of the macro command and verify that it's in range */
-    XtVaGetValues(w, XmNuserData, &userData, 0);
-    index = userData - 10;
+    XtVaGetValues(w, XmNuserData, &userData, NULL);
+    index = (int)userData - 10;
     if (index <0 || index >= NBGMenuItems)
     	return;
     
@@ -1885,7 +1900,7 @@ static void disableTextW(Widget textW)
     /* replace the translation table with the slimmed down one above */
     if (emptyTable == NULL)
     	emptyTable = XtParseTranslationTable(emptyTranslations);
-    XtVaSetValues(textW, XmNtranslations, emptyTable, 0);
+    XtVaSetValues(textW, XmNtranslations, emptyTable, NULL);
 }
 
 static char *writeMenuItemString(menuItemRec **menuItems, int nItems,
@@ -1989,7 +2004,8 @@ static int loadMenuItemString(char *inString, menuItemRec **menuItems,
 	int *nItems, int listType)
 {
     menuItemRec *f;
-    char *cmdStr, *inPtr = inString;
+    char *cmdStr;
+    char *inPtr = inString;
     char *nameStr, accStr[MAX_ACCEL_LEN], mneChar;
     KeySym keysym;
     unsigned int modifiers;
@@ -2086,7 +2102,7 @@ static int loadMenuItemString(char *inString, menuItemRec **menuItems,
     	    cmdStr[cmdLen] = '\0';
     	    inPtr += cmdLen;
 	} else {
-	    cmdStr = copyMacroToEnd(&inPtr, nameStr);
+	    cmdStr = copyMacroToEnd(&inPtr);
 	    if (cmdStr == NULL)
 	    	return False;
 	}
@@ -2127,7 +2143,7 @@ static int loadMenuItemString(char *inString, menuItemRec **menuItems,
     }
 }
 
-static int parseError(char *message)
+static int parseError(const char *message)
 {
     fprintf(stderr, "NEdit: Parse error in user defined menu item, %s\n",
     	    message);
@@ -2296,7 +2312,7 @@ static int parseAcceleratorString(char *string, unsigned int *modifiers,
 ** to be re-generated from the text as needed, but compile time is
 ** negligible for most macros.
 */
-static char *copyMacroToEnd(char **inPtr, char *itemName)
+static char *copyMacroToEnd(char **inPtr)
 {
     char *retStr, *errMsg, *stoppedAt, *p, *retPtr;
     Program *prog;
