@@ -1,3 +1,4 @@
+/* $Id: userCmds.h,v 1.4 2001/08/15 08:56:33 amai Exp $ */
 void EditShellMenu(WindowInfo *window);
 void EditMacroMenu(WindowInfo *window);
 void EditBGMenu(WindowInfo *window);
@@ -10,9 +11,9 @@ char *WriteBGMenuCmdsString(void);
 int LoadShellCmdsString(char *inString);
 int LoadMacroCmdsString(char *inString);
 int LoadBGMenuCmdsString(char *inString);
-int DoNamedShellMenuCmd(WindowInfo *window, char *itemName, int fromMacro);
-int DoNamedMacroMenuCmd(WindowInfo *window, char *itemName);
-int DoNamedBGMenuCmd(WindowInfo *window, char *itemName);
+int DoNamedShellMenuCmd(WindowInfo *window, const char *itemName, int fromMacro);
+int DoNamedMacroMenuCmd(WindowInfo *window, const char *itemName);
+int DoNamedBGMenuCmd(WindowInfo *window, const char *itemName);
 void SetBGMenuUndoSensitivity(WindowInfo *window, int sensitive);
 void SetBGMenuRedoSensitivity(WindowInfo *window, int sensitive);
 void DimSelectionDepUserMenuItems(WindowInfo *window, int sensitive);
