@@ -1,3 +1,4 @@
+static const char CVSID[] = "$Id: getfiles.c,v 1.15 2001/08/14 08:37:16 jlous Exp $";
 /******************************************************************************
 *                                                                             *
 * Getfiles.c -- File Interface Routines                                       *
@@ -11,7 +12,7 @@
 * 									       *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
-* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
 * for more details.							       *
 * 									       *
 * You should have received a copy of the GNU General Public License along with *
@@ -57,10 +58,11 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <dirent.h>
+#ifndef __MVS__
 #include <sys/param.h>
+#endif
 #endif /*VMS*/
 #include <sys/stat.h>
-#include <sys/errno.h>
 #include <X11/keysym.h>
 #include <Xm/Xm.h>
 #include <Xm/PushBG.h>
@@ -85,9 +87,9 @@
 
 #define SET_ONE_RSRC(widget, name, newValue) \
 { \
-    static Arg args[1] = {{name, (XtArgVal)0}}; \
-    args[0].value = (XtArgVal)newValue; \
-    XtSetValues(widget, args, 1); \
+    static Arg tmpargs[1] = {{name, (XtArgVal)0}}; \
+    tmpargs[0].value = (XtArgVal)newValue; \
+    XtSetValues(widget, tmpargs, 1); \
 }	
 
 enum yesNoValues {ynNone, ynYes, ynNo};
@@ -104,8 +106,7 @@ static int RemoveRedundantTextField = True;
 /* Text for help button help display */
 /* ... needs variant for VMS */
 #ifndef SGI_CUSTOM
-#ifndef MOTIF10
-static char *HelpExist =
+static const char *HelpExist =
 "The file open dialog shows a list of directories on the left, and a list \
 of files on the right.  Double clicking on a file name in the list on the \
 right, or selecting it and pressing the OK button, will open that file.  \
@@ -123,7 +124,8 @@ match.  When you leave off the file name or trailing \"/\", you won't see \
 any files to open in the list \
 because the filter specification matched the directory file itself, rather \
 than the files in the directory.";
-static char *HelpNew = 
+
+static const char *HelpNew = 
 "This dialog allows you to create a new file, or to save the current file \
 under a new name.  To specify a file \
 name in the current directory, complete the name displayed in the \"Save File \
@@ -143,35 +145,8 @@ click on the directory entry ending in \"..\".  You can also move directly \
 to a directory by typing the file specification of the path in the \"Filter\" \
 field and pressing the \"Filter\" button.";
 
-#else /* MOTIF 1.0 file dialogs are different */
-static char *HelpExist =
-"The file open dialog initially shows a list of files in the current \
-directory.  Double clicking on a file name in the list, or selecting it \
-and pressing the OK button, will open that file.  \n\
-\n\
-To open a file outside of the current directory, click on the field labeled \
-\"Filter\", type in a directory specification, and press return, or \
-press the Filter button in the dialog.  The files in that directory will \
-then show up in the file list and you can select one as above.";
-static char *HelpNew = 
-"The Save As... dialog allows you to save the file you are editing under a \
-new name, or to specify the name for an Untitled file.  To specify a file \
-name in the current directory, complete the name displayed in the \"Save File \
-As:\" field near the bottom of the dialog.  If you delete or change \
-the path shown in the field, the file will be saved using whatever path \
-you type provided that it is a valid Unix file specification.\n\
-\n\
-To replace an existing file, select it from the Files list \
-and press \"OK\", or simply double click on the name.\n\
-\n\
-If you would like to save a file in another directory, either type the \
-full name in the \"Save File As:\" field, or type the directory name \
-in the \"File Filter\" field and press the \
-\"Filter\" button to see that directory in the file list.";
-#endif
-
 #else /* SGI_CUSTOM */
-static char *HelpExist =
+static const char *HelpExist =
 "The \"File to Edit:\" field shows a list of directories and files in the \
 current directory.\n\
 \n\
@@ -193,7 +168,8 @@ directories.\n\
 The \"Filter\" button allows you to narrow down the list of files and \
 directories shown in the \"File to Edit:\" field.  The default filter of \
 \"*\" allows all files to be listed.";
-static char *HelpNew = 
+
+static const char *HelpNew = 
 "This dialog allows you to create a new file or to save the current file \
 under a new name.\n\
 \n\
@@ -234,16 +210,16 @@ static void newFileCancelCB(Widget w, Boolean *client_data, caddr_t
 static void newHelpCB(Widget w, Widget helpPanel, caddr_t call_data);
 static void createYesNoDialog(Widget parent);
 static void createErrorDialog(Widget parent);
-static int doYesNoDialog(char *msg);
-static void doErrorDialog(char *errorString, char *filename);
+static int  doYesNoDialog(const char *msg);
+static void doErrorDialog(const char *errorString, const char *filename);
 static void existOkCB(Widget w, Boolean * client_data,
-	       XmFileSelectionBoxCallbackStruct *call_data);
+	              XmFileSelectionBoxCallbackStruct *call_data);
 static void existCancelCB(Widget w, Boolean * client_data, caddr_t call_data);
 static void existHelpCB(Widget w, Widget helpPanel, caddr_t call_data);
 static void errorOKCB(Widget w, caddr_t client_data, caddr_t call_data);
 static void yesNoOKCB(Widget w, caddr_t client_data, caddr_t call_data);
 static void yesNoCancelCB(Widget w, caddr_t client_data, caddr_t call_data);
-static Widget createPanelHelp(Widget parent, char *text, char *title);
+static Widget createPanelHelp(Widget parent, const char *text, const char *title);
 static void helpDismissCB(Widget w, Widget helpPanel, caddr_t call_data);
 static void makeListTypeable(Widget listW);
 static void listCharEH(Widget w, XtPointer callData, XEvent *event,
@@ -299,7 +275,7 @@ int GetExistingFilename (Widget parent, char *promptString, char *filename)
     XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
     XtSetArg(args[n], XmNdialogTitle, titleString); n++;
     XtSetArg(args[n], XmNresizePolicy, XmRESIZE_GROW); n++;
-    existFileSB = XmCreateFileSelectionDialog(parent,"FileSelect",args,n);
+    existFileSB = CreateFileSelectionDialog(parent,"FileSelect",args,n);
     XmStringFree(labelString);
     XmStringFree(titleString);
 #ifndef SGI_CUSTOM
@@ -309,14 +285,14 @@ int GetExistingFilename (Widget parent, char *promptString, char *filename)
     	    XmDIALOG_SELECTION_LABEL));
     XtVaSetValues(XmFileSelectionBoxGetChild(existFileSB,
     	    XmDIALOG_FILTER_LABEL), XmNmnemonic, 'l', XmNuserData,
-    	    XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_FILTER_TEXT), 0);
+    	    XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_FILTER_TEXT), NULL);
     XtVaSetValues(XmFileSelectionBoxGetChild(existFileSB,
     	    XmDIALOG_DIR_LIST_LABEL), XmNmnemonic, 'D', XmNuserData,
-    	    XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_DIR_LIST), 0);
+    	    XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_DIR_LIST), NULL);
     XtVaSetValues(XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_LIST_LABEL),
     	    XmNmnemonic, promptString[strspn(promptString, "lD")], XmNuserData,
-    	    XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_LIST), 0);
-    AddDialogMnemonicHandler(existFileSB);
+    	    XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_LIST), NULL);
+    AddDialogMnemonicHandler(existFileSB, FALSE);
     RemapDeleteKey(XmFileSelectionBoxGetChild(existFileSB,
                     XmDIALOG_FILTER_TEXT));
     RemapDeleteKey(XmFileSelectionBoxGetChild(existFileSB,
@@ -349,6 +325,7 @@ int HandleCustomExistFileSB(Widget existFileSB, char *filename)
 {
     Boolean   done_with_dialog=False; /* ok to destroy dialog flag	   */
     char      *fileString;            /* C string for file selected        */
+    char      *dirString;             /* C string for dir of file selected */
     XmString  cFileString;            /* compound string for file selected */
     XmString  cDir;	              /* compound directory selected	   */
     XmString  cPattern;               /* compound filter pattern	   */
@@ -369,13 +346,13 @@ int HandleCustomExistFileSB(Widget existFileSB, char *filename)
     	    (char *)help);
     if (DefaultDirectory != NULL || DefaultPattern != NULL)
     	XtVaSetValues(existFileSB, XmNdirectory, DefaultDirectory,
-    		XmNpattern, DefaultPattern, 0);
+    		XmNpattern, DefaultPattern, NULL);
 #ifndef SGI_CUSTOM
     makeListTypeable(XmFileSelectionBoxGetChild(existFileSB,XmDIALOG_LIST));
     makeListTypeable(XmFileSelectionBoxGetChild(existFileSB,XmDIALOG_DIR_LIST));
 #if XmVersion >= 1002
     XtVaSetValues(existFileSB, XmNinitialFocus, XtParent(
-    	    XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_LIST)), 0);
+    	    XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_LIST)), NULL);
 #endif
 #endif
     ManageDialogCenteredOnPointer(existFileSB);
@@ -388,9 +365,9 @@ int HandleCustomExistFileSB(Widget existFileSB, char *filename)
        To sort them properly, we have to patch the directory and file
        searching routines to re-sort the lists when they change */
     XtVaGetValues(existFileSB, XmNdirSearchProc, &OrigDirSearchProc,
-    	    XmNfileSearchProc, &OrigFileSearchProc, 0);
+    	    XmNfileSearchProc, &OrigFileSearchProc, NULL);
     XtVaSetValues(existFileSB, XmNdirSearchProc, replacementDirSearchProc,
-    	    XmNfileSearchProc, replacementFileSearchProc, 0);
+    	    XmNfileSearchProc, replacementFileSearchProc, NULL);
     sortWidgetList(XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_DIR_LIST));
     sortWidgetList(XmFileSelectionBoxGetChild(existFileSB, XmDIALOG_LIST));
 #if XmVersion < 1002
@@ -399,7 +376,7 @@ int HandleCustomExistFileSB(Widget existFileSB, char *filename)
        button after the keyboard focus is established.  Note the voodoo
        below: calling XmProcess traversal extra times (a recommendation from
        OSF technical support) somehow succeedes in giving the file list focus */
-    XtVaSetValues(existFileSB, XmNdefaultButton, NULL, 0);
+    XtVaSetValues(existFileSB, XmNdefaultButton, NULL, NULL);
     for (i=1; i<30; i++)
     	XmProcessTraversal(XmFileSelectionBoxGetChild(existFileSB,
     		XmDIALOG_LIST), XmTRAVERSE_CURRENT);
@@ -411,7 +388,7 @@ int HandleCustomExistFileSB(Widget existFileSB, char *filename)
     
     if (SelectResult == GFN_OK) {
 	XtVaGetValues(existFileSB, XmNdirSpec, &cFileString, XmNdirectory,
-		&cDir, XmNpattern, &cPattern, 0);
+		&cDir, XmNpattern, &cPattern, NULL);
 	/* Undocumented: file selection box widget allocates copies of these
 	   strings on getValues calls.  I have risked freeing them to avoid
 	   memory leaks, since I assume other developers have made this same
@@ -421,7 +398,29 @@ int HandleCustomExistFileSB(Widget existFileSB, char *filename)
 	DefaultDirectory = cDir;
 	DefaultPattern = cPattern;
 	XmStringGetLtoR(cFileString, XmSTRING_DEFAULT_CHARSET, &fileString);
-	strcpy(filename, fileString);
+        /* Motif 2.x seem to contain a bug that causes it to return only 
+           the relative name of the file in XmNdirSpec when XmNpathMode is set
+           to XmPATH_MODE_RELATIVE (through X resources), although the man
+           page states that it always returns the full path name. We can
+           easily work around this by checking that the first character of the
+           file name is a `/'. */
+#ifdef VMS
+       /* VMS  won't return `/' as the 1st character of the full file spec.
+         `:' terminates the device name and is not allowed elsewhere */
+        if (strchr(fileString, ':') != NULL) {
+#else
+        if (fileString[0] == '/') {
+#endif        /* VMS */
+	    /* The directory name is already present in the file name or
+	       the user entered a full path name. */
+	    strcpy(filename, fileString);
+	} else {
+	    /* Concatenate the directory name and the file name */
+   	    XmStringGetLtoR(cDir, XmSTRING_DEFAULT_CHARSET, &dirString);
+	    strcpy(filename, dirString);
+	    strcat(filename, fileString);
+  	    XtFree(dirString);
+	}
 	XmStringFree(cFileString);
 	XtFree(fileString);
     }
@@ -468,30 +467,26 @@ int GetNewFilename (Widget parent, char *promptString, char *filename)
                   XmSTRING_DEFAULT_CHARSET);
     titleString = XmStringCreateLtoR (" ", XmSTRING_DEFAULT_CHARSET);
     XtSetArg(args[n], XmNselectionLabelString, labelString); n++;     
-#ifdef MOTIF10
-    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
-#else
     XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
-#endif
     XtSetArg(args[n], XmNdialogTitle, titleString); n++;
     XtSetArg(args[n], XmNresizePolicy, XmRESIZE_GROW); n++;
-    newFileSB=XmCreateFileSelectionDialog(parent,"FileSelect",args,n);
+    newFileSB=CreateFileSelectionDialog(parent,"FileSelect",args,n);
     XmStringFree(labelString);
     XmStringFree(titleString);
     XtVaSetValues(XmFileSelectionBoxGetChild(newFileSB,
     	    XmDIALOG_FILTER_LABEL), XmNmnemonic, 'l', XmNuserData,
-    	    XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_FILTER_TEXT), 0);
+    	    XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_FILTER_TEXT), NULL);
     XtVaSetValues(XmFileSelectionBoxGetChild(newFileSB,
     	    XmDIALOG_DIR_LIST_LABEL), XmNmnemonic, 'D', XmNuserData,
-    	    XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_DIR_LIST), 0);
+    	    XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_DIR_LIST), NULL);
     XtVaSetValues(XmFileSelectionBoxGetChild(newFileSB,
     	    XmDIALOG_LIST_LABEL), XmNmnemonic, 'F', XmNuserData,
-    	    XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_LIST), 0);
+    	    XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_LIST), NULL);
     XtVaSetValues(XmFileSelectionBoxGetChild(newFileSB,
     	    XmDIALOG_SELECTION_LABEL), XmNmnemonic,
     	    promptString[strspn(promptString, "lFD")], XmNuserData,
-    	    XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_TEXT), 0);
-    AddDialogMnemonicHandler(newFileSB);
+    	    XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_TEXT), NULL);
+    AddDialogMnemonicHandler(newFileSB, FALSE);
     RemapDeleteKey(XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_FILTER_TEXT));
     RemapDeleteKey(XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_TEXT));
     return HandleCustomNewFileSB(newFileSB, filename, NULL);
@@ -527,6 +522,7 @@ int HandleCustomNewFileSB(Widget newFileSB, char *filename, char *defaultName)
     XmString  cDir;	              /* compound directory selected	   */
     XmString  cPattern;               /* compound filter pattern	   */
     char      *fileString;            /* C string for file selected        */
+    char      *dirString;             /* C string for dir of file selected */
 #if XmVersion < 1002
     int       i;
 #endif
@@ -542,7 +538,7 @@ int HandleCustomNewFileSB(Widget newFileSB, char *filename, char *defaultName)
 #endif
     if (DefaultDirectory != NULL || DefaultPattern != NULL)
     	XtVaSetValues(newFileSB, XmNdirectory, DefaultDirectory,
-    		XmNpattern, DefaultPattern, 0);
+    		XmNpattern, DefaultPattern, NULL);
     help = createPanelHelp(newFileSB, HelpNew, "Saving a File");
     createYesNoDialog(newFileSB);
     createErrorDialog(newFileSB);
@@ -555,11 +551,11 @@ int HandleCustomNewFileSB(Widget newFileSB, char *filename, char *defaultName)
 #if XmVersion >= 1002
 #ifndef SGI_CUSTOM
     XtVaSetValues(newFileSB, XmNinitialFocus, 
-    	    XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_TEXT), 0);
+    	    XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_TEXT), NULL);
 #else /* SGI_CUSTOM */
     { Widget finder = XmFileSelectionBoxGetChild(newFileSB, SgDIALOG_FINDER);
       if ( finder != NULL )
-    	  XtVaSetValues(newFileSB, XmNinitialFocus, finder, 0);
+    	  XtVaSetValues(newFileSB, XmNinitialFocus, finder, NULL);
     }
 #endif
 #endif
@@ -572,7 +568,7 @@ int HandleCustomNewFileSB(Widget newFileSB, char *filename, char *defaultName)
        button after the keyboard focus is established.  Note the voodoo
        below: calling XmProcess traversal FOUR times (a recommendation from
        OSF technical support) somehow succeedes in changing the focus */
-    XtVaSetValues(newFileSB, XmNdefaultButton, NULL, 0);
+    XtVaSetValues(newFileSB, XmNdefaultButton, NULL, NULL);
     for (i=1; i<30; i++)
     	XmProcessTraversal(XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_TEXT),
 	    XmTRAVERSE_CURRENT);
@@ -585,9 +581,9 @@ int HandleCustomNewFileSB(Widget newFileSB, char *filename, char *defaultName)
        To sort them properly, we have to patch the directory and file
        searching routines to re-sort the lists when they change */
     XtVaGetValues(newFileSB, XmNdirSearchProc, &OrigDirSearchProc,
-    	    XmNfileSearchProc, &OrigFileSearchProc, 0);
+    	    XmNfileSearchProc, &OrigFileSearchProc, NULL);
     XtVaSetValues(newFileSB, XmNdirSearchProc, replacementDirSearchProc,
-    	    XmNfileSearchProc, replacementFileSearchProc, 0);
+    	    XmNfileSearchProc, replacementFileSearchProc, NULL);
     sortWidgetList(XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_DIR_LIST));
     sortWidgetList(XmFileSelectionBoxGetChild(newFileSB, XmDIALOG_LIST));
 #endif /* SGI_CUSTOM */
@@ -598,13 +594,24 @@ int HandleCustomNewFileSB(Widget newFileSB, char *filename, char *defaultName)
 	/* See note in existing file routines about freeing the values
 	   obtained in the following call */
 	XtVaGetValues(newFileSB, XmNdirSpec, &cFileString, XmNdirectory,
-		&cDir, XmNpattern, &cPattern, 0);
+		&cDir, XmNpattern, &cPattern, NULL);
 	if (DefaultDirectory != NULL) XmStringFree(DefaultDirectory);
 	if (DefaultPattern != NULL) XmStringFree(DefaultPattern);
 	DefaultDirectory = cDir;
 	DefaultPattern = cPattern;
 	XmStringGetLtoR(cFileString, XmSTRING_DEFAULT_CHARSET, &fileString);
-	strcpy(filename, fileString);
+	/* See note in existing file routines about Motif 2.x bug. */
+	if (fileString[0] == '/') {
+	    /* The directory name is already present in the file name or
+	       the user entered a full path name. */
+	    strcpy(filename, fileString);
+	} else {
+	    /* Concatenate the directory name and the file name */
+   	    XmStringGetLtoR(cDir, XmSTRING_DEFAULT_CHARSET, &dirString);
+	    strcpy(filename, dirString);
+	    strcat(filename, fileString);
+  	    XtFree(dirString);
+	}
 	XmStringFree(cFileString);
 	XtFree(fileString);
     }
@@ -698,13 +705,9 @@ static void createYesNoDialog(Widget parent)
     Arg       args[MAX_ARGS];	      /* arg list                          */
 
     n = 0;
-#ifdef MOTIF10
-    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
-#else
     XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
-#endif
     XtSetArg(args[n], XmNtitle, " "); n++;
-    YesNoDialog = XmCreateQuestionDialog(parent, "yesNo", args, n);
+    YesNoDialog = CreateQuestionDialog(parent, "yesNo", args, n);
     XtAddCallback (YesNoDialog, XmNokCallback, (XtCallbackProc)yesNoOKCB, NULL);
     XtAddCallback (YesNoDialog, XmNcancelCallback,
     	    (XtCallbackProc)yesNoCancelCB, NULL);
@@ -724,23 +727,19 @@ static void createErrorDialog(Widget parent)
     Arg       args[MAX_ARGS];	      /* arg list                          */
 
     n = 0;
-#ifdef MOTIF10
-    XtSetArg(args[n], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); n++;
-#else
     XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
-#endif
     XtSetArg(args[n], XmNtitle, " "); n++;
-    ErrorDialog = XmCreateErrorDialog(parent, "error", args, n);
+    ErrorDialog = CreateErrorDialog(parent, "error", args, n);
     XtAddCallback(ErrorDialog, XmNcancelCallback, (XtCallbackProc)errorOKCB,
     	    NULL);
     XtUnmanageChild(XmMessageBoxGetChild(ErrorDialog, XmDIALOG_OK_BUTTON));
     XtUnmanageChild(XmMessageBoxGetChild(ErrorDialog, XmDIALOG_HELP_BUTTON));
     buttonString = XmStringCreateLtoR ("Dismiss", XmSTRING_DEFAULT_CHARSET);
-    XtVaSetValues(ErrorDialog, XmNcancelLabelString, buttonString, 0);
+    XtVaSetValues(ErrorDialog, XmNcancelLabelString, buttonString, NULL);
     XmStringFree(buttonString);
 }
 
-static int doYesNoDialog(char *filename)
+static int doYesNoDialog(const char *filename)
 {
     char string[255];
     XmString mString;
@@ -771,7 +770,7 @@ static int doYesNoDialog(char *filename)
     return YesNoResult == ynYes;
 }
 
-static void doErrorDialog(char *errorString, char *filename)
+static void doErrorDialog(const char *errorString, const char *filename)
 {
     char string[255];
     XmString mString;
@@ -914,17 +913,16 @@ static void yesNoCancelCB(Widget w, caddr_t client_data, caddr_t call_data)
     YesNoResult = ynNo;
 }
 
-static Widget createPanelHelp(Widget parent, char *helpText, char *title)
+static Widget createPanelHelp(Widget parent, const char *helpText, const char *title)
 {
-    Arg al[50];
+    Arg al[20];
     int ac;
     Widget form, text, button;
     XmString st1;
     
     ac = 0;
-    form = XmCreateFormDialog(parent, "helpForm", al, ac);
+    form = CreateFormDialog(parent, "helpForm", al, ac);
 
-#ifndef MOTIF10
     ac = 0;
     XtSetArg (al[ac], XmNbottomAttachment, XmATTACH_FORM);  ac++;
     XtSetArg (al[ac], XmNtopAttachment, XmATTACH_NONE);  ac++;
@@ -936,7 +934,6 @@ static Widget createPanelHelp(Widget parent, char *helpText, char *title)
     XmStringFree(st1);
     XtManageChild(button);
     SET_ONE_RSRC(form, XmNdefaultButton, button);
-#endif
     
     ac = 0;
     XtSetArg(al[ac], XmNrows, 15);  ac++;
@@ -948,13 +945,11 @@ static Widget createPanelHelp(Widget parent, char *helpText, char *title)
     XtSetArg(al[ac], XmNeditMode, XmMULTI_LINE_EDIT);  ac++;
     XtSetArg(al[ac], XmNeditable, False);  ac++;
     XtSetArg(al[ac], XmNvalue, helpText);  ac++;
-#ifndef MOTIF10
     XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM);  ac++;
     XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM);  ac++;
     XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_WIDGET);  ac++;
     XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM);  ac++;
     XtSetArg(al[ac], XmNbottomWidget, button);  ac++;
-#endif
     text = XmCreateScrolledText(form, "helpText", al, ac);
     XtManageChild(text);
     
@@ -1019,7 +1014,7 @@ static void listCharEH(Widget w, XtPointer callData, XEvent *event,
     	
     /* Accumulate the current keystroke, just beep if there are too many */
     if (nKeystrokes >= MAX_LIST_KEYSTROKES)
-    	XBell(XtDisplay(w), 100);
+    	XBell(XtDisplay(w), 0);
     else
 #ifdef VMS
     	keystrokes[nKeystrokes++] = toupper(c);
@@ -1028,7 +1023,7 @@ static void listCharEH(Widget w, XtPointer callData, XEvent *event,
 #endif
     
     /* Get the items (filenames) in the list widget */
-    XtVaGetValues(w, XmNitems, &items, XmNitemCount, &nItems, 0);
+    XtVaGetValues(w, XmNitems, &items, XmNitemCount, &nItems, NULL);
     
     /* compare them with the accumulated user keystrokes & decide the
        appropriate line in the list widget to select */
@@ -1052,12 +1047,12 @@ static void listCharEH(Widget w, XtPointer callData, XEvent *event,
     if (selectPos == 0) /* XmListSelectPos curiously returns 0 for last item */
     	selectPos = nItems + 1;
     XtVaGetValues(w, XmNtopItemPosition, &topPos,
-    	    XmNvisibleItemCount, &nVisible, 0);
+    	    XmNvisibleItemCount, &nVisible, NULL);
     if (selectPos < topPos)
     	XmListSetPos(w, selectPos-2 > 1 ? selectPos-2 : 1);
     else if (selectPos > topPos+nVisible-1)
     	XmListSetBottomPos(w, selectPos+2 <= nItems ? selectPos+2 : 0);
-    /* For Lesstif .89.9 */
+    /* For LessTif 0.89.9. Obsolete now? */
     XmListSelectPos(w, selectPos, True);
 }
 
@@ -1082,20 +1077,21 @@ static void replacementDirSearchProc(Widget w, XtPointer searchData)
     
     /* Call the original search procedure to do the actual search */
     (*OrigDirSearchProc)(w, searchData);
-    XtVaGetValues(w, XmNlistUpdated, &updated, 0);
+    XtVaGetValues(w, XmNlistUpdated, &updated, NULL);
     if (!updated)
     	return;
     	
     /* Sort the items in the list */
     sortWidgetList(XmFileSelectionBoxGetChild(w, XmDIALOG_DIR_LIST));
 }
+
 static void replacementFileSearchProc(Widget w, XtPointer searchData)
 {
     Boolean updated;
     
     /* Call the original search procedure to do the actual search */
     (*OrigFileSearchProc)(w, searchData);
-    XtVaGetValues(w, XmNlistUpdated, &updated, 0);
+    XtVaGetValues(w, XmNlistUpdated, &updated, NULL);
     if (!updated)
     	return;
     	
@@ -1111,7 +1107,7 @@ static void sortWidgetList(Widget listWidget)
     XmString *items, *sortedItems;
     int nItems, i;
     
-    XtVaGetValues(listWidget, XmNitems, &items, XmNitemCount, &nItems, 0);
+    XtVaGetValues(listWidget, XmNitems, &items, XmNitemCount, &nItems, NULL);
     sortedItems = (XmString *)XtMalloc(sizeof(XmString) * nItems);
     for (i=0; i<nItems; i++)
     	sortedItems[i] = XmStringCopy(items[i]);
