@@ -1,3 +1,4 @@
+static const char CVSID[] = "$Id: misc.c,v 1.27.2.2 2001/08/24 08:33:39 amai Exp $";
 /*******************************************************************************
 *									       *
 * misc.c -- Miscelaneous Motif convenience functions			       *
@@ -11,7 +12,7 @@
 * 									       *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
-* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
 * for more details.							       *
 * 									       *
 * You should have received a copy of the GNU General Public License along with *
@@ -25,7 +26,9 @@
 *									       *
 *******************************************************************************/
 #include <stdlib.h>
+#include <string.h>
 #include <stdarg.h>
+#include <ctype.h>
 #include <math.h>
 #include <stdio.h>
 #include <time.h>
@@ -37,6 +40,7 @@
 #include <X11/Intrinsic.h>
 #include <X11/Xatom.h>
 #include <X11/keysym.h>
+#include <X11/keysymdef.h>
 #include <Xm/Xm.h>
 #include <Xm/Label.h>
 #include <Xm/LabelG.h>
@@ -48,6 +52,11 @@
 #include <Xm/AtomMgr.h>
 #include <Xm/Protocols.h>
 #include <Xm/Text.h>
+#include <Xm/MessageB.h>
+#include <Xm/DialogS.h>
+#include <Xm/SelectioB.h>
+#include <Xm/Form.h>
+#include <Xm/FileSB.h>
 #include "DialogF.h"
 #include "misc.h"
 
@@ -62,12 +71,14 @@ typedef struct {
     int index;
 } histInfo;
 
+typedef Widget (*MotifDialogCreationCall)(Widget, String, ArgList, Cardinal);
+
 /* Maximum size of a history-recall list.  Typically never invoked, since
    user must first make this many entries in the text field, limited for
    safety, to the maximum reasonable number of times user can hit up-arrow
    before carpal tunnel syndrome sets in */
-#define HISTORY_LIST_TRIM_TO 3 /* 1000 */
-#define HISTORY_LIST_MAX 6 /* 2000 */
+#define HISTORY_LIST_TRIM_TO 1000
+#define HISTORY_LIST_MAX 2000
 
 /* flags to enable/disable delete key remapping and pointer centered dialogs */
 static int RemapDeleteEnabled = True;
@@ -91,16 +102,27 @@ static unsigned char watch_mask_bits[] = {
    0xf0, 0x0f, 0xf0, 0x0f, 0xf0, 0x0f, 0xf0, 0x0f
 };
 
-static void addMnemonicGrabs(Widget addTo, Widget w);
+static void addMnemonicGrabs(Widget addTo, Widget w, int unmodified);
 static void mnemonicCB(Widget w, XtPointer callData, XKeyEvent *event);
 static void findAndActivateMnemonic(Widget w, unsigned int keycode);
+static void addAccelGrabs(Widget topWidget, Widget w);
+static void addAccelGrab(Widget topWidget, Widget w);
+static int parseAccelString(Display *display, const char *string, KeySym *keysym,
+	unsigned int *modifiers);
+static void lockCB(Widget w, XtPointer callData, XEvent *event,
+	Boolean *continueDispatch);
+static int findAndActivateAccel(Widget w, unsigned int keyCode,
+	unsigned int modifiers, XEvent *event);
 static void removeWhiteSpace(char *string);
+static int stripCaseCmp(const char *str1, const char *str2);
 static void warnHandlerCB(String message);
 static void passwdCB(Widget w, char * passTxt, XmTextVerifyCallbackStruct
 	*txtVerStr);
 static void histDestroyCB(Widget w, XtPointer clientData, XtPointer callData);
 static void histArrowKeyEH(Widget w, XtPointer callData, XEvent *event,
 	Boolean *continueDispatch);
+static Widget addParentVisArgsAndCall(MotifDialogCreationCall callRoutine,
+	Widget parent, char *name, ArgList arglist, Cardinal  argcount);
 
 /*
 ** Set up closeCB to be called when the user selects close from the
@@ -113,7 +135,7 @@ void AddMotifCloseCallback(Widget shell, XtCallbackProc closeCB, void *arg)
     Display *display = XtDisplay(shell);
 
     /* deactivate the built in delete response of killing the application */
-    XtVaSetValues(shell, XmNdeleteResponse, XmDO_NOTHING, 0);
+    XtVaSetValues(shell, XmNdeleteResponse, XmDO_NOTHING, NULL);
 
     /* add a delete window protocol callback instead */
     if (dwAtom == 0) {
@@ -198,8 +220,8 @@ void RealizeWithoutForcingPosition(Widget shell)
     
     /* Temporarily set value of XmNmappedWhenManaged
        to stop the window from popping up right away */
-    XtVaGetValues(shell, XmNmappedWhenManaged, &mappedWhenManaged, 0);
-    XtVaSetValues(shell, XmNmappedWhenManaged, False, 0);
+    XtVaGetValues(shell, XmNmappedWhenManaged, &mappedWhenManaged, NULL);
+    XtVaSetValues(shell, XmNmappedWhenManaged, False, NULL);
     
     /* Realize the widget in unmapped state */
     XtRealizeWidget(shell);
@@ -216,7 +238,317 @@ void RealizeWithoutForcingPosition(Widget shell)
     XtMapWidget(shell);
     
     /* Restore the value of XmNmappedWhenManaged */
-    XtVaSetValues(shell, XmNmappedWhenManaged, mappedWhenManaged, 0);
+    XtVaSetValues(shell, XmNmappedWhenManaged, mappedWhenManaged, NULL);
+}
+
+/*
+** Older X applications and X servers were mostly designed to operate with
+** visual class PseudoColor, because older displays were at most 8 bits
+** deep.  Modern X servers, however, usually support 24 bit depth and other
+** color models.  Sun (and others?) still sets their default visual to
+** 8-bit PseudoColor, because some of their X applications don't work
+** properly with the other color models.  The problem with PseudoColor, of
+** course, is that users run out of colors in the default colormap, and if
+** they install additional colormaps for individual applications, colors
+** flash and change weirdly when you change your focus from one application
+** to another.
+**
+** In addition to the poor choice of default, a design flaw in Xt makes it
+** impossible even for savvy users to specify the XtNvisual resource to
+** switch to a deeper visual.  The problem is that the colormap resource is
+** processed independently of the visual resource, and usually results in a
+** colormap for the default visual rather than for the user-selected one.
+**
+** This routine should be called before creating a shell widget, to
+** pre-process the visual, depth, and colormap resources, and return the
+** proper values for these three resources to be passed to XtAppCreateShell.
+** Applications which actually require a particular color model (i.e. for
+** doing color table animation or dynamic color assignment) should not use
+** this routine.
+**
+** Note that a consequence of using the "best" as opposed to the default
+** visual is that some color resources are still converted with the default
+** visual (particularly *background), and these must be avoided by widgets
+** which are allowed to handle any visual.
+*/
+void FindBestVisual(Display *display, const char *appName, char *appClass,
+	Visual **visual, int *depth, Colormap *colormap)
+{
+    char rsrcName[256], rsrcClass[256], *valueString, *type, *endPtr;
+    XrmValue value;
+    int screen = DefaultScreen(display);
+    int reqDepth = -1;
+    int reqID = -1;
+    int reqClass = -1;
+    int installColormap = FALSE;
+    int maxDepth, bestClass, bestVisual, nVis, i, j;
+    XVisualInfo visTemplate, *visList = NULL;
+    static Visual *cachedVisual = NULL;
+    static Colormap cachedColormap;
+    static int cachedDepth = 0;
+    int bestClasses[] = {StaticGray, GrayScale, StaticColor, PseudoColor,
+    	    DirectColor, TrueColor};
+
+    /* If results have already been computed, just return them */
+    if (cachedVisual != NULL) {
+	*visual = cachedVisual;
+	*depth = cachedDepth;
+	*colormap = cachedColormap;
+	return;
+    }
+    
+    /* Read the visualID and installColormap resources for the application.
+       visualID can be specified either as a number (the visual id as
+       shown by xdpyinfo), as a visual class name, or as Best or Default. */
+    sprintf(rsrcName,"%s.%s", appName, "visualID");
+    sprintf(rsrcClass, "%s.%s", appClass, "VisualID");
+    if (XrmGetResource(XtDatabase(display), rsrcName, rsrcClass, &type,
+	    &value)) {
+	valueString = value.addr;
+	reqID = (int)strtol(valueString, &endPtr, 0);
+	if (endPtr == valueString) {
+	    reqID = -1;
+	    if (stripCaseCmp(valueString, "Default"))
+		reqID = DefaultVisual(display, screen)->visualid;
+	    else if (stripCaseCmp(valueString, "StaticGray"))
+		reqClass = StaticGray;
+	    else if (stripCaseCmp(valueString, "StaticColor"))
+		reqClass = StaticColor;
+	    else if (stripCaseCmp(valueString, "TrueColor"))
+		reqClass = TrueColor;
+	    else if (stripCaseCmp(valueString, "GrayScale"))
+		reqClass = GrayScale;
+	    else if (stripCaseCmp(valueString, "PseudoColor"))
+		reqClass = PseudoColor;
+	    else if (stripCaseCmp(valueString, "DirectColor"))
+		reqClass = DirectColor;
+	    else if (!stripCaseCmp(valueString, "Best"))
+		fprintf(stderr, "Invalid visualID resource value\n");
+	}
+    }
+    sprintf(rsrcName,"%s.%s", appName, "installColormap");
+    sprintf(rsrcClass, "%s.%s", appClass, "InstallColormap");
+    if (XrmGetResource(XtDatabase(display), rsrcName, rsrcClass, &type,
+	    &value)) {
+	if (stripCaseCmp(value.addr, "Yes") || stripCaseCmp(value.addr, "True"))
+	    installColormap = TRUE;
+    }
+    
+    /* Generate a list of visuals to consider.  (Note, vestigial code for
+       user-requested visual depth is left in, just in case that function
+       might be needed again, but it does nothing)  */
+    if (reqID != -1) {
+	visTemplate.visualid = reqID;
+	visList = XGetVisualInfo(display, VisualIDMask, &visTemplate, &nVis);
+	if (visList == NULL)
+	    fprintf(stderr, "VisualID resource value not valid\n");
+    }
+    if (visList == NULL && reqClass != -1 && reqDepth != -1) {
+	visTemplate.class = reqClass;
+	visTemplate.depth = reqDepth;
+    	visList = XGetVisualInfo(display,
+		VisualClassMask | VisualDepthMask, &visTemplate, &nVis);
+    	if (visList == NULL)
+	    fprintf(stderr, "Visual class/depth combination not available\n");
+    }
+    if (visList == NULL && reqClass != -1) {
+ 	visTemplate.class = reqClass;
+    	visList = XGetVisualInfo(display, VisualClassMask, &visTemplate, &nVis);
+    	if (visList == NULL)
+	    fprintf(stderr,
+		    "Visual Class from resource \"visualID\" not available\n");
+    }
+    if (visList == NULL && reqDepth != -1) {
+	visTemplate.depth = reqDepth;
+	visTemplate.depth = reqDepth;
+    	visList = XGetVisualInfo(display, VisualDepthMask, &visTemplate, &nVis);
+    	if (visList == NULL)
+	    fprintf(stderr, "Requested visual depth not available\n");
+    }
+    if (visList == NULL) {
+	visList = XGetVisualInfo(display, VisualNoMask, &visTemplate, &nVis);
+	if (visList == NULL) {
+	    fprintf(stderr, "Internal Error: no visuals available?\n");
+	    *visual = DefaultVisual(display, screen);
+	    *depth =  DefaultDepth(display, screen);
+    	    *colormap = DefaultColormap(display, screen);
+    	    return;
+	}
+    }
+    
+    /* Choose among the visuals in the candidate list.  Prefer maximum
+       depth first then matching default, then largest value of bestClass
+       (I'm not sure whether we actually care about class) */
+    maxDepth = 0;
+    bestClass = 0;
+    bestVisual = 0;
+    for (i=0; i < nVis; i++) {
+	if (visList[i].depth > maxDepth) {
+	    maxDepth = visList[i].depth;
+	    bestClass = 0;
+	    bestVisual = i;
+	}
+	if (visList[i].depth == maxDepth) {
+	    if (visList[i].visual == DefaultVisual(display, screen))
+		bestVisual = i;
+	    if (visList[bestVisual].visual != DefaultVisual(display, screen)) {
+		for (j = 0; j < XtNumber(bestClasses); j++) {
+		    if (visList[i].class == bestClasses[j] && j > bestClass) {
+			bestClass = j;
+			bestVisual = i;
+		    }
+		}
+	    }
+	}
+    }
+    *visual = cachedVisual = visList[bestVisual].visual;
+    *depth = cachedDepth = visList[bestVisual].depth;
+    
+    /* If the chosen visual is not the default, it needs a colormap allocated */
+    if (*visual == DefaultVisual(display, screen) && !installColormap)
+	*colormap = cachedColormap = DefaultColormap(display, screen);
+    else {
+	*colormap = cachedColormap = XCreateColormap(display,
+		RootWindow(display, screen), cachedVisual, AllocNone);
+	XInstallColormap(display, cachedColormap);
+    }
+    /* printf("Chose visual with depth %d, class %d, colormap %ld, id 0x%x\n",
+	    visList[bestVisual].depth, visList[bestVisual].class,
+	    *colormap, cachedVisual->visualid); */
+    /* Fix memory leak */
+    if (visList != NULL) {
+       XFree(visList);
+    }
+}
+
+/*
+** If you want to use a non-default visual with Motif, shells all have to be
+** created with that visual, depth, and colormap, even if the parent has them
+** set up properly. Substituting these routines, will append visual args copied
+** from the parent widget (CreatePopupMenu and CreatePulldownMenu), or from the
+** best visual, obtained via FindBestVisual above (CreateShellWithBestVis).
+*/
+Widget CreateDialogShell(Widget parent, char *name,
+	ArgList arglist, Cardinal  argcount)
+{
+    return addParentVisArgsAndCall(XmCreateDialogShell, parent, name, arglist,
+	    argcount);
+}
+Widget CreatePopupMenu(Widget parent, char *name, ArgList arglist,
+	Cardinal argcount)
+{
+    return addParentVisArgsAndCall(XmCreatePopupMenu, parent, name,
+	    arglist, argcount);
+}
+Widget CreatePulldownMenu(Widget parent, char *name,
+	ArgList arglist, Cardinal  argcount)
+{
+    return addParentVisArgsAndCall(XmCreatePulldownMenu, parent, name, arglist,
+	    argcount);
+}
+Widget CreatePromptDialog(Widget parent, char *name,
+	ArgList arglist, Cardinal  argcount)
+{
+    return addParentVisArgsAndCall(XmCreatePromptDialog, parent, name, arglist,
+	    argcount);
+}
+Widget CreateSelectionDialog(Widget parent, char *name,
+	ArgList arglist, Cardinal  argcount)
+{
+    return addParentVisArgsAndCall(XmCreateSelectionDialog, parent, name,
+	    arglist, argcount);
+}
+Widget CreateFormDialog(Widget parent, char *name,
+	ArgList arglist, Cardinal  argcount)
+{
+    return addParentVisArgsAndCall(XmCreateFormDialog, parent, name, arglist,
+	    argcount);
+}
+Widget CreateFileSelectionDialog(Widget parent, char *name,
+	ArgList arglist, Cardinal  argcount)
+{
+    return addParentVisArgsAndCall(XmCreateFileSelectionDialog, parent, name,
+	    arglist, argcount);
+}
+Widget CreateQuestionDialog(Widget parent, char *name,
+	ArgList arglist, Cardinal  argcount)
+{
+    return addParentVisArgsAndCall(XmCreateQuestionDialog, parent, name,
+	    arglist, argcount);
+}
+Widget CreateMessageDialog(Widget parent, char *name,
+	ArgList arglist, Cardinal  argcount)
+{
+    return addParentVisArgsAndCall(XmCreateMessageDialog, parent, name,
+	    arglist, argcount);
+}
+Widget CreateErrorDialog(Widget parent, char *name,
+	ArgList arglist, Cardinal  argcount)
+{
+    return addParentVisArgsAndCall(XmCreateErrorDialog, parent, name, arglist,
+	    argcount);
+}
+Widget CreateShellWithBestVis(String appName, String appClass, 
+	   WidgetClass class, Display *display, ArgList args, Cardinal nArgs)
+{
+    Visual *visual;
+    int depth;
+    Colormap colormap;
+    ArgList al;
+    Cardinal ac = nArgs;
+    Widget result;
+
+    FindBestVisual(display, appName, appClass, &visual, &depth, &colormap);
+    al = (ArgList)XtMalloc(sizeof(Arg) * (nArgs + 3));
+    if (nArgs != 0)
+    	memcpy(al, args, sizeof(Arg) * nArgs);
+    XtSetArg(al[ac], XtNvisual, visual); ac++;
+    XtSetArg(al[ac], XtNdepth, depth); ac++;
+    XtSetArg(al[ac], XtNcolormap, colormap); ac++;
+    result = XtAppCreateShell(appName, appClass, class, display, al, ac);
+    XtFree((char *)al);
+    return result;
+}
+
+/*
+** Calls one of the Motif widget creation routines, splicing in additional
+** arguments for visual, colormap, and depth.
+*/
+static Widget addParentVisArgsAndCall(MotifDialogCreationCall createRoutine,
+	Widget parent, char *name, ArgList arglist, Cardinal  argcount)
+{
+    Visual *visual;
+    int depth;
+    Colormap colormap;
+    ArgList al;
+    Cardinal ac = argcount;
+    Widget result;
+    Widget parentShell = parent;
+    
+    /* Find the application/dialog/menu shell at the top of the widget
+       hierarchy, which has the visual resource being used */
+    while (True) {
+    	if (XtIsShell(parentShell))
+    	    break;
+    	if (parentShell == NULL) {
+	    fprintf(stderr, "failed to find shell\n");
+	    exit(EXIT_FAILURE);
+	}
+    	parentShell = XtParent(parentShell);
+    }
+
+    /* Add the visual, depth, and colormap resources to the argument list */
+    XtVaGetValues(parentShell, XtNvisual, &visual, XtNdepth, &depth,
+	    XtNcolormap, &colormap, NULL);
+    al = (ArgList)XtMalloc(sizeof(Arg) * (argcount + 3));
+    if (argcount != 0)
+    	memcpy(al, arglist, sizeof(Arg) * argcount);
+    XtSetArg(al[ac], XtNvisual, visual); ac++;
+    XtSetArg(al[ac], XtNdepth, depth); ac++;
+    XtSetArg(al[ac], XtNcolormap, colormap); ac++;
+    result = (*createRoutine)(parent, name, al, ac);
+    XtFree((char *)al);
+    return result;
 }
 
 /*
@@ -243,8 +575,8 @@ void ManageDialogCenteredOnPointer(Widget dialogChild)
     
     /* Temporarily set value of XmNmappedWhenManaged
        to stop the dialog from popping up right away */
-    XtVaGetValues(shell, XmNmappedWhenManaged, &mappedWhenManaged, 0);
-    XtVaSetValues(shell, XmNmappedWhenManaged, False, 0);
+    XtVaGetValues(shell, XmNmappedWhenManaged, &mappedWhenManaged, NULL);
+    XtVaSetValues(shell, XmNmappedWhenManaged, False, NULL);
     
     /* Manage the dialog */
     XtManageChild(dialogChild);
@@ -277,7 +609,7 @@ void ManageDialogCenteredOnPointer(Widget dialogChild)
     XtMapWidget(shell);
     
     /* Restore the value of XmNmappedWhenManaged */
-    XtVaSetValues(shell, XmNmappedWhenManaged, mappedWhenManaged, 0);
+    XtVaSetValues(shell, XmNmappedWhenManaged, mappedWhenManaged, NULL);
 }
 
 /*
@@ -333,11 +665,11 @@ void RaiseWindow(Display *display, Window w)
 ** the XmNuserData resource of the label to the widget to get the focus
 ** when the mnemonic is typed.
 */
-void AddDialogMnemonicHandler(Widget dialog)
+void AddDialogMnemonicHandler(Widget dialog, int unmodifiedToo)
 {
     XtAddEventHandler(dialog, KeyPressMask, False,
     	    (XtEventHandler)mnemonicCB, (XtPointer)0);
-    addMnemonicGrabs(dialog, dialog);
+    addMnemonicGrabs(dialog, dialog, unmodifiedToo);
 }
 
 /*
@@ -350,6 +682,35 @@ void RemoveDialogMnemonicHandler(Widget dialog)
     	    (XtEventHandler)mnemonicCB, (XtPointer)0);
 }
 
+/*
+** Patch around Motif's poor handling of menu accelerator keys.  Motif
+** does not process menu accelerators when the caps lock or num lock
+** keys are engaged.  To enable accelerators in these cases, call this
+** routine with the completed menu bar widget as "topMenuContainer", and
+** the top level shell widget as "topWidget".  It will add key grabs for
+** all of the accelerators it finds in the topMenuContainer menu tree, and
+** an event handler which can process dropped accelerator events by (again)
+** traversing the menu tree looking for matching accelerators, and invoking
+** the appropriate button actions.  Any dynamic additions to the menus
+** require a call to UpdateAccelLockPatch to add the additional grabs.
+** Unfortunately, these grabs can not be removed.
+*/
+void AccelLockBugPatch(Widget topWidget, Widget topMenuContainer)
+{
+    XtAddEventHandler(topWidget, KeyPressMask, False, lockCB, topMenuContainer);
+    addAccelGrabs(topWidget, topMenuContainer);
+}
+
+/*
+** Add additional key grabs for new menu items added to the menus, for
+** patching around the Motif Caps/Num Lock problem. "topWidget" must be
+** the same widget passed in the original call to AccelLockBugPatch.
+*/
+void UpdateAccelLockPatch(Widget topWidget, Widget newButton)
+{
+    addAccelGrab(topWidget, newButton);
+}
+
 /*
 ** PopDownBugPatch
 **
@@ -469,15 +830,32 @@ void FreeStringTable(XmString *table)
 */ 
 void SimulateButtonPress(Widget widget)
 {
-    XKeyPressedEvent keyEvent;
+    XEvent keyEvent;
     
     memset((char *)&keyEvent, 0, sizeof(XKeyPressedEvent));
     keyEvent.type = KeyPress;
-    keyEvent.serial = 1;
-    keyEvent.send_event = True;
-    keyEvent.display = XtDisplay(widget);
-    keyEvent.window = XtWindow(widget);
-    XtCallActionProc(widget, "ArmAndActivate", (XEvent *)&keyEvent, NULL, 0);
+    keyEvent.xkey.serial = 1;
+    keyEvent.xkey.send_event = True;
+
+    if (XtIsSubclass(widget, xmGadgetClass))
+    {
+        /* On some Motif implementations, asking a gadget for its
+           window will crash, rather than return the window of its
+           parent. */
+        Widget parent = XtParent(widget);
+        keyEvent.xkey.display = XtDisplay(parent);
+        keyEvent.xkey.window = XtWindow(parent);
+
+        XtCallActionProc(parent, "ManagerGadgetSelect",
+                         &keyEvent, NULL, 0);
+    }                 
+    else
+    {
+        keyEvent.xkey.display = XtDisplay(widget);
+        keyEvent.xkey.window = XtWindow(widget);
+
+        XtCallActionProc(widget, "ArmAndActivate", &keyEvent, NULL, 0);
+    }
 }
 
 /*
@@ -547,11 +925,11 @@ Widget AddSubMenu(Widget parent, char *name, char *label, char mnemonic)
     Widget menu;
     XmString st1;
     
-    menu = XmCreatePulldownMenu(parent, name, NULL, 0);
+    menu = CreatePulldownMenu(parent, name, NULL, 0);
     XtVaCreateManagedWidget(name, xmCascadeButtonWidgetClass, parent, 
     	XmNlabelString, st1=XmStringCreateSimple(label),
     	XmNmnemonic, mnemonic,
-    	XmNsubMenuId, menu, 0);
+    	XmNsubMenuId, menu, NULL);
     XmStringFree(st1);
     return menu;
 }
@@ -569,7 +947,7 @@ void SetIntLabel(Widget label, int value)
     
     sprintf(labelString, "%d", value);
     s1=XmStringCreateSimple(labelString);
-    XtVaSetValues(label, XmNlabelString, s1, 0);
+    XtVaSetValues(label, XmNlabelString, s1, NULL);
     XmStringFree(s1);
 }
 void SetFloatLabel(Widget label, double value)
@@ -579,7 +957,7 @@ void SetFloatLabel(Widget label, double value)
     
     sprintf(labelString, "%g", value);
     s1=XmStringCreateSimple(labelString);
-    XtVaSetValues(label, XmNlabelString, s1, 0);
+    XtVaSetValues(label, XmNlabelString, s1, NULL);
     XmStringFree(s1);
 }
 void SetIntText(Widget text, int value)
@@ -625,6 +1003,7 @@ int GetFloatText(Widget text, double *value)
     XtFree(strValue);
     return retVal;
 }
+
 int GetIntText(Widget text, int *value)
 {
     char *strValue, *endPtr;
@@ -642,6 +1021,7 @@ int GetIntText(Widget text, int *value)
     XtFree(strValue);
     return retVal;
 }
+
 int GetFloatTextWarn(Widget text, double *value, char *fieldName, int warnBlank)
 {
     int result;
@@ -661,6 +1041,7 @@ int GetFloatTextWarn(Widget text, double *value, char *fieldName, int warnBlank)
     XtFree(valueStr);
     return result;
 }
+
 int GetIntTextWarn(Widget text, int *value, char *fieldName, int warnBlank)
 {
     int result;
@@ -990,13 +1371,93 @@ static void removeWhiteSpace(char *string)
     }
 }
 
+/*
+** Compares two strings and return TRUE if the two strings
+** are the same, ignoring whitespace and case differences.
+*/
+static int stripCaseCmp(const char *str1, const char *str2)
+{
+    const char *c1, *c2;
+    
+    for (c1=str1, c2=str2; *c1!='\0' && *c2!='\0'; c1++, c2++) {
+	while (*c1 == ' ' || *c1 == '\t')
+	    c1++;
+	while (*c2 == ' ' || *c2 == '\t')
+	    c2++;
+    	if (toupper((unsigned char)*c1) != toupper((unsigned char)*c2))
+    	    return FALSE;
+    }
+    return *c1 == '\0' && *c2 == '\0';
+}
+
 static void warnHandlerCB(String message)
 {
     if (strstr(message, "XtRemoveGrab"))
     	return;
     if (strstr(message, "Attempt to remove non-existant passive grab"))
     	return;
-    fprintf(stderr, message);
+    fputs(message, stderr);
+    fputc('\n', stderr);
+}
+
+static XModifierKeymap *getKeyboardMapping(Display *display) {
+    static XModifierKeymap *keyboardMap = NULL;
+
+    if (keyboardMap == NULL) {
+        keyboardMap = XGetModifierMapping(display);
+    }
+    return(keyboardMap);
+}
+
+/*
+** get mask for a modifier
+**
+*/
+
+static Modifiers findModifierMapping(Display *display, KeyCode keyCode) {
+    int i, j;
+    KeyCode *mapentry;
+    XModifierKeymap *modMap = getKeyboardMapping(display);
+
+    if (modMap == NULL || keyCode == 0) {
+        return(0);
+    }
+
+    mapentry = modMap->modifiermap;
+    for (i = 0; i < 8; ++i) {
+        for (j = 0; j < (modMap->max_keypermod); ++j) {
+            if (keyCode == *mapentry) {
+                return(1 << ((mapentry - modMap->modifiermap) / modMap->max_keypermod));
+            }
+            ++mapentry;
+        }
+    }
+    return(0);
+}
+
+static Modifiers getNumLockModMask(Display *display) {
+    static int numLockMask = -1;
+
+    if (numLockMask == -1) {
+        numLockMask = findModifierMapping(display, XKeysymToKeycode(display, XK_Num_Lock));
+    }
+    return(numLockMask);
+}
+
+/*
+** Grab a key regardless of caps-lock and other silly latching keys.
+**
+*/
+
+static void reallyGrabAKey(Widget dialog, int keyCode, Modifiers mask) {
+    Modifiers numLockMask = getNumLockModMask(XtDisplay(dialog));
+
+    XtGrabKey(dialog, keyCode, mask, True, GrabModeAsync, GrabModeAsync);
+    XtGrabKey(dialog, keyCode, mask|LockMask, True, GrabModeAsync, GrabModeAsync);
+    if (numLockMask && numLockMask != LockMask) {
+        XtGrabKey(dialog, keyCode, mask|numLockMask, True, GrabModeAsync, GrabModeAsync);
+        XtGrabKey(dialog, keyCode, mask|LockMask|numLockMask, True, GrabModeAsync, GrabModeAsync);
+    }
 }
 
 /*
@@ -1005,33 +1466,37 @@ static void warnHandlerCB(String message)
 ** dialog widget for the mnemonic character, thus directing mnemonic
 ** events to the dialog widget.
 */
-static void addMnemonicGrabs(Widget dialog, Widget w)
+static void addMnemonicGrabs(Widget dialog, Widget w, int unmodifiedToo)
 {
     char mneString[2];
     WidgetList children;
-    int numChildren, i, isMenu;
+    Cardinal numChildren;
+    int i, isMenu;
     KeySym mnemonic = '\0';
     unsigned char rowColType;
+    unsigned int keyCode;
     
     if (XtIsComposite(w)) {
 	if (XtClass(w) == xmRowColumnWidgetClass) {
-	    XtVaGetValues(w, XmNrowColumnType, &rowColType, 0);
+	    XtVaGetValues(w, XmNrowColumnType, &rowColType, NULL);
 	    isMenu = rowColType != XmWORK_AREA;
 	} else
 	    isMenu = False;
 	if (!isMenu) {
 	    XtVaGetValues(w, XmNchildren, &children, XmNnumChildren,
-		    &numChildren, 0);
+		    &numChildren, NULL);
 	    for (i=0; i<numChildren; i++)
-    		addMnemonicGrabs(dialog, children[i]);
+    		addMnemonicGrabs(dialog, children[i], unmodifiedToo);
     	}
     } else {
-	XtVaGetValues(w, XmNmnemonic, &mnemonic, 0);
-	if (mnemonic != '\0') {
+	XtVaGetValues(w, XmNmnemonic, &mnemonic, NULL);
+	if (mnemonic != XK_VoidSymbol && mnemonic != '\0') {
 	    mneString[0] = mnemonic; mneString[1] = '\0';
-	    XtGrabKey(dialog, XKeysymToKeycode(XtDisplay(dialog),
-	    	    XStringToKeysym(mneString)), Mod1Mask,
-	    	    True, GrabModeAsync, GrabModeAsync);
+	    keyCode = XKeysymToKeycode(XtDisplay(dialog),
+	    	    XStringToKeysym(mneString));
+            reallyGrabAKey(dialog, keyCode, Mod1Mask);
+            if (unmodifiedToo)
+                reallyGrabAKey(dialog, keyCode, 0);
 	}
     }
 }
@@ -1056,7 +1521,8 @@ static void mnemonicCB(Widget w, XtPointer callData, XKeyEvent *event)
 static void findAndActivateMnemonic(Widget w, unsigned int keycode)
 {
     WidgetList children;
-    int numChildren, i, isMenu;
+    Cardinal numChildren;
+    int i, isMenu;
     KeySym mnemonic = '\0';
     char mneString[2];
     Widget userData;
@@ -1064,28 +1530,29 @@ static void findAndActivateMnemonic(Widget w, unsigned int keycode)
     
     if (XtIsComposite(w)) {
 	if (XtClass(w) == xmRowColumnWidgetClass) {
-	    XtVaGetValues(w, XmNrowColumnType, &rowColType, 0);
+	    XtVaGetValues(w, XmNrowColumnType, &rowColType, NULL);
 	    isMenu = rowColType != XmWORK_AREA;
 	} else
 	    isMenu = False;
 	if (!isMenu) {
 	    XtVaGetValues(w, XmNchildren, &children, XmNnumChildren,
-		    &numChildren, 0);
+		    &numChildren, NULL);
 	    for (i=0; i<numChildren; i++)
     		findAndActivateMnemonic(children[i], keycode);
     	}
     } else {
-	XtVaGetValues(w, XmNmnemonic, &mnemonic, 0);
+	XtVaGetValues(w, XmNmnemonic, &mnemonic, NULL);
 	if (mnemonic != '\0') {
 	    mneString[0] = mnemonic; mneString[1] = '\0';
 	    if (XKeysymToKeycode(XtDisplay(XtParent(w)),
 	    	    XStringToKeysym(mneString)) == keycode) {
 	    	if (XtClass(w) == xmLabelWidgetClass ||
 	    		XtClass(w) == xmLabelGadgetClass) {
-	    	    XtVaGetValues(w, XmNuserData, &userData, 0);
-	    	    if (userData!=NULL && XtIsWidget(userData))
+	    	    XtVaGetValues(w, XmNuserData, &userData, NULL);
+	    	    if (userData!=NULL && XtIsWidget(userData) && 
+                        XmIsTraversable(userData))
 	    	    	XmProcessTraversal(userData, XmTRAVERSE_CURRENT);
-	    	} else {
+	    	} else if (XmIsTraversable(w)) {
 	    	    XmProcessTraversal(w, XmTRAVERSE_CURRENT);
 	    	    SimulateButtonPress(w);
 	    	}
@@ -1093,3 +1560,194 @@ static void findAndActivateMnemonic(Widget w, unsigned int keycode)
 	}
     }
 }
+
+/*
+** Part of workaround for Motif Caps/Num Lock bug.  Search the widget tree
+** under w for widgets with accelerators.  When found, add three passive
+** grabs to topWidget, one for the accelerator keysym + modifiers + Caps
+** Lock, one for Num Lock, and one for both, thus directing lock +
+** accelerator events to topWidget.
+*/
+static void addAccelGrabs(Widget topWidget, Widget w)
+{
+    WidgetList children;
+    Widget menu;
+    Cardinal numChildren;
+    int i;
+    
+    if (XtIsComposite(w)) {
+	XtVaGetValues(w, XmNchildren, &children, XmNnumChildren,
+		&numChildren, NULL);
+	for (i=0; i<numChildren; i++)
+    	    addAccelGrabs(topWidget, children[i]);
+    } else if (XtClass(w) == xmCascadeButtonWidgetClass) {
+	XtVaGetValues(w, XmNsubMenuId, &menu, NULL);
+	if (menu != NULL)
+	    addAccelGrabs(topWidget, menu);
+    } else
+	addAccelGrab(topWidget, w);
+}
+
+/*
+** Grabs the key + modifier defined in the widget's accelerator resource,
+** in combination with the Caps Lock and Num Lock accelerators.
+*/
+static void addAccelGrab(Widget topWidget, Widget w)
+{
+    char *accelString = NULL;
+    KeySym keysym;
+    unsigned int modifiers;
+    Modifiers numLockMask = getNumLockModMask(XtDisplay(topWidget));
+    
+    XtVaGetValues(w, XmNaccelerator, &accelString, NULL);
+    if (accelString == NULL || *accelString == '\0')
+	return;
+    
+    if (!parseAccelString(XtDisplay(topWidget), accelString, &keysym, &modifiers))
+	return;
+    XtGrabKey(topWidget, XKeysymToKeycode(XtDisplay(topWidget), keysym),
+	    modifiers | LockMask, True, GrabModeAsync, GrabModeAsync);
+    if (numLockMask && numLockMask != LockMask) {
+        XtGrabKey(topWidget, XKeysymToKeycode(XtDisplay(topWidget), keysym),
+	        modifiers | numLockMask, True, GrabModeAsync, GrabModeAsync);
+        XtGrabKey(topWidget, XKeysymToKeycode(XtDisplay(topWidget), keysym),
+	        modifiers | LockMask | numLockMask, True, GrabModeAsync, GrabModeAsync);
+    }
+}
+
+/*
+** Read a Motif accelerator string and translate it into a keysym + modifiers.
+** Returns TRUE if the parse was successful, FALSE, if not.
+*/
+static int parseAccelString(Display *display, const char *string, KeySym *keySym,
+	unsigned int *modifiers)
+{
+#define N_MODIFIERS 12
+    /*... Is NumLock always Mod3? */
+    static char *modifierNames[N_MODIFIERS] = {"Ctrl", "Shift", "Alt", "Mod2",
+	    "Mod3", "Mod4", "Mod5", "Button1", "Button2", "Button3", "Button4",
+	    "Button5"};
+    static unsigned int modifierMasks[N_MODIFIERS] = {ControlMask, ShiftMask,
+	    Mod1Mask, Mod2Mask, Mod3Mask, Mod4Mask, Mod5Mask, Button1Mask, Button2Mask,
+	    Button3Mask, Button4Mask, Button5Mask};
+    Modifiers numLockMask = getNumLockModMask(display);
+    char modStr[MAX_ACCEL_LEN];
+    char evtStr[MAX_ACCEL_LEN];
+    char keyStr[MAX_ACCEL_LEN];
+    const char *c, *evtStart, *keyStart;
+    int i;
+    
+    if (strlen(string) >= MAX_ACCEL_LEN)
+	return FALSE;
+    
+    /* Get the modifier part */
+    for (c = string; *c != '<'; c++)
+	if (*c == '\0')
+	    return FALSE;
+    strncpy(modStr, string, c - string);
+    modStr[c - string] = '\0';
+    
+    /* Verify the <key> or <keypress> part */
+    evtStart = c;
+    for ( ; *c != '>'; c++)
+	if (*c == '\0')
+	    return FALSE;
+    c++;
+    strncpy(evtStr, evtStart, c - evtStart);
+    evtStr[c - evtStart] = '\0';
+    if (!stripCaseCmp(evtStr, "<key>") && !stripCaseCmp(evtStr, "<keypress>"))
+	return FALSE;
+    
+    /* Get the keysym part */
+    keyStart = c;
+    for ( ; *c != '\0'; c++);
+    strncpy(keyStr, keyStart, c - keyStart);
+    keyStr[c - keyStart] = '\0';
+    *keySym = XStringToKeysym(keyStr);
+    
+    /* Parse the modifier part */
+    *modifiers = 0;
+    c = modStr;
+    while (*c != '\0') {
+	while (*c == ' ' || *c == '\t')
+	    c++;
+	if (*c == '\0')
+	    break;
+	for (i = 0; i < N_MODIFIERS; i++) {
+	    if (!strncmp(c, modifierNames[i], strlen(modifierNames[i]))) {
+	    	c += strlen(modifierNames[i]);
+                if (modifierMasks[i] != numLockMask) {
+		    *modifiers |= modifierMasks[i];
+                }
+		break;
+	    }
+	}
+	if (i == N_MODIFIERS)
+	    return FALSE;
+    }
+    
+    return TRUE;
+}
+
+/*
+** Event handler for patching around Motif's lock + accelerator problem.
+** Looks for a menu item in the patched menu hierarchy and invokes its
+** ArmAndActivate action.
+*/
+static void lockCB(Widget w, XtPointer callData, XEvent *event,
+	Boolean *continueDispatch)
+{
+    Modifiers numLockMask = getNumLockModMask(XtDisplay(w));
+    Widget topMenuWidget = (Widget)callData;
+    *continueDispatch = TRUE;
+    if (!(((XKeyEvent *)event)->state & (LockMask | numLockMask)))
+	return;
+
+    if (!findAndActivateAccel(topMenuWidget, ((XKeyEvent *)event)->keycode,
+	    ((XKeyEvent *)event)->state & ~(LockMask | numLockMask), event))
+	*continueDispatch = FALSE;
+}
+
+/*
+** Search through menu hierarchy under w and look for a button with
+** accelerator matching keyCode + modifiers, and do its action
+*/
+static int findAndActivateAccel(Widget w, unsigned int keyCode,
+	unsigned int modifiers, XEvent *event)
+{
+
+    WidgetList children;
+    Widget menu;
+    Cardinal numChildren;
+    int i;
+    char *accelString = NULL;
+    KeySym keysym;
+    unsigned int mods;
+    
+    if (XtIsComposite(w)) {
+	XtVaGetValues(w, XmNchildren, &children, XmNnumChildren,
+		&numChildren, NULL);
+	for (i=0; i<numChildren; i++)
+    	    if (findAndActivateAccel(children[i], keyCode, modifiers, event))
+		return TRUE;
+    } else if (XtClass(w) == xmCascadeButtonWidgetClass) {
+	XtVaGetValues(w, XmNsubMenuId, &menu, NULL);
+	if (menu != NULL)
+	    if (findAndActivateAccel(menu, keyCode, modifiers, event))
+		return TRUE;
+    } else {
+	XtVaGetValues(w, XmNaccelerator, &accelString, NULL);
+	if (accelString != NULL && *accelString != '\0') {
+	    if (!parseAccelString(XtDisplay(w), accelString, &keysym, &mods))
+		return FALSE;
+	    if (keyCode == XKeysymToKeycode(XtDisplay(w), keysym) &&
+		    modifiers == mods) {
+		if (XtIsSensitive(w)) {
+		    XtCallActionProc(w, "ArmAndActivate", event, NULL, 0);
+		    return TRUE;
+		}
+	    }
+	}
+    }
+    return FALSE;
+}
