@@ -1,20 +1,25 @@
+static const char CVSID[] = "$Id: printUtils.c,v 1.11 2001/08/14 08:37:16 jlous Exp $";
 /*******************************************************************************
 *									       *
-* printUtils.c -- Fermilab Nirvana library Printer Menu	& Printing Routines    *
+* printUtils.c -- Nirvana library Printer Menu	& Printing Routines   	       *
 *									       *
-* Copyright (c) 1991 Universities Research Association, Inc.		       *
-* All rights reserved.							       *
+* Copyright (C) 1999 Mark Edel						       *
+*									       *
+* This is free software; you can redistribute it and/or modify it under the    *
+* terms of the GNU General Public License as published by the Free Software    *
+* Foundation; either version 2 of the License, or (at your option) any later   *
+* version.							               *
 * 									       *
-* This material resulted from work developed under a Government Contract and   *
-* is subject to the following license:  The Government retains a paid-up,      *
-* nonexclusive, irrevocable worldwide license to reproduce, prepare derivative *
-* works, perform publicly and display publicly by or for the Government,       *
-* including the right to distribute to other Government contractors.  Neither  *
-* the United States nor the United States Department of Energy, nor any of     *
-* their employees, makes any warranty, express or implied, or assumes any      *
-* legal liability or responsibility for the accuracy, completeness, or         *
-* usefulness of any information, apparatus, product, or process disclosed, or  *
-* represents that its use would not infringe privately owned rights.           *
+* This software is distributed in the hope that it will be useful, but WITHOUT *
+* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
+* for more details.							       *
+* 									       *
+* You should have received a copy of the GNU General Public License along with *
+* software; if not, write to the Free Software Foundation, Inc., 59 Temple     *
+* Place, Suite 330, Boston, MA  02111-1307 USA		                       *
+*									       *
+* Nirvana Text Editor	    						       *
 *                                        				       *
 * April 20, 1992							       *
 *									       *
@@ -26,18 +31,16 @@
 * Modified by Donna Reid and Joy Kyriakopulos 4/8/93 - VMS port		       *
 *									       *
 *******************************************************************************/
-static char SCCSID[] = "@(#)printUtils.c	1.20	1/10/96";
-
 #include <string.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
+#include <ctype.h>
 #ifdef VMS
 #include "vmsparam.h"
 #include <ssdef.h>
 #include <lib$routines.h>
-#include <ctype.h>
 #include <descrip.h>
 #include <starlet.h>
 #include <lnmdef.h>
@@ -48,7 +51,9 @@ static char SCCSID[] = "@(#)printUtils.c	1.20	1/10/96";
 #else
 #include <sys/dir.h>
 #endif /* USE_DIRENT */
+#ifndef __MVS__
 #include <sys/param.h>
+#endif
 #endif /*VMS*/
 #include <sys/stat.h>
 #include <X11/StringDefs.h>
@@ -66,7 +71,11 @@ static char SCCSID[] = "@(#)printUtils.c	1.20	1/10/96";
 #include "printUtils.h"
 
 /* Separator between directory references in PATH environmental variable */
+#ifdef __EMX__  /* For OS/2 */
+#define SEPARATOR ';'
+#else
 #define SEPARATOR ':'  
+#endif
 
 /* Number of extra pixels down to place a label even with a text widget */
 #define LABEL_TEXT_DIFF 6
@@ -102,13 +111,15 @@ static void updatePrintCmd(Widget w, caddr_t client_data, caddr_t call_data);
 static void printCmdModified(Widget w, caddr_t client_data, caddr_t call_data);
 static void printButtonCB(Widget widget, caddr_t client_data, caddr_t call_data);
 static void cancelButtonCB(Widget widget, caddr_t client_data, caddr_t call_data);
-static void setQueueLabelText();
-static int fileInDir(char *filename,char *dirpath,unsigned short mode_flags);
-static int fileInPath(char *filename,unsigned short mode_flags);
-static int flprPresent();
+static void setQueueLabelText(void);
+static int fileInDir(const char *filename, const char *dirpath, unsigned short mode_flags);
+static int fileInPath(const char *filename, unsigned short mode_flags);
+static int flprPresent(void);
 static void getLprQueueDefault(char *defqueue);
+#ifndef USE_LPR_PRINT_CMD
 static void getLpQueueDefault(char *defqueue);
-static void setHostLabelText();
+#endif
+static void setHostLabelText(void);
 #ifdef VMS
 static void getVmsQueueDefault(char *defqueue);
 #else
@@ -126,8 +137,8 @@ static Widget	Text1;
 static Widget	Text2;
 static Widget	Text3;
 static Widget	Text4;
-static char *PrintFileName;
-static char *PrintJobName;
+static const char *PrintFileName;
+static const char *PrintJobName;
 static char PrintCommand[MAX_PCMD_STR];	/* print command string */
 static char CopiesOption[MAX_OPT_STR];	/* # of copies argument string */
 static char QueueOption[MAX_OPT_STR];	/* queue name argument string */
@@ -174,9 +185,9 @@ static struct printPrefDescrip PrintPrefDescrip = {
 **	jobName		Title for the print banner page
 */
 #ifdef VMS
-void PrintFile(Widget parent, char *printFile, char *jobName, int delete)
+void PrintFile(Widget parent, const char *printFile, const char *jobName, int delete)
 #else
-void PrintFile(Widget parent, char *printFile, char *jobName)
+void PrintFile(Widget parent, const char *printFile, const char *jobName)
 #endif /*VMS*/
 {
     /* In case the program hasn't called LoadPrintPreferences, set up the
@@ -300,8 +311,8 @@ static Widget createForm(Widget parent)
     XtSetArg(args[argcnt], XmNdialogTitle, (st0=XmStringCreateLtoR(
 		"Printer Options", XmSTRING_DEFAULT_CHARSET))); argcnt++;
     XtSetArg(args[argcnt], XmNautoUnmanage, False); argcnt++;
-    form = XmCreateFormDialog(parent, "printForm", args, argcnt);
-    XtVaSetValues(form, XmNshadowThickness, 0, 0);
+    form = CreateFormDialog(parent, "printForm", args, argcnt);
+    XtVaSetValues(form, XmNshadowThickness, 0, NULL);
     
     XmStringFree( st0 );
 
@@ -337,7 +348,7 @@ static Widget createForm(Widget parent)
 		(XtCallbackProc)updatePrintCmd, NULL);
 	RemapDeleteKey(Text1);
 	topWidget = Text1;
-	XtVaSetValues(label1, XmNuserData, Text1, 0); /* mnemonic procesing */
+	XtVaSetValues(label1, XmNuserData, Text1, NULL); /* mnemonic procesing */
     }
     
     /************************ LABEL 2 and TEXT 2 ************************/
@@ -378,7 +389,7 @@ static Widget createForm(Widget parent)
 		(XtCallbackProc)updatePrintCmd, NULL);
 	bwidgetarray[bwidgetcnt] = Text2; bwidgetcnt++;
 	RemapDeleteKey(Text2);
-	XtVaSetValues(Label2, XmNuserData, Text2, 0); /* mnemonic procesing */
+	XtVaSetValues(Label2, XmNuserData, Text2, NULL); /* mnemonic procesing */
 	topWidget = Text2;
     }
     
@@ -421,7 +432,7 @@ static Widget createForm(Widget parent)
 		(XtCallbackProc)updatePrintCmd, NULL);
 	bwidgetarray[bwidgetcnt] = Text3; bwidgetcnt++; 
     	RemapDeleteKey(Text3);
-	XtVaSetValues(Label3, XmNuserData, Text3, 0); /* mnemonic procesing */
+	XtVaSetValues(Label3, XmNuserData, Text3, NULL); /* mnemonic procesing */
     	topWidget = Text3;
     }
 
@@ -493,11 +504,11 @@ static Widget createForm(Widget parent)
     XtSetValues(form, args, argcnt);
 
     XtManageChildren(bwidgetarray, bwidgetcnt);
-    AddDialogMnemonicHandler(form);
+    AddDialogMnemonicHandler(form, FALSE);
     return form;
 }
 
-static void setQueueLabelText()
+static void setQueueLabelText(void)
 {
     Arg args[15];
     int	argcnt;
@@ -515,7 +526,7 @@ static void setQueueLabelText()
     XmStringFree( st0 );
 }
 
-static void setHostLabelText()
+static void setHostLabelText(void)
 {
     Arg args[15];
     int	argcnt;
@@ -691,7 +702,7 @@ static void printButtonCB(Widget widget, caddr_t client_data, caddr_t call_data)
     if (spawn_sts != SS$_NORMAL) {
        	DialogF(DF_WARN, widget, 1,
        		"Unable to Print:\n%d - %s\n  spawnFlags = %d\n",
-       		"Acknowledged", spawn_sts, strerror(EVMSERR, spawn_sts),
+       		"Dismiss", spawn_sts, strerror(EVMSERR, spawn_sts),
        		spawnFlags);
         return;
     }
@@ -712,7 +723,7 @@ static void printButtonCB(Widget widget, caddr_t client_data, caddr_t call_data)
     pipe = (FILE *)popen(command,"r");
     if (pipe == NULL) {
 	DialogF(DF_WARN, widget, 1, "Unable to Print:\n%s",
-       		"Acknowledged", strerror(errno));
+       		"Dismiss", strerror(errno));
 	return;
     }
     errorString[0] = 0;
@@ -721,7 +732,7 @@ static void printButtonCB(Widget widget, caddr_t client_data, caddr_t call_data)
 	errorString[nRead] = '\0';
     if (pclose(pipe)) {
 	DialogF(DF_WARN, widget, 1, "Unable to Print:\n%s",
-       		"Acknowledged", errorString);
+       		"Dismiss", errorString);
 	return;
     }
 #endif /*(VMS)*/
@@ -754,6 +765,7 @@ static void printButtonCB(Widget widget, caddr_t client_data, caddr_t call_data)
 static void cancelButtonCB(Widget widget, caddr_t client_data, caddr_t call_data)
 {
     DoneWithDialog = True;
+    CmdFieldModified = False;
 }
 
 #ifndef VMS
@@ -761,7 +773,7 @@ static void cancelButtonCB(Widget widget, caddr_t client_data, caddr_t call_data
 ** Is the filename file in the directory dirpath
 ** and does it have at least some of the mode_flags enabled ?
 */
-static int fileInDir(char *filename, char *dirpath, unsigned short mode_flags)
+static int fileInDir(const char *filename, const char *dirpath, unsigned short mode_flags)
 {
     DIR           *dfile;
 #ifdef USE_DIRENT
@@ -793,7 +805,7 @@ static int fileInDir(char *filename, char *dirpath, unsigned short mode_flags)
 ** Is the filename file in the environment path directories 
 ** and does it have at least some of the mode_flags enabled ?
 */
-static int fileInPath(char *filename,unsigned short mode_flags)
+static int fileInPath(const char *filename, unsigned short mode_flags)
 {
     char path[MAXPATHLEN];
     char *pathstring,*lastchar;
@@ -830,32 +842,35 @@ static int fileInPath(char *filename,unsigned short mode_flags)
 /*
 ** Is flpr present in the search path and is it executable ?
 */
-static int flprPresent()
+static int flprPresent(void)
 {
     /* Is flpr present in the search path and is it executable ? */
     return fileInPath("flpr",0111);
 }
 
-static int foundTag(char *tagfilename,char *tagname,char *result)
+static int foundTag(const char *tagfilename, const char *tagname, char *result)
 {
-   FILE *tfile;
-   char tagformat[512],line[512];
+    FILE *tfile;
+    char tagformat[512],line[512];
 
-   strcpy(tagformat, tagname);
-   strcat(tagformat, " %s");
+    strcpy(tagformat, tagname);
+    strcat(tagformat, " %s");
 
-   tfile = fopen(tagfilename,"r");
-   if (tfile != NULL) {
-       while (!feof(tfile)) {
-           fgets(line,sizeof(line),tfile);
-           if (sscanf(line,tagformat,result) != 0)
-               return True;
-       }
-   }
-   return False;
+    tfile = fopen(tagfilename,"r");
+    if (tfile != NULL) {
+	while (!feof(tfile)) {
+            fgets(line,sizeof(line),tfile);
+            if (sscanf(line,tagformat,result) != 0) {
+		fclose(tfile);
+        	return True;
+	    }
+	}
+	fclose(tfile);
+    }
+    return False;
 }
 
-static int foundEnv(char *EnvVarName,char *result)
+static int foundEnv(const char *EnvVarName, char *result)
 {
     char *dqstr;
 
@@ -887,12 +902,14 @@ static void getLprQueueDefault(char *defqueue)
         strcpy(defqueue,"");
 }
 
+#ifndef USE_LPR_PRINT_CMD
 static void getLpQueueDefault(char *defqueue)
 {
     if (!foundEnv("LPDEST",defqueue))
         defqueue[0] = '\0';
 }
 #endif
+#endif
 
 #ifdef VMS
 static void getVmsQueueDefault(char *defqueue)
@@ -932,7 +949,7 @@ static void getVmsQueueDefault(char *defqueue)
     translate_sts = sys$trnlnm(0,&tabName,&logName,0,&translStruct);
     if (translate_sts != SS$_NORMAL && translate_sts != SS$_NOLOGNAM){
        fprintf(stderr,"Error return from sys$trnlnm: %d\n", translate_sts);
-       DialogF(DF_WARN,Label2,1,"Error translating SYS$PRINT","Acknowledged");
+       DialogF(DF_WARN, Label2, 1, "Error translating SYS$PRINT", "Dismiss");
        defqueue[0] = '\0';
     }
     else
